*==== BOF chcnstrt =====================================================


*   Module data:

*  'network.inc'
*     MaxChannels - maximum number of channels.
*     NumCh - current number of channels.
*     Branch - current selected or active channel.
*     MaxLocations - maximum number of computational or user locations.


*  'chcnstrt.inc'
*     Reverse(i) - .True. if a companion 3-parameter table exists for
*                  negative flow, false otherwise.  Refers to upstream
*                  end of channel if i is odd, downstream if i is even.
*                  Channel number is INT( (i + 1) / 2 ).

*     ParmRating3 - .TRUE. if any 3-parameter ratings exist, .FALSE.
*                   otherwise.
*    

*     UserConstraint - .TRUE. if any user-programmed constraints
*                      exist, false otherwise.

*== Public (InitStreamConstraint) ================================================

      LOGICAL FUNCTION InitStreamConstraint()

      IMPLICIT NONE

*   Purpose:  Initialize channel constraint module, must not be
*             called before schematic channel data has been initialized.

*   Note:  This routine checks for and initializes 3-parameter
*          constraints if they exist.  This routine checks for
*          user-programmed constraints and calls the initialization
*          routine if they exist.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcnstrt.inc'

*   Local Variables:
      INTEGER Channel, I, Code, Type
      LOGICAL OK

*   Routines by module:

***** Local:

      LOGICAL  InitUserStreamConstraint
      EXTERNAL InitUserStreamConstraint

***** Channel schematic:

      INTEGER  NumberOfChannels
      EXTERNAL NumberOfChannels
      
      INTEGER  DownstreamCode
      EXTERNAL DownstreamCode

      INTEGER  UpstreamCode
      EXTERNAL UpstreamCode
      
      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

***** 3-parameter-rating table:
      
      LOGICAL  GETAB3, INITABLE3
      EXTERNAL GETAB3, INITABLE3

*   Intrinsics:
      INTEGER   IABS, MOD
      INTRINSIC IABS, MOD

*   Programmed by: Lew DeLong
*   Date:          December 1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      InitStreamConstraint = .FALSE.      


*---- Check boundary conditions for any 3-parameter ratings
*     and user-programmed constraints.

      ParmRating3 = .FALSE.
      UserConstraint = .FALSE.
      
      DO 100 I = 1,NumberOfChannels()
        Channel = I

        IF( OpenChannel( Channel ) ) THEN

          Code = IABS( MOD( UpstreamCode(), 100 ) )

          IF( .NOT. ParmRating3 ) THEN
            IF( Code .EQ. 31
     #               .OR.
     #          Code .EQ. 32 ) THEN
                ParmRating3 = .TRUE.
            END IF
          END IF
 
          IF( .NOT. UserConstraint ) THEN
            IF( Code .EQ. 51
     #               .OR.
     #          Code .EQ. 52 ) THEN
                UserConstraint = .TRUE.
            END IF
          END IF

          Code = IABS( MOD( DownstreamCode(), 100 ) )

          IF( .NOT. ParmRating3 ) THEN
            IF( Code .EQ. 31
     #               .OR.
     #          Code .EQ. 32 ) THEN
                ParmRating3 = .TRUE.
            END IF
          END IF
 
          IF( .NOT. UserConstraint ) THEN
            IF( Code .EQ. 51
     #               .OR.
     #          Code .EQ. 52 ) THEN
                UserConstraint = .TRUE.
            END IF
          END IF

          OK = CloseChannel()

        ELSE
          WRITE(*,*) ' ####Error(InitStreamConstraint)'
          WRITE(*,*) ' Could not open channel...',Channel
          RETURN
        END IF

 100  CONTINUE
      

      IF( ParmRating3 ) THEN

*------ Initialize 3-parameter rating module.

        IF( INITABLE3() ) THEN

*-------- Check for existence of tables.

          DO 200 I = 1,NumberOfChannels()
            Channel = I
            OK = OpenChannel( Channel )
            

*---------- Upstream end of channel.

            Code = UpstreamCode()
            Type = MOD( Code, 100 ) - MOD( Code, 10 )


            IF( Type .EQ. 30 ) THEN

*------------ Code indicates 3-parameter type rating.

              IF( GETAB3( Code ) ) THEN

*--------------- Standard table for positive flow exists.

                IF( GETAB3( -Code ) ) THEN
*---------------- Companion table for reversed flow exists.
                  Reverse( 2 * I -1 ) = .TRUE.
                ELSE
*---------------- Companion table for reversed flow does not exist.
                  Reverse( 2 * I - 1 ) = .FALSE.
                END IF

              ELSE
                WRITE(*,*) ' #### Error(InitStreamConstraint)'
                WRITE(*,*) ' Failed to find 3-parameter table...',Code
                WRITE(*,*) ' Abnormal program end.'
                STOP
              END IF
            END IF

            Code = DownstreamCode()
            Type = MOD( Code, 100 ) - MOD( Code, 10 )


            IF( Type .EQ. 30 ) THEN

*------------ Code indicates 3-parameter type rating.

              IF( GETAB3( Code ) ) THEN

*--------------- Standard table for positive flow exists.

                IF( GETAB3( -Code ) ) THEN
*---------------- Companion table for reversed flow exists.
                  Reverse( 2 * I ) = .TRUE.
                ELSE
*---------------- Companion table for reversed flow does not exist.
                  Reverse( 2 * I ) = .FALSE.
                END IF

              ELSE
                WRITE(*,*) ' #### Error(InitStreamConstraint)'
                WRITE(*,*) ' Failed to find 3-parameter table...',Code
                WRITE(*,*) ' Abnormal program end.'
                STOP
              END IF
            END IF

            OK = CloseChannel()

 200  CONTINUE


        ELSE
          WRITE(*,*) ' #### Error(InitStreamConstraint)'
          WRITE(*,*) ' Initialization of 3-parameter table failed...'
          WRITE(*,*) ' Abnormal program end.'
          STOP
        END IF

      END IF
      
      IF( UserConstraint ) THEN
        IF( InitUserStreamConstraint() ) THEN
        ELSE
          RETURN
        END IF
      END IF
      
      InitStreamConstraint = .TRUE.

      RETURN
      END




*== Public (ForceStreamSurface) ========================================

      LOGICAL FUNCTION ForceStreamSurface( Extremity )

      IMPLICIT NONE

*   Purpose:  Compute and store constraint-equation coefficients
*             forcing a known stream surface elevation.

*   Arguments:
      INTEGER Extremity

*   Argument definitions:
*     Extremity - index indicating
*                   [+1] upstream end of current channel.
*                   [-1] downstream end of current channel.

*   Module data:

*   Local Variables:
      INTEGER Zero, Row, ConstraintNode
      REAL    One, DesiredElevation
      PARAMETER ( Zero = 0, One = 1.0 )
      LOGICAL OK

*   Routines by module:

***** Channel flow status:
      REAL     StreamSurfaceElevation
      EXTERNAL StreamSurfaceElevation

***** Channel schematic:
      INTEGER  NumberOfStreamLocations
      EXTERNAL NumberOfStreamLocations

***** Network control:
      REAL     UpstreamBoundaryValue, DownstreamBoundaryValue
      EXTERNAL UpstreamBoundaryValue, DownstreamBoundaryValue

***** Solver:
      INTEGER  UpstreamConstraintRow, DownstreamConstraintRow
      EXTERNAL UpstreamConstraintRow, DownstreamConstraintRow

      LOGICAL  StoreAtRowOffset, StoreAtRow
      EXTERNAL StoreAtRowOffset, StoreAtRow

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( Extremity .EQ. +1 ) THEN

*------ Upstream constraint.
        ConstraintNode = 1
        Row = UpstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, One )
        DesiredElevation = UpstreamBoundaryValue()

      ELSE IF( Extremity .EQ. -1 ) THEN

*------ Downstream constraint.
        ConstraintNode = NumberOfStreamLocations()
        Row = DownstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, One )
        DesiredElevation = DownstreamBoundaryValue()

      END IF

      OK = StoreAtRow(
     #                 Row,
     #   DesiredElevation - StreamSurfaceElevation( ConstraintNode )
     #               )

      ForceStreamSurface = .TRUE.

      RETURN
      END



*== Public (ForceStreamFlow) ===========================================

      LOGICAL FUNCTION ForceStreamFlow( Extremity )

      IMPLICIT NONE

*   Purpose:  Compute and store constraint-equation coefficients
*             forcing a known streamflow.

*   Arguments:
      INTEGER Extremity

*   Argument definitions:
*     Extremity - index indicating
*                   [+1] upstream end of current channel.
*                   [-1] downstream end of current channel.

*   Module data:

*   Local Variables:
      INTEGER Zero, Row, ConstraintNode
      REAL    One, DesiredFlow
      PARAMETER ( Zero = 0, One = 1.0 )
      LOGICAL OK

*   Routines by module:

***** Channel flow status:
      REAL     StreamFlow
      EXTERNAL StreamFlow

***** Channel schematic:
      INTEGER  NumberOfStreamLocations
      EXTERNAL NumberOfStreamLocations

***** Network control:
      REAL     UpstreamBoundaryValue, DownstreamBoundaryValue
      EXTERNAL UpstreamBoundaryValue, DownstreamBoundaryValue

***** Solver:
      INTEGER  UpstreamConstraintRow, DownstreamConstraintRow
      EXTERNAL UpstreamConstraintRow, DownstreamConstraintRow

      LOGICAL  StoreAtRowOffset, StoreAtRow
      EXTERNAL StoreAtRowOffset, StoreAtRow

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      IF( Extremity .EQ. +1 ) THEN

*------ Upstream constraint.
        ConstraintNode = 1
        Row = UpstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, One )
        DesiredFlow = UpstreamBoundaryValue()

      ELSE IF( Extremity .EQ. -1 ) THEN

*------ Downstream constraint.
        ConstraintNode = NumberOfStreamLocations()
        Row = DownstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, One )
        DesiredFlow = DownstreamBoundaryValue()

      END IF

      OK = StoreAtRow(
     #         Row, DesiredFlow - StreamFlow( ConstraintNode )
     #               )

      ForceStreamFlow = .TRUE.

      RETURN
      END



*== Public (ForceSumStreamFlow) ========================================

      LOGICAL FUNCTION ForceSumStreamFlow( Extremity )

      IMPLICIT NONE

*   Purpose:  Compute and store constraint-equation coefficients
*             forcing the sum of stream flows at a location to zero.

*   Arguments:
      INTEGER Extremity

*   Argument definitions:
*     Extremity - index indicating
*                   [+1] upstream end of current channel.
*                   [-1] downstream end of current channel.

*   Module data:
      INCLUDE 'network.inc'

*   Local Variables:
      INTEGER MaxConnections
      PARAMETER ( MaxConnections = MaxConnectingChannels )
      INTEGER I, ConnectingNodes, Node(MaxConnections)
      INTEGER ConstraintNode, DF
      INTEGER Zero, Row, AbsNode, Sign
      REAL    One, Residual
      PARAMETER ( Zero = 0, One = 1.0, DF = 2 )
      LOGICAL OK

*   Routines by module:

***** Channel flow status:
      REAL     GlobalStreamFlow
      EXTERNAL GlobalStreamFlow

***** Channel schematic:
      INTEGER  CurrentChannel
      EXTERNAL CurrentChannel
      
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

      INTEGER  UpstreamConnect, UpstreamConnections
      EXTERNAL UpstreamConnect, UpstreamConnections

      INTEGER  DownstreamConnect, DownstreamConnections
      EXTERNAL DownstreamConnect, DownstreamConnections

      INTEGER  StreamEndNode
      EXTERNAL StreamEndNode

***** Network control:
      REAL     UpstreamBoundaryValue, DownstreamBoundaryValue
      EXTERNAL UpstreamBoundaryValue, DownstreamBoundaryValue

***** Solver:
      INTEGER  UpstreamConstraintRow, DownstreamConstraintRow
      EXTERNAL UpstreamConstraintRow, DownstreamConstraintRow

      LOGICAL  StoreAtRowOffset, StoreAtRow, AddAtRow
      EXTERNAL StoreAtRowOffset, StoreAtRow, AddAtRow

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: August   1992
*

*   Intrinsics:
      INTEGER   IABS
      REAL      FLOAT
      INTRINSIC IABS, FLOAT


*-- Implementation -----------------------------------------------------

      ForceSumStreamFlow = .FALSE.

*---- Note, by convention, any flow into a junction is positive.
*     Constraint equation is written from the perspective of
*     the junction.

      IF( Extremity .EQ. +1 ) THEN

*------ Upstream constraint.

        ConstraintNode = UpstreamPointer()
        Row = UpstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, -One )
        OK = StoreAtRow( Row, - UpstreamBoundaryValue() )
        ConnectingNodes = UpstreamConnections()
        IF( ConnectingNodes .LE. MaxConnections ) THEN
        ELSE
          WRITE(*,*) ' ####error(ForceSumStreamFlow)'
          WRITE(*,*) ' Upstream end Channel...', CurrentChannel()
          WRITE(*,*) ' Number of connections (',ConnectingNodes,')'
          WRITE(*,*) ' exceeds maximum (',MaxConnections,').'
          WRITE(*,*) ' Abnormal program end.'
          STOP
        END IF
        IF( ConnectingNodes .GT. 0 ) THEN
          DO 100 I=1,ConnectingNodes
            Node(I) = StreamEndNode( UpstreamConnect(I) ) 
 100      CONTINUE
        ELSE
        END IF

      ELSE IF( Extremity .EQ. -1 ) THEN

*------ Downstream constraint.

        ConstraintNode = DownstreamPointer()
        Row = DownstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, One )
        OK = StoreAtRow( Row, - DownstreamBoundaryValue() )
        ConnectingNodes = DownstreamConnections()
        IF( ConnectingNodes .LE. MaxConnections ) THEN
        ELSE
          WRITE(*,*) ' ####error(ForceSumStreamFlow)'
          WRITE(*,*) ' Downstream end Channel...', CurrentChannel()
          WRITE(*,*) ' Number of connections (',ConnectingNodes,')'
          WRITE(*,*) ' exceeds maximum (',MaxConnections,').'
          WRITE(*,*) ' Abnormal program end.'
          STOP
        END IF
        IF( ConnectingNodes .GT. 0 ) THEN
          DO 200 I=1,ConnectingNodes
            Node(I) = StreamEndNode( DownstreamConnect(I) )
 200      CONTINUE
        ELSE
        END IF

      ELSE
        WRITE(*,*) ' *** Error (ForceSumStreamFlow)'
        WRITE(*,*) ' Extremity not equal +1 or -1 ...'
        RETURN
      END IF

      Residual = 0.0

      IF( ConnectingNodes .GT. 0 ) THEN

        DO 300 I=1,ConnectingNodes

          AbsNode = IABS(Node(I))
          Sign =   AbsNode / Node(I)

          OK = StoreAtRowOffset(
     #            Row, DF*(AbsNode-ConstraintNode), - FLOAT(Sign)
     #                         )

          Residual = Residual
     #               + FLOAT(Sign) * GlobalStreamFlow( AbsNode )
     

 300    CONTINUE

      END IF

      IF( Extremity .EQ. +1 ) THEN
        OK = AddAtRow(
     #      Row, + GlobalStreamFlow( ConstraintNode ) + Residual
     #               )
       ELSE
        OK = AddAtRow(
     #      Row, - GlobalStreamFlow( ConstraintNode ) + Residual
     #               )
       END IF

       ForceSumStreamFlow = .TRUE.

      RETURN
      END



*== Public (ForceEqualStreamSurface) ===================================

      LOGICAL FUNCTION ForceEqualStreamSurface( Extremity )

      IMPLICIT NONE

*   Purpose:  Compute and store constraint-equation coefficients forcing
*             equal stream-surface elevations at two cross sections.

*   Arguments:
      INTEGER Extremity

*   Argument definitions:
*     Extremity - index indicating
*                   [+1] upstream end of current channel.
*                   [-1] downstream end of current channel.

*   Module data:

*   Local Variables:
      INTEGER First
      PARAMETER (First = 1)
      INTEGER ConnectingNodes
      INTEGER ConstraintNode, DF
      INTEGER Zero, Row, Node
      INTEGER ConnectingChannel
      REAL    One, Fall
      PARAMETER ( Zero = 0, One = 1.0, DF = 2 )
      LOGICAL OK

*   Routines by module:

***** Channel flow status:
      REAL     GlobalStreamSurfaceElevation
      EXTERNAL GlobalStreamSurfaceElevation

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

      INTEGER  UpstreamConnect, UpstreamConnections
      EXTERNAL UpstreamConnect, UpstreamConnections

      INTEGER  DownstreamConnect, DownstreamConnections
      EXTERNAL DownstreamConnect, DownstreamConnections

      INTEGER  StreamEndNode
      EXTERNAL StreamEndNode

***** Network control:
      REAL     UpstreamBoundaryValue, DownstreamBoundaryValue
      EXTERNAL UpstreamBoundaryValue, DownstreamBoundaryValue

***** Solver:
      INTEGER  UpstreamConstraintRow, DownstreamConstraintRow
      EXTERNAL UpstreamConstraintRow, DownstreamConstraintRow

      LOGICAL  StoreAtRowOffset, StoreAtRow
      EXTERNAL StoreAtRowOffset, StoreAtRow

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: August   1992
*

*   Intrinsics:
      INTEGER   IABS
      INTRINSIC IABS


*-- Implementation -----------------------------------------------------

      ForceEqualStreamSurface = .FALSE.

      IF( Extremity .EQ. +1 ) THEN

*------ Upstream constraint.

        ConstraintNode = UpstreamPointer()
        Row = UpstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, One )
        Fall = UpstreamBoundaryValue()
        ConnectingNodes = UpstreamConnections()
        ConnectingChannel = UpstreamConnect(First)

      ELSE IF( Extremity .EQ. -1 ) THEN

*------ Downstream constraint.

        ConstraintNode = DownstreamPointer()
        Row = DownstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, One )
        Fall = DownstreamBoundaryValue()
        ConnectingNodes = DownstreamConnections()
        ConnectingChannel = DownstreamConnect(First)

      ELSE
        WRITE(*,*) ' *** Error (ForceEqualStreamSurface)'
        WRITE(*,*) ' Extremity not equal +1 or -1 ...'
        RETURN
      END IF



*---- Determine node number of attached channel.

      IF( ConnectingChannel .NE. 0 ) THEN

        Node = IABS( StreamEndNode( ConnectingChannel ) )

      ELSE

*------ No channel number ... not actually attached.

        OK = StoreAtRow(
     #       Row, Fall - GlobalStreamSurfaceElevation( ConstraintNode )
     #                 )

      END IF


      IF( ConnectingNodes .GT. 0 ) THEN


          OK = StoreAtRowOffset(
     #            Row, DF*(Node-ConstraintNode), -One
     #                         )

          OK = StoreAtRow(
     #                     Row,
     #                          Fall
     #         + GlobalStreamSurfaceElevation( Node )
     #         - GlobalStreamSurfaceElevation( ConstraintNode )
     #                   )

      ELSE

        WRITE(*,*) ' *** Error (ForceEqualStreamSurface)'
        WRITE(*,*) ' Connections = ', ConnectingNodes

      END IF

      ForceEqualStreamSurface = .TRUE.

      RETURN
      END



*== Public (ForceStreamSurfaceSlope) ===================================

      LOGICAL FUNCTION ForceStreamSurfaceSlope( Extremity )

      IMPLICIT NONE

*   Purpose:  Compute and store constraint-equation coefficients forcing
*             a known stream-surface slope.

*   Arguments:
      INTEGER Extremity

*   Argument definitions:
*     Extremity - index indicating
*                   [+1] upstream end of current channel.
*                   [-1] downstream end of current channel.

*   Module data:

*   Local Variables:
      INTEGER ConstraintNode, DF
      INTEGER Zero, Row, AdjacentNode
      REAL    One, Fall, Slope,  X1, X2
      REAL    DX
      PARAMETER ( Zero = 0, One = 1.0, DF = 2 )
      LOGICAL OK

*   Routines by module:

***** Channel flow status:
      REAL     StreamSurfaceElevation
      EXTERNAL StreamSurfaceElevation

***** Channel properties:
      EXTERNAL BtmElev
      REAL     BtmElev

***** Channel schematic:
      INTEGER  NumberOfStreamLocations
      EXTERNAL NumberOfStreamLocations

      REAL     StreamDistance
      EXTERNAL StreamDistance

***** Network control:
      REAL     UpstreamBoundaryValue, DownstreamBoundaryValue
      EXTERNAL UpstreamBoundaryValue, DownstreamBoundaryValue

***** Solver:
      INTEGER  UpstreamConstraintRow, DownstreamConstraintRow
      EXTERNAL UpstreamConstraintRow, DownstreamConstraintRow

      LOGICAL  StoreAtRowOffset, StoreAtRow
      EXTERNAL StoreAtRowOffset, StoreAtRow

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: August   1992
*

*   Intrinsics:
      INTEGER   IABS
      REAL      ABS
      INTRINSIC IABS, ABS


*-- Implementation -----------------------------------------------------

      ForceStreamSurfaceSlope = .FALSE.

      IF( Extremity .EQ. +1 ) THEN

*------ Upstream constraint.

        ConstraintNode = 1
        Row = UpstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, One )
        AdjacentNode = ConstraintNode + 1
        Slope = UpstreamBoundaryValue()
        X1 = StreamDistance( ConstraintNode )
        X2 = StreamDistance( AdjacentNode   )

      ELSE IF( Extremity .EQ. -1 ) THEN

*------ Downstream constraint.

        ConstraintNode = NumberOfStreamLocations()
        Row = DownstreamConstraintRow()
        OK = StoreAtRowOffset( Row, Zero, One )
        AdjacentNode = ConstraintNode - 1
        Slope = DownstreamBoundaryValue()
        X2 = StreamDistance( ConstraintNode )
        X1 = StreamDistance( AdjacentNode   )

      ELSE
        WRITE(*,*) ' *** Error (ForceStreamSurfaceSlope)'
        WRITE(*,*) ' Extremity not equal +1 or -1 ...'
        RETURN
      END IF


*---- Estimate fall between adjacent nodes...

      DX = X2 - X1

      Fall = DX * Slope

*---- Store coefficients.

      OK = StoreAtRowOffset(
     #            Row, DF*(AdjacentNode-ConstraintNode), -One
     #                     )

      OK = StoreAtRow(
     #                  Row,
     #                  Fall
     #       + StreamSurfaceElevation( AdjacentNode   )
     #       - StreamSurfaceElevation( ConstraintNode )
     #               )


      ForceStreamSurfaceSlope = .TRUE.

      RETURN
      END



*== Public (Force3ParmStreamSurface) ===================================

      LOGICAL FUNCTION Force3ParmStreamSurface( Extremity )

      IMPLICIT NONE

*   Purpose:  Constrain water surface elevation by forcing a
*             3-parameter-rating relation between two cross sections.

*   Arguments:
      INTEGER Extremity

*   Argument definitions:
*     Extremity - index indicating
*                   [+1] upstream end of current channel.
*                   [-1] downstream end of current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcnstrt.inc'

*   Local Variables:
      INTEGER ConnectingNodes
      INTEGER ConstraintNode, DF, ErrorFlag
      INTEGER Zero, Row, ConnectingNode, Code, MinusOne
      INTEGER ConnectingChannel, TableNo, Offset
      INTEGER First
      PARAMETER( First = 1 )
      REAL    One, Tw, Hw, dHwdTw, dHwdQ, Q
      PARAMETER ( Zero = 0, One = 1.0, DF = 2, MinusOne = -1 )
      LOGICAL OK
      LOGICAL Upstream, Downstream
      LOGICAL PositiveFlow, NegativeFlow

*   Routines by module:

***** Channel flow status:
      REAL     GlobalStreamSurfaceElevation
      REAL     GlobalStreamFlow
      EXTERNAL GlobalStreamSurfaceElevation, GlobalStreamFlow

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

      INTEGER  UpstreamConnect, UpstreamConnections
      EXTERNAL UpstreamConnect, UpstreamConnections

      INTEGER  DownstreamConnect, DownstreamConnections
      EXTERNAL DownstreamConnect, DownstreamConnections

      INTEGER  StreamEndNode, CurrentChannel, DownstreamCode
      EXTERNAL StreamEndNode, CurrentChannel, DownstreamCode
      
      INTEGER  UpstreamCode
      EXTERNAL UpstreamCode

***** Solver:
      INTEGER  UpstreamConstraintRow, DownstreamConstraintRow
      EXTERNAL UpstreamConstraintRow, DownstreamConstraintRow

      LOGICAL  StoreAtRowOffset, StoreAtRow
      EXTERNAL StoreAtRowOffset, StoreAtRow
      
***** 3-parameter rating:
      EXTERNAL TB3DWSUS

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: August   1992
*

*   Intrinsics:
      INTEGER   IABS
      REAL       ABS
      INTRINSIC IABS, ABS


*-- Implementation -----------------------------------------------------

      Force3ParmStreamSurface = .FALSE.

      IF( Extremity .EQ. +1 ) THEN
      
*------ Upstream constraint.

        Upstream = .TRUE.
        Downstream = .FALSE.
        ConstraintNode = UpstreamPointer()
        Row = UpstreamConstraintRow()
        ConnectingNodes = UpstreamConnections()
        ConnectingChannel = UpstreamConnect(First)
        Code = UpstreamCode()
        Offset = 0

      ELSE IF( Extremity .EQ. -1 ) THEN

*------ Downstream constraint.

        Upstream = .FALSE.
        Downstream = .TRUE.
        ConstraintNode = DownstreamPointer()
        Row = DownstreamConstraintRow()
        ConnectingNodes = DownstreamConnections()
        ConnectingChannel = DownstreamConnect(First)
        Code = DownstreamCode()
        Offset = 1

      ELSE
        WRITE(*,*) ' *** Error (Force3ParmStreamSurface)'
        WRITE(*,*) ' Channel...',CurrentChannel()
        WRITE(*,*) ' Extremity not equal +1 or -1 ...'
        RETURN
      END IF

      ConnectingNode = IABS( StreamEndNode( ConnectingChannel ) )

          
*         Q = flow
*         Hw = headwater elevation
*         Tw = tailwater elevation
*         dHwdTw = d(Hw)/d(Tw)
*         dHwdQ = d(Hw)/dQ


      IF( ConnectingNodes .EQ. 1 ) THEN

*------ Check direction of flow.

        Q = GlobalStreamFlow( ConstraintNode )
          
        IF( Q .GT. -1.0E-20 ) THEN
          PositiveFlow = .TRUE.
          NegativeFlow = .FALSE.
        ELSE
          PositiveFlow = .FALSE.
          NegativeFlow = .TRUE.
        END IF

*------ Determine table number.

        IF( NegativeFlow .AND. Reverse( 
     #               2 * CurrentChannel() - 1 + Offset
     #                                )
     #    ) THEN
*------------ use companion negative-flow table.
              TableNo = -Code
        ELSE
*------------ use positive-flow table for positive and negative flow.
              TableNo =  Code
        END IF


        IF( Downstream .AND. PositiveFlow
     #                 .OR.
     #      Upstream   .AND. NegativeFlow ) THEN
          
*-------- Hw at ConstraintNode, Tw at ConnectingNode.

          Tw = GlobalStreamSurfaceElevation( ConnectingNode )

            
          CALL TB3DWSUS
     I                   ( TableNo,Tw, Q,
     O                    Hw, dHwdTw, dHwdQ, ErrorFlag)
     
*-------- Place coefficients in matrix:

*------------ head water ... at constraint node, 

              OK = StoreAtRowOffset( Row, Zero, One )

*------------ tail water ... end of connecting channel,

              OK = StoreAtRowOffset(
     #            Row, DF*(ConnectingNode-ConstraintNode), -dHwdTw
     #                             )

*------------ flow ... at connecting node, and

              OK = StoreAtRowOffset(
     #            Row, MinusOne, -dHwdQ
     #                             )

*------------ right-hand side.

              OK = StoreAtRow(
     #                     Row, + Hw
     #         - GlobalStreamSurfaceElevation( ConstraintNode )
     #                       )
        ELSE

*-------- Downstream .AND. NegativeFlow
*                    .OR.
*         Upstream   .AND. PositiveFlow


*-------- Tailwater at ConstraintNode.

          Tw = GlobalStreamSurfaceElevation( ConstraintNode )

          CALL TB3DWSUS
     I                   ( TableNo,Tw, Q,
     O                    Hw,dHwdTw,dHwdQ,ErrorFlag)

*-------- Place coefficients in matrix:

*-------- tail water ... at constraint node,

          IF( ABS(dHwdTw) .LT. 0.0001 ) THEN            


*=============================================================
*        NOTE: If the 3-parameter rating becomes a 2-Parameter
*              rating, that is, Q becomes a function of Hw and
*              not Tw, this constrain equation is likely to be
*              unstable ... warn the user ... its happened!
               WRITE(*,*) ' '
               WRITE(*,*) ' ####Warning(Force3ParmStreamSurface)'
               WRITE(*,*) ' Execution may become unstable ...'
               WRITE(*,*) ' 3-parameter constraint applied to'
               WRITE(*,*) ' tailwater cross section when flow'
               WRITE(*,*) ' is no longer effected by tailwater.'
               WRITE(*,*) ' Constrain flow rather than water'
               WRITE(*,*) ' surface with 3-parameter rating...'
               WRITE(*,*) ' Current channel...',CurrentChannel()
               WRITE(*,*) ' '
*=============================================================


            OK = StoreAtRowOffset( Row, Zero, -dHwdTw )

*---------- head water ... end of a connecting channel,

            OK = StoreAtRowOffset(
     #          Row, DF*(ConnectingNode-ConstraintNode), One
     #                           )

*---------- flow ... downstream end of current channel, and

            OK = StoreAtRowOffset(
     #            Row, MinusOne, -dHwdQ
     #                           )

*---------- right-hand side.

            OK = StoreAtRow(
     #                     Row, + Hw
     #         - GlobalStreamSurfaceElevation( ConnectingNode )
     #                     )

            
          END IF

        END IF
        
      ELSE
          WRITE(*,*) ' *** Error (Force3ParmStreamSurface)'
          WRITE(*,*) ' Channel...',CurrentChannel()
          WRITE(*,*) ' Number of connections must be 1.'
          WRITE(*,*) ' Currently, connections = ', ConnectingNodes
      END IF



      Force3ParmStreamSurface = .TRUE.

      RETURN
      END







*== Public (Force3ParmStreamFlow) ===================================

      LOGICAL FUNCTION Force3ParmStreamFlow( Extremity )

      IMPLICIT NONE

*   Purpose:  Constrain stream flow by forcing a
*             3-parameter-rating relation between two cross sections.

*   Arguments:
      INTEGER Extremity

*   Argument definitions:
*     Extremity - index indicating
*                   [+1] upstream end of current channel.
*                   [-1] downstream end of current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcnstrt.inc'

*   Local Variables:
      INTEGER ConnectingNodes
      INTEGER ConstraintNode, DF, ErrorFlag
      INTEGER Zero, Row, ConnectingNode, Code, PlusOne
      INTEGER ConnectingChannel, TableNo, Offset
      INTEGER First
      PARAMETER( First = 1 )
      REAL    One, Tw, Hw, dHwdTw, dHwdQ, Q, TableQ
      PARAMETER ( Zero = 0, One = 1.0, DF = 2, PlusOne = 1 )
      LOGICAL OK
      LOGICAL Upstream, Downstream
      LOGICAL PositiveFlow, NegativeFlow

*   Routines by module:

***** Channel flow status:
      REAL     GlobalStreamSurfaceElevation
      REAL     GlobalStreamFlow
      EXTERNAL GlobalStreamSurfaceElevation, GlobalStreamFlow

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

      INTEGER  UpstreamConnect, UpstreamConnections
      EXTERNAL UpstreamConnect, UpstreamConnections

      INTEGER  DownstreamConnect, DownstreamConnections
      EXTERNAL DownstreamConnect, DownstreamConnections

      INTEGER  StreamEndNode, CurrentChannel, DownstreamCode
      EXTERNAL StreamEndNode, CurrentChannel, DownstreamCode
      
      INTEGER  UpstreamCode
      EXTERNAL UpstreamCode

***** Network control:

***** Solver:
      INTEGER  UpstreamConstraintRow, DownstreamConstraintRow
      EXTERNAL UpstreamConstraintRow, DownstreamConstraintRow

      LOGICAL  StoreAtRowOffset, StoreAtRow
      EXTERNAL StoreAtRowOffset, StoreAtRow
      
***** 3-parameter rating:
      REAL     TB3FLOW
      EXTERNAL TB3DWSUS, TB3FLOW

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: August   1992
*

*   Intrinsics:
      INTEGER   IABS
      INTRINSIC IABS


*-- Implementation -----------------------------------------------------

      Force3ParmStreamFlow = .FALSE.

      IF( Extremity .EQ. +1 ) THEN
      
*------ Upstream constraint.

        Upstream = .TRUE.
        Downstream = .FALSE.
        ConstraintNode = UpstreamPointer()
        Row = UpstreamConstraintRow()
        ConnectingNodes = UpstreamConnections()
        ConnectingChannel = UpstreamConnect(First)
        Code = UpstreamCode()
        Offset = 0

      ELSE IF( Extremity .EQ. -1 ) THEN

*------ Downstream constraint.

        Upstream = .FALSE.
        Downstream = .TRUE.
        ConstraintNode = DownstreamPointer()
        Row = DownstreamConstraintRow()
        ConnectingNodes = DownstreamConnections()
        ConnectingChannel = DownstreamConnect(First)
        Code = DownstreamCode()
        Offset = 1

      ELSE
        WRITE(*,*) ' *** Error (Force3ParmStreamFlow)'
        WRITE(*,*) ' Channel...',CurrentChannel()
        WRITE(*,*) ' Extremity not equal +1 or -1 ...'
        RETURN
      END IF

      ConnectingNode = IABS( StreamEndNode( ConnectingChannel ) )

          
*         Q = current flow
*         TableQ = Q from table, given current Hw & Tw
*         Hw = headwater elevation
*         Tw = tailwater elevation
*         dHwdTw = d(Hw)/d(Tw)
*         dHwdQ = d(Hw)/dQ


      IF( ConnectingNodes .EQ. 1 ) THEN

*------ Check direction of flow.

        Q = GlobalStreamFlow( ConstraintNode )
          
        IF( Q .GT. -1.0E-20 ) THEN
          PositiveFlow = .TRUE.
          NegativeFlow = .FALSE.
        ELSE
          PositiveFlow = .FALSE.
          NegativeFlow = .TRUE.
        END IF

*------ Determine table number.

        IF( NegativeFlow .AND. Reverse( 
     #               2 * CurrentChannel() - 1 + Offset
     #                                )
     #    ) THEN
*------------ use companion negative-flow table.
              TableNo = -Code
        ELSE
*------------ use positive-flow table for positive and negative flow.
              TableNo =  Code
        END IF


        IF( Downstream .AND. PositiveFlow
     #                 .OR.
     #      Upstream   .AND. NegativeFlow ) THEN
          
*-------- Hw at ConstraintNode, Tw at ConnectingNode.

          Tw = GlobalStreamSurfaceElevation( ConnectingNode )

            
          CALL TB3DWSUS
     I                   ( TableNo,Tw, Q,
     O                    Hw, dHwdTw, dHwdQ, ErrorFlag)

*-------- Place coefficients in matrix:

*------------ flow ... at ConstraintNode, 

              OK = StoreAtRowOffset( Row, Zero, dHwdQ )

*------------ tail water ... at ConnectingNode,

              OK = StoreAtRowOffset(
     #         Row, DF*(ConnectingNode-ConstraintNode)+1, -dHwdTw
     #                             )

*------------ head water ... at ConstraintNode, and

              OK = StoreAtRowOffset(
     #                     Row, PlusOne, -One
     #                             )

*------------ right-hand side.

              TableQ = TB3FLOW(
     #                   TableNo,
     #                   GlobalStreamSurfaceElevation( ConstraintNode ),
     #                   Tw
     #                        )

              OK = StoreAtRow(
     #             Row, dHwdQ * ( TableQ - Q )
     #                       )
        ELSE

*-------- Downstream .AND. NegativeFlow
*                    .OR.
*         Upstream   .AND. PositiveFlow


*-------- Tailwater at ConstraintNode, headwater at ConnectingNode.

          Tw = GlobalStreamSurfaceElevation( ConstraintNode )

          CALL TB3DWSUS
     I                   ( TableNo,Tw, Q,
     O                    Hw,dHwdTw,dHwdQ,ErrorFlag)



*-------- Place coefficients in matrix:

*---------- flow ... at ConstraintNode,

            OK = StoreAtRowOffset( Row, Zero, dHwdQ )

*---------- head water ... at ConnectingNode,

            OK = StoreAtRowOffset(
     #          Row, DF*(ConnectingNode-ConstraintNode)+1, -One
     #                           )

*---------- tailwater ... at ConstraintNode, and

            OK = StoreAtRowOffset(
     #            Row, PlusOne, -dHwdTw
     #                           )

*---------- right-hand side.

*------------ right-hand side.

              TableQ = TB3FLOW(
     #                   TableNo,
     #                   GlobalStreamSurfaceElevation( ConnectingNode ),
     #                   Tw
     #                        )

              OK = StoreAtRow(
     #             Row, dHwdQ * ( TableQ - Q )
     #                       )

        END IF
        
      ELSE
          WRITE(*,*) ' *** Error (Force3ParmStreamFlow)'
          WRITE(*,*) ' Channel...',CurrentChannel()
          WRITE(*,*) ' Number of connections must be 1.'
          WRITE(*,*) ' Currently, connections = ', ConnectingNodes
      END IF



      Force3ParmStreamFlow = .TRUE.

      RETURN
      END




*== Private (InitUserStreamConstraint) ============================

      LOGICAL FUNCTION InitUserStreamConstraint()

      IMPLICIT NONE

*   Purpose: Initialize user-supplied constraints ... for example,
*              read required coefficients from a file, set initial
*              value of variables, etc.

*   Note:  This routine will be called once, prior to the first time
*            step if any boundary-condition codes, ending with the
*            digits 51 or 52, are read from the channel-schematic file.
*            Any initializations required by user-progammed constraints
*            should be called from this routine.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcnstrt.inc'

*   Local Variables:
      INTEGER  NextToLastPair, Code
      INTEGER fUnit
      CHARACTER*5 Begin
      
      INTEGER DamCount
      INTEGER LogCount

      CHARACTER*12 FileName, InternalFileName

*   Routines by module:

***** Master file:
        INTEGER GetFileUnit
        CHARACTER*12 GetFileName
        EXTERNAL GetFileName, GetFileUnit

***** Local:

      LOGICAL  ReadDAMBRKData
      EXTERNAL ReadDAMBRKData
      
      LOGICAL  ReadLogStreamConstraint
      EXTERNAL ReadLogStreamConstraint

*  ( Make additional constraint-initialization declarations here...)
      
***** Channel schematic:

      INTEGER  NumberOfChannels, UpstreamCode, DownstreamCode
      EXTERNAL NumberOfChannels, UpstreamCode, DownstreamCode

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

***** File utilities:
        LOGICAL  OpenOldText
        EXTERNAL OpenOldText

      
*   Intrinsics:
      INTEGER   IABS, MOD
      INTRINSIC IABS, MOD

*   Programmed by: Lew DeLong
*   Date:          Oct   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

*---- Boundary-condition codes for user-programmed constraints
*     end with the pair of digits 51 or 52.  The next to last
*     pair of digits indicates which user-programmed constraint.
*
*       .....01.. an example dam break constraint, addapted from
*                  DAMBRK, National Weather Service, (Fread about 1984).
*        
*       .....02..  Q = a + b * Log Z.
*
*       .....03..  Q = a * Z ** b
*
*       .....04..  ...(add your own constraints, up to .....99.. )
*


      InitUserStreamConstraint = .FALSE.

*---- Initialize constraint-specific counters.

      DamCount = 0
      LogCount = 0


*---- Read any and all stream-constraint data as required.

*------ Open user-programmed-constraint data file.

*-------- Set properties file name and unit number.

           InternalFileName = 'strmcnst.dat'
      
*-------- Retrieve file name and unit number from master file

           fUnit = GetFileUnit(InternalFileName)
           FileName = GetFileName(InternalFileName)

           IF(OpenOldText(fUnit,FileName)) THEN
           ELSE
             WRITE(*,*) ' ####Error(InitUserStreamConstraint)'
             WRITE(*,*) ' Could not open ',FileName
             RETURN
           END IF

*------ Read data ... loop till EOF.

 300    CONTINUE
 
        READ(fUnit,'(A5)',END=350) Begin

        IF( Begin .EQ. 'begin' ) THEN

          READ(fUnit,*) Code
          NextToLastPair = IABS( MOD( Code/100 , 100 ) )
          
*-------- Read data for specific constraint.

          IF( NextToLastPair .EQ. 1 ) THEN

*---------- Read dam-break-constraint data.

            DamCount = DamCount + 1
            IF( ReadDAMBRKData( fUnit, DamCount, Code ) ) THEN
            ELSE
              RETURN
            END IF

          ELSE IF( NextToLastPair .EQ. 2 ) THEN
          
*---------  Read semi-log relation.
            
            LogCount = LogCount + 1
            IF( ReadLogStreamConstraint( fUnit, LogCount, Code ) ) THEN
            ELSE
              RETURN
            END IF


          ELSE IF( NextToLastPair .EQ. 3 ) THEN

*---------  Read log relation.
            
            LogCount = LogCount + 1
            IF( ReadLogStreamConstraint( fUnit, LogCount, Code ) ) THEN
            ELSE
              RETURN
            END IF

          ELSE IF( NextToLastPair .EQ. 4 ) THEN

*---------- Insert data-reading functions as required for
*            additional constraints.          
          
          END IF


        ELSE

*         Skip for "begin" record...

          GO TO 300
        

        END IF
		
		GO TO 300
     
 350  CONTINUE

      InitUserStreamConstraint = .TRUE.

      RETURN
      END




*== Public (ForceUserStreamConstraint) ===================================

      LOGICAL FUNCTION ForceUserStreamConstraint( Extremity )

      IMPLICIT NONE

*   Purpose:  Constrain water surface elevation, discharge, or a
*             combination by forcing a user-programmed relation.

*   Arguments:
      INTEGER Extremity

*   Argument definitions:
*     Extremity - index indicating
*                   [+1] upstream end of current channel.
*                   [-1] downstream end of current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'chcnstrt.inc'

      INTEGER NextToLastPair, Code

*   Routines by module:

***** Local:

      LOGICAL   SetStreamConstraintVariables
      EXTERNAL  SetStreamConstraintVariables
      
      LOGICAL   SetUserStreamConstraintCoef
      EXTERNAL  SetUserStreamConstraintCoef
      
      LOGICAL   FreadsDAMBRK
      EXTERNAL  FreadsDAMBRK
      
      LOGICAL   ZLogStreamConstraint, QLogStreamConstraint
      EXTERNAL  ZLogStreamConstraint, QLogStreamConstraint
      
      LOGICAL   ZSemiLogStreamConstraint
      EXTERNAL  ZSemiLogStreamConstraint

*     ( Declare additional user-programmed constraint functions here. )

      
***** Channel schematic:

      INTEGER  UpstreamCode, DownstreamCode
      EXTERNAL UpstreamCode, DownstreamCode

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel
      
*   Intrinsics:
      INTEGER   IABS, MOD
      INTRINSIC IABS, MOD

*   Programmed by: Lew DeLong
*   Date:          Oct   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

*---- Boundary-condition codes for user-programmed constraints
*     end with the pair of digits 51 or 52.  The next to last
*     pair of digits indicates which user-programmed constraint.
*
*       .....01.. an example dam break constraint, addapted from
*                  DAMBRK, National Weather Service, (Fread about 1984).
*
*
*
*                 an example semi-log stage-discharge relation.
*        
*       .....0251  Z = Zo + a + b * LOG10 ( Q ), water-surface constraint,
*                  or
*       .....0252  Q = a + b * LOG10 ( Z - Zo ), if discharge constraint.
*
*
*
*                  an example log stage-discharge relation.
*
*       .....0351  Z = Zo + a * Q ** b if water-surface constraint.
*                  or
*       .....0352  ( not implemented ).
*
*
*
*
*       .....04..  ...(add your own constraints, up to .....99.. )
*


      ForceUserStreamConstraint = .FALSE.
      
*---- Get boundary-condition code.

      IF( Extremity .EQ. +1 ) THEN
        Code = UpstreamCode()
      ELSE 
        Code = DownstreamCode()
      END IF
      
*---- Set current values of extremities and dependent variables.

      IF( SetStreamConstraintVariables( Extremity ) ) THEN
      ELSE
        RETURN
      END IF


*---- Check for specific constraint.


      NextToLastPair = IABS( MOD( Code/100 , 100 ) )
            
      IF( NextToLastPair .EQ. 1 ) THEN
            
*------ Example dam-break constraint.
        IF( FreadsDAMBRK() ) THEN
        ELSE
          RETURN
        END IF

      ELSE IF( NextToLastPair .EQ. 2 ) THEN

*-----  Example semi-log constraint.

        IF( IABS( MOD( Code, 2 )  ).EQ. 1  )  THEN

*-------- applied as a water-surface constraint.

          IF( ZSemiLogStreamConstraint() ) THEN
          ELSE
            RETURN
          END IF
        ELSE
		
*-------- applied as a discharge constraint.

          WRITE(*,*) ' *** error(ForceUserStreamConstraint)'
          WRITE(*,*) ' Semi-log stage-discharge relation is'
          WRITE(*,*) ' implemented as a discharge constraint.'
          WRITE(*,*) ' Boundary-condition code...',Code
          RETURN
        END IF
      
      ELSE IF( NextToLastPair .EQ. 3 ) THEN

*-----  Example log constraint.

        IF( IABS( MOD( Code, 2 )  )  .EQ. 1 ) THEN

*-------- applied as a water-surface constraint.

          IF( ZLogStreamConstraint() ) THEN
          ELSE
            RETURN
          END IF
		  
        ELSE
		
*-------- applied as a discharge constraint.

          IF( QLogStreamConstraint() ) THEN
          ELSE
            RETURN
          END IF
          
        END IF

      ELSE IF( NextToLastPair .EQ. 4 ) THEN
              
            
*------ Do another type constraint...

        WRITE(*,*) ' ####Error(ForceUserStreamConstraint)'
        WRITE(*,*) ' Condition code...',Code,' not implemented.'
        RETURN
              
*------ Replace preceding 3 lines with your code and declare
*       functions appropriately above.

*       Add as many "ELSE IF(..." blocks as needed,
*       up to a total of 99 types of constraints...
*       Remember to appropriately initialize in
*       InitUserStreamConstraint.

      ELSE
              
        WRITE(*,*) ' ####Error(ForceUserStreamConstraint)'
        WRITE(*,*) ' Condition code...',Code,' not implemented.'
        RETURN
                
      END IF
                
*---- Place constraint coefficients in solution matrix.

      IF( SetUserStreamConstraintCoef() ) THEN
      ELSE
        RETURN
      END IF

 
 

      ForceUserStreamConstraint = .TRUE.

      RETURN
      END






*== Private (SetUserStreamConstraintCoef) ===============================

      LOGICAL FUNCTION SetUserStreamConstraintCoef()

      IMPLICIT NONE

*   Purpose:  Place coefficients computed by specific user-programmed
*             constraints in global solution matrix.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'strmcnst.inc'

*   Local Variables:
      INTEGER Extremity, Offset, Row, Span
      INTEGER I, ConnectingNodes, Node(MaxConnectingChannels)
      INTEGER ConstraintNode
      LOGICAL OK
      
*   Routines by module:

***** Local:

***** Channel schematic:
      INTEGER  CurrentChannel
      EXTERNAL CurrentChannel
      
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

      INTEGER  UpstreamConnect, UpstreamConnections
      EXTERNAL UpstreamConnect, UpstreamConnections

      INTEGER  DownstreamConnect, DownstreamConnections
      EXTERNAL DownstreamConnect, DownstreamConnections

      INTEGER  StreamEndNode
      EXTERNAL StreamEndNode

***** Solver:
      INTEGER  UpstreamConstraintRow, DownstreamConstraintRow
      EXTERNAL UpstreamConstraintRow, DownstreamConstraintRow

      LOGICAL  StoreAtRowOffset, StoreAtRow
      EXTERNAL StoreAtRowOffset, StoreAtRow


*   Intrinsics:
      INTEGER   IABS, MOD
      INTRINSIC IABS, MOD

*   Programmed by: Lew DeLong
*   Date:          Oct   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      SetUserStreamConstraintCoef = .FALSE.

      Extremity = ConnectingExtremity(1)
      
      IF( Extremity .EQ. +1 ) THEN

*------ Upstream constraint.

        ConstraintNode = UpstreamPointer()
        Row = UpstreamConstraintRow()
        ConnectingNodes = UpstreamConnections()
        IF( ConnectingNodes .GT. 0 ) THEN
          DO 100 I=1,ConnectingNodes
            Node(I) = IABS(StreamEndNode( UpstreamConnect(I) ))
 100      CONTINUE
        ELSE
        END IF

      ELSE IF( Extremity .EQ. -1 ) THEN

*------ Downstream constraint.

        ConstraintNode = DownstreamPointer()
        Row = DownstreamConstraintRow()
        ConnectingNodes = DownstreamConnections()
        IF( ConnectingNodes .GT. 0 ) THEN
          DO 200 I=1,ConnectingNodes
            Node(I) = IABS(StreamEndNode( DownstreamConnect(I) ))
 200      CONTINUE
        ELSE
        END IF

      ELSE
        WRITE(*,*) ' *** Error(SetUserStreamConstraintCoef)'
        WRITE(*,*) ' Extremity not equal +1 or -1 ...'
        WRITE(*,*) ' Channel...', CurrentChannel()
        RETURN
      END IF
      
      
*---- Determine if flow or water-surface constraint, set offset.

      IF( MOD( ConditionCode, 10 ) .EQ. 2 ) THEN
        Offset = 0
      ELSE
        Offset = -1
      END IF
      
*---- Set coefficients for constraint node.

*---- Constraint-node coefficients for

*          discharge, and
      OK = StoreAtRowOffset( Row, Offset  , ConstraintCoef(1) ) 

*          water surface.
      OK = StoreAtRowOffset( Row, Offset+1, ConstraintCoef(2) ) 

*---- Right-hand side.

      OK = StoreAtRow( Row, ConstraintRightSide )


      IF( ConnectingNodes .GT. 0 ) THEN

        DO 300 I=1,ConnectingNodes

          Span = 2*(Node(I)-ConstraintNode)

*-------- discharge coefficient.
          OK = StoreAtRowOffset( 
     #           Row, Offset+Span,   ConstraintCoef(I+2)
     #                         )

*-------- water-surface coeeficient.
          OK = StoreAtRowOffset( 
     #           Row, Offset+Span+1, ConstraintCoef(I+3)
     #                         )

 300    CONTINUE

      END IF

      SetUserStreamConstraintCoef = .TRUE.

      RETURN
      END




*== Private (SetStreamConstraintVariables) =============================

      LOGICAL FUNCTION SetStreamConstraintVariables( Extremity )

      IMPLICIT NONE

*   Purpose:  Set value of variables that may be used by
*             user-programmed constraints.


*   Arguments:
      INTEGER Extremity

*   Argument definitions:
*     Extremity - index indicating
*                   [+1] upstream end of current channel.
*                   [-1] downstream end of current channel.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'strmcnst.inc'

*   Local Variables:
      INTEGER I
      INTEGER ConnectingNodes
      INTEGER ConstraintNode
      INTEGER ConnectingNode
      INTEGER ConnectingChannel


*   Routines by module:

***** Channel flow status:
      REAL     GlobalStreamSurfaceElevation
      REAL     GlobalStreamFlow
      EXTERNAL GlobalStreamSurfaceElevation, GlobalStreamFlow

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

      INTEGER  UpstreamConnect, UpstreamConnections
      EXTERNAL UpstreamConnect, UpstreamConnections

      INTEGER  DownstreamConnect, DownstreamConnections
      EXTERNAL DownstreamConnect, DownstreamConnections

      INTEGER  StreamEndNode, CurrentChannel, DownstreamCode
      EXTERNAL StreamEndNode, CurrentChannel, DownstreamCode
      
      INTEGER  UpstreamCode
      EXTERNAL UpstreamCode

*   Intrinsics:
      INTEGER   IABS
      INTRINSIC IABS

*   Programmed by: Lew DeLong
*   Date:          Oct   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      SetStreamConstraintVariables = .FALSE.

      ConnectingExtremity(1) = Extremity

      IF( Extremity .EQ. +1 ) THEN
      
*------ Upstream constraint.

        ConstraintNode = UpstreamPointer()
        ConnectingNodes = UpstreamConnections()
        ConditionCode = UpstreamCode()

      ELSE IF( Extremity .EQ. -1 ) THEN

*------ Downstream constraint.

        ConstraintNode = DownstreamPointer()
        ConnectingNodes = DownstreamConnections()
        ConditionCode = DownstreamCode()

      ELSE
        WRITE(*,*) ' *** Error(SetStreamConstraintVariables)'
        WRITE(*,*) ' Channel...',CurrentChannel()
        WRITE(*,*) ' Extremity not equal +1 or -1 ...'
        RETURN
      END IF

        ConnectingChannels = ConnectingNodes
        Discharge(1) = GlobalStreamFlow( ConstraintNode )
        WSElev(1) = GlobalStreamSurfaceElevation( ConstraintNode )

        IF( ConnectingNodes .GT. 0 ) THEN
          IF( ConnectingNodes .LE. MaxConnectingChannels ) THEN
          
            IF( Extremity .EQ. +1 ) THEN
            
              DO 100 I=1,ConnectingNodes
                        
                ConnectingChannel = UpstreamConnect(I)

                ConnectingNode =
     #           IABS( StreamEndNode( ConnectingChannel ) )

                Discharge(I+1) = GlobalStreamFlow( ConnectingNode )
          
                WSElev(I+1) =
     #            GlobalStreamSurfaceElevation( ConnectingNode )
     
                ConnectingExtremity(I+1) = 
     #            ConnectingChannel / IABS( ConnectingChannel )

 100          CONTINUE
            
            ELSE

              DO 200 I=1,ConnectingNodes
                        
                ConnectingChannel = DownstreamConnect(I)

                ConnectingNode =
     #           IABS( StreamEndNode( ConnectingChannel ) )

                Discharge(I+1) = GlobalStreamFlow( ConnectingNode )
          
                WSElev(I+1) =
     #            GlobalStreamSurfaceElevation( ConnectingNode )

                ConnectingExtremity(I+1) = 
     #            ConnectingChannel / IABS( ConnectingChannel )

 200          CONTINUE
            
            END IF
              
      
          ELSE
            WRITE(*,*) ' ####Error(SetStreamConstraintVariables)'
            WRITE(*,*) ' Channel...',CurrentChannel()
            WRITE(*,*) ' Number of connections must be fewer than...',
     #               MaxConnectingChannels
            WRITE(*,*) ' Currently, connections = ', ConnectingNodes
            RETURN
          END IF

        END IF
        
*----- Initialize global solution coefficients.
       
       ConstraintRightSide = 0.0
       
       DO 300 I=1,2+2*MaxConnectingChannels

         ConstraintCoef(I) = 0.0

 300   CONTINUE


      SetStreamConstraintVariables = .TRUE.

      RETURN
      END




*== Private (FreadsDAMBRK) ===============================================

      LOGICAL FUNCTION FreadsDAMBRK()

      IMPLICIT NONE

*   Purpose:  Compute constraint coefficients for a trapezoidal  breach,
*             enlarging with time. Basic equations are adapted from 
*             DAMBRK, the National Weather Service dam-break forcasting
*             model (Fread,1984).  The basic equations for flow over a
*             dam and flow over uncontrolled or gated spillways have 
*             also been adapted from Fread(1984).  This constraint is 
*             included as an example of a user-programmed constraint.
*
*             As implemented, this constraint may be applied only to
*             discharge at the downstream end of a channel.  The
*             channel may represent an impoundment of water.
*
*             Note that typically, implementation of a user-programmed
*             constraint requires minor modification of just two existing
*             routines:
*
*              [1] InitUserStreamConstraint - used for general
*                  initialization of all user-programmed constraints,
*
*              [2] ForceUserStreamConstraint - used for general 
*                  execution all user-programmed constraints,
*
*             and addition of routines to implement specific
*             constraint initialization and  execution, in this case
*             just three routines:
*
*              [1] ReadDAMBRKData - for reading coefficients,
*              [2] FreadsDAMBRK, for the actual implementaion of the
*                  specific constraint, and
*              [3] GetDAMBRKCoef, to get specific required coefficients
*                  during model execution.
*

*   Arguments:

*   Argument definitions:

*   Module data:
*     
      INCLUDE 'network.inc'
*             ( This is the one file that any module may include, and
*               is required for 'strmcnst.inc'. ) 

      INCLUDE 'strmcnst.inc'
*             ( This file should be included with all user-programmed
*               constraints. It contains current values of required
*               flows and water-surface elevations as well as  arrays of 
*               coefficients to be computed by the specific constraint
*               routine.)


*   Local Variables:
      INTEGER i
      REAL   Q
      REAL   QB, QS, QG, QD
      REAL   C1,C2,BI,CV,KS, Y, YT
      REAL   DC1DQ,DC1DY,DC2DQ,DC2DY,DCVDQ,DCVDY,DKSDY
      REAL   DFBDY,DFBDQ,DFGDY,DFDDY
      REAL   YB,HB,HBM,HTB,HTR,DIF,DFSDY

      REAL    YD, BD
      REAL    BT, TTB, ISS, YBM, YBRCH, TB
      REAL    SC, SL, SH
      REAL    GC, GA, GH
      REAL    DC, DL, DH
      REAL    QT
      
      REAL    QTotal, DDU, DDY
      
      LOGICAL WSConstrained, ok, Breached

*   Routines by module:

***** Local:
      LOGICAL  GetDAMBRKData, StoreBreachElapseTime
      EXTERNAL GetDAMBRKData, StoreBreachElapseTime
      
      LOGICAL  StoreBreachInitiation
      EXTERNAL StoreBreachInitiation
      
***** Network control:
      INTEGER  NetworkTimeIncrement, NetworkIteration
      EXTERNAL NetworkTimeIncrement, NetworkIteration


*   Programmed by: Lew DeLong
*   Date:          October 1992
*   Modified by:   
*   Last modified: 

*   Intrinsics:


*-- Implementation -----------------------------------------------------

      FreadsDAMBRK = .FALSE.
          


************************************************************************
*
*     Dam-break coefficient definitions:
*
************************************************************************
*
*                 dam
*
*     ID(i)     - associated boundary-condition code
*     YD(i)     - elevation of top of dam
*     BD(i)     - width of the reservoir at the dam
*
*
*                  breach
*
*     BT(i)     - final breach width
*     TTB(i)    - total time of breach formation, in hours
*     ISS(i)    - inverse of side slope of breach (horizontal/vertical)
*     YBM(i)    - final elevation of breach bottom
*     YBRCH(i)  - water surface elevation at which breach initiates
*     TB(i)     - time elapsed since start of breach
*
*
*                 uncontrolled spillway
*
*     SC(i)     - uncontrolled spillway coefficient
*     SL(i)     - spillway length
*     SH(i)     - spillway crest elevation
*
*
*                 gate
*
*     GC(i)     - gate discharge coefficient
*     GA(i)     - gate cross section area
*     GH(i)     - center-line elevation of gated spillway
*
*
*                 flow over dam
*
*     DC(i)     - coefficient fo flow over crest of dam
*     DL(i)     - length of dam less uncontrolled spillway length
*     DH(i)     - dam crest elevation
*
*
*                 flow through turbine
*
*     QT(i)     - constant flow through turbine
*
*
*     i         - associated sequence number
*
*
*                 computed discharges
*
*     QB        - discharge through breach
*     QS        - discharge over uncontrolled spillway
*     QG        - discharge through gate
*     QD        - discharge over dam
*
************************************************************************

*---- Get required data.

*     Note: All arguments in this function except the first are
*           returned only.  Only the first argument must be set
*           prior to call.  The first argument was previously 
*           set by a call to SetStreamConstraintVariables.

      IF( GetDAMBRKData(
     I                   ConditionCode,
     O                   i,
     O                   YD, BD,
     O                   BT, TTB, ISS, YBM, YBRCH, TB,
     O                   SC, SL, SH,
     O                   GC, GA, GH,
     O                   DC, DL, DH,
     O                   QT, Breached
     #                 )                ) THEN
      ELSE
        RETURN
      END IF

*---- Check to see what is constrained.

      IF( MOD( ConditionCode, 100 ) .EQ. 52 ) THEN
        WSConstrained = .FALSE.
      ELSE
        WSConstrained = .TRUE.
      END IF


************************************************************************
*
*        Discharge through breach
*
*        QB = discharge through the breach
*        QB = C1*(Y-YB)**3/2 + C2*(Y-YB)**5/2
*
*        Q  = current total discharge at dam
*        Y  = current water elevation at dam
*        C1 = 3.1*BI*CV*KS
*        C2 = 2.45*ISS*CV*KS
*        CV = velocity correction coefficient
*           = 1.0+0.023*Q**2/( BD**2*(Y-YBM)**2*(Y-YB) )
*        BI = instantaneous breach width at water surface
*           = BT*TB/TTB
*        YB = elevation of bottom of breach
*        KS = submergence correction coefficient
*           = 1.0 if (YT-YB)/(Y-YB) < 0.67
*           = 1.0-27.8*( (YT-YB)/(Y-YB)-0.67 )**3 otherwise
*
************************************************************************

*---- Set current values.

*     discharge at constraint cross section
      Q  = Discharge(1)

*     water-surface elevation at constraint cross section
      Y  = WSElev(1)

*     water-surface elevation at connecting cross section
      YT = WSElev(2)

      IF(Y.GE.YBRCH
     #     .OR.
     #   Breached ) THEN
     
        IF( .NOT. Breached ) THEN
          ok = StoreBreachInitiation(i)
        END IF
        

*------ Increment breach elapse time.

        IF(NetworkIteration().EQ.1
     #                       .OR.
     #                     TB .LT. 0.0001 ) THEN
          TB=TB+FLOAT(NetworkTimeIncrement())/3600.
          ok = StoreBreachElapseTime( i, TB )
        END IF

*------ Compute current breach width (BI)
*       & elevation of breach bottom (YB).

        IF (TB.LT.TTB) THEN
          BI=BT*TB/TTB
          YB=YD-(YD-YBM)*TB/TTB
        ELSE
          BI=BT
          YB=YBM
        END IF

*------ Compute coefficients for trapezoidal breach.

        HB=Y-YB
        IF( HB .GT. 0.0 ) THEN
          HBM=Y-YBM
          DCVDQ=0.046*Q/((BD*HBM)**2*HB)
          HTB=YT-YB
          HTR=HTB/HB
          IF (HTR.LT.0.67) THEN
            KS=1.0
          ELSE
            KS=1.0-27.8*(HTR-0.67)**3
          END IF
          DC1DQ=3.1*BI*DCVDQ*KS
          CV=1.0+0.5*Q*DCVDQ
          DC2DQ=2.45*ISS*KS*DCVDQ
          DCVDY=-0.023*Q*Q*(2.0*HB+HBM)/((HB*BD)**2*HBM*HBM*HBM)
          IF (HTR.LT.0.67) THEN
            DKSDY=0.0
          ELSE
            DKSDY=27.8*(HTR-0.67)**2*HTR/HB
          END IF
          DC2DY=CV*DKSDY+KS*DCVDY
          DC1DY=3.1*BI*DC2DY
          DC2DY=2.45*ISS*DC2DY
          C1=CV*KS
          C2=2.45*ISS*C1
          C1=C1*3.1*BI
          DFBDY=HB**.5*( C1+HB*(DC1DY+C2+HB*DC2DY) )
          DFBDQ=HB**1.5*(DC1DQ+DC2DQ*HB)
          DFBDQ=DFBDQ-1.0

*-------- Compute breach discharge.

          QB=C1*HB**1.5+C2*HB**2.5
          
        ELSE
          DFBDQ=0.0
          DFBDY=0.0
          QB=0.0
        END IF
        
      ELSE
        DFBDQ=0.0
        DFBDY=0.0
        QB=0.0
      END IF



************************************************************************
*
*       Uncontrolled spillway.
*
*        QS = Discharge over uncontroled spillway
*        QS = SC*SL*(Y-SH)**1.5
*
************************************************************************
      
      DIF=Y-SH
      IF(DIF.GT.0.01) THEN
        DFSDY=SC*SL*DIF**0.5
        QS=SC*SL*DIF**1.5
      ELSE
        DFSDY=0.0
        QS=0.0
      END IF


************************************************************************
*
*     Gated spillway.
*             
*     QG = discharge through gate
*     QG = GC*GA*(Y-GH)**0.5
*
************************************************************************

      DIF=Y-GH
      IF(DIF.GT.0.01) THEN
        DFGDY=GC*GA/DIF**0.5
        QG = GC*GA*DIF**0.5
      ELSE
        DFGDY=0.0
        QG=0.0
      END IF


************************************************************************
*
*      Flow over dam.
*
*      QD = flow over dam
*      QD = DC*DL*(Y-DH)**1.5
*
************************************************************************

      DIF=Y-DH
      IF(DIF.GT.0.01) THEN
        DFDDY=DC*DL*DIF**0.5
        QD=DC*DL*DIF**1.5
      ELSE
        DFDDY=0.0
        QD=0.0
      END IF


************************************************************************
*     Sum coefficients for all flows.
************************************************************************

      QTotal = QB + QS + QG + QD + QT
      DDU=DFBDQ
      DDY=DFBDY+DFSDY+DFGDY+DFDDY
      

************************************************************************
*     Load coefficients for export.
************************************************************************

      ConstraintCoef(1) = DDU-1.0
      ConstraintCoef(2) = DDY
      ConstraintRightSide = Q - QTotal

*---- Check coefficients for weak diagonal.

      IF( WSConstrained ) THEN
      
        IF( ABS( ConstraintCoef(1) / ConstraintCoef(2) )
     #                           .GT.
     #                          1.0E-06 ) THEN
        ELSE
          
          WRITE(*,*) ' ****Warning(FreadsDAMBRK)'
          WRITE(*,*) ' May not be able to constrain WS elevation'
          WRITE(*,*) ' using this function -- not diagonally dominant.'
        END IF
      END IF

      FreadsDAMBRK = .TRUE.


      RETURN
      END





*== Private (ReadDAMBRKData) ===========================================

      LOGICAL FUNCTION ReadDAMBRKData( fUnit, i, Code )

      IMPLICIT NONE

*   Purpose:  Read data (to be used by FreadsDAMBRK function) for 1 dam.

*   Arguments:
      INTEGER fUnit, i, Code

*   Argument definitions:
*     fUnit - FORTRAN unit number for input.
*     i - sequence number for the current dam.
*     Code - boundary condition code.

*   Module data:
      INCLUDE 'network.inc'

      INCLUDE 'dambrk.inc'
*             ( This file is specific to the FreadsDAMBRK constraint.
*               Files for other constraints, if required should
*               be implemented separately. )

*   Local Variables:
      CHARACTER*3 TheEnd

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Oct   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      ReadDAMBRKData = .FALSE.
            
      ID(i) = Code

*---- Read parameters.

      READ(fUnit,*) YD(i)
      READ(fUnit,*) BD(i)
      
      READ(fUnit,*) BT(i)
      READ(fUnit,*) TTB(i)
      TB(i) = 0.0
      
      READ(fUnit,*) ISS(i)
      READ(fUnit,*) YBM(i)
      READ(fUnit,*) YBRCH(i)
      
      READ(fUnit,*) SC(i)
      READ(fUnit,*) SL(i)
      READ(fUnit,*) SH(i)
      
      READ(fUnit,*) GC(i)
      READ(fUnit,*) GA(i)
      READ(fUnit,*) GH(i)
      
      READ(fUnit,*) DC(i)
      READ(fUnit,*) DL(i)
      READ(fUnit,*) DH(i)
      
      READ(fUnit,*) QT(i)
      
      Breached(i) = .FALSE.
      
 100  CONTINUE
      READ(fUnit,'(A3)',END=200) TheEnd
      
      IF( TheEnd .EQ. 'end' ) THEN
      ELSE
*       WRITE(*,*) ' ...(ReadDAMBRKData), skipping for "end"...'
*       WRITE(*,*) ' ...read "',TheEnd,'"'
        GO TO 100
      END IF
      
      GO TO 300
      
 200  CONTINUE
        WRITE(*,*) ' ####Error(ReadDAMBRKData)'
        WRITE(*,*) ' Failed to successfully read DAMBRK data...'
        WRITE(*,*) ' for code...',Code
        WRITE(*,*) YD(i)
        WRITE(*,*) BD(i)
        
        WRITE(*,*) BT(i)
        WRITE(*,*) TTB(i)
        WRITE(*,*) ISS(i)
        WRITE(*,*) YBM(i)
        WRITE(*,*) YBRCH(i)
        
        WRITE(*,*) SC(n)
        WRITE(*,*) SL(i)
        WRITE(*,*) SH(i)
        
        WRITE(*,*) GC(i)
        WRITE(*,*) GA(i)
        WRITE(*,*) GH(i)
        
        WRITE(*,*) DC(i)
        WRITE(*,*) DL(i)
        WRITE(*,*) DH(i)
        
        WRITE(*,*) QT(i)
        
        WRITE(*,'(A4)') TheEnd
        RETURN

 300  CONTINUE        
      
        ReadDAMBRKData = .TRUE.

      RETURN
      END




*== Private (GetDAMBRKData) ============================================

      LOGICAL FUNCTION GetDAMBRKData( 
     I                      sID,
     O                       i,
     O                      sYD, sBD,
     O                      sBT, sTTB, sISS, sYBM, sYBRCH, sTB,
     O                      sSC, sSL, sSH,
     O                      sGC, sGA, sGH,
     O                      sDC, sDL, sDH,
     O                      sQT, sBreached
     #                              )

      IMPLICIT NONE

*   Purpose:  Get data (to be used by FreadsDAMBRK function) for 1 dam,
*             instance i.

*   Arguments:

*     (input)
      INTEGER sID

*     (output)
      INTEGER i
      REAL    sYD, sBD
      REAL    sBT, sTTB, sISS, sYBM, sYBRCH, sTB
      REAL    sSC, sSL, sSH
      REAL    sGC, sGA, sGH
      REAL    sDC, sDL, sDH
      REAL    sQT
      LOGICAL sBreached

*   Argument definitions:
*   ( see FreadsDAMBRK for balance of definitions )
*   i  - instance number.
*   Breached - TRUE if breach has been initiated, otherwise FALSE.

*   Module data:
      INCLUDE 'network.inc'

      INCLUDE 'dambrk.inc'
*             ( This file is specific to the FreadsDAMBRK constraint.
*               Files for other constraints, if required should
*               be implemented separately. )

*   Local Variables:
      INTEGER j

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Oct   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      GetDAMBRKData = .FALSE.

*---- Determine which dam.

      DO 100 j=1,2*MaxChannels
        IF( sID .EQ. ID(j) ) THEN
          i = j
          GO TO 102
        END IF
100   CONTINUE
      WRITE(*,*) ' ####Error(GetDAMBRKData)'
      WRITE(*,*) ' Could not find boundary-condition code...',sID
      RETURN
102   CONTINUE


*---- Load coefficients.

      sID = ID(i)
      sYD = YD(i)
      sBD = BD(i)

      sBT    = BT(i)
      sTTB   = TTB(i)
      sISS   = ISS(i)
      sYBM   = YBM(i)
      sYBRCH = YBRCH(i)
      sTB    = TB(i)
      
      sSC = SC(i)
      sSL = SL(i)
      sSH = SH(i)
      
      sGC = GC(i)
      sGA = GA(i)
      sGH = GH(i)
      
      sDC = DC(i)
      sDL = DL(i)
      sDH = DH(i)

      sQT = QT(i)
      
      sBreached = Breached(i)
      
      GetDAMBRKData = .TRUE.

      RETURN
      END




*== Private (StoreBreachElapseTime) ===============================

      LOGICAL FUNCTION StoreBreachElapseTime(
     #                                i, ElapseTime
     #                                      )

      IMPLICIT NONE

*   Purpose:  Store time elapsed since beginning of breach for
*             instance i.

*   Arguments:
      INTEGER i
      REAL    ElapseTime

*   Argument definitions:
*      i -  instance number.
*      ElapseTime - time elapsed since beginning of breach.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'dambrk.inc'
*             ( This file is specific to the FreadsDAMBRK constraint.
*               Files for other constraints, if required should
*               be implemented separately. )


*   Local Variables:

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Nov   1994
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      
      StoreBreachElapseTime = .FALSE.

      TB(i) = ElapseTime
      
      StoreBreachElapseTime = .TRUE.

      RETURN
      END



*== Private (StoreBreachInitiation) ===============================

      LOGICAL FUNCTION StoreBreachInitiation(
     #                                        i
     #                                      )

      IMPLICIT NONE

*   Purpose:  Store initiation of breach for
*             instance i.

*   Arguments:
      INTEGER i

*   Argument definitions:
*      i -  instance number.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'dambrk.inc'
*             ( This file is specific to the FreadsDAMBRK constraint.
*               Files for other constraints, if required should
*               be implemented separately. )


*   Local Variables:

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Nov   1994
*   Modified by:   
*   Last modified: 

*-- Implementation -----------------------------------------------------

      
      StoreBreachInitiation = .FALSE.

      Breached(i) = .TRUE.
      
      StoreBreachInitiation = .TRUE.

      RETURN
      END






*== Private (ReadLogStreamConstraint) ===============================

      LOGICAL FUNCTION ReadLogStreamConstraint(
     #                                fUnit, LogCount, Code
     #                                        )

      IMPLICIT NONE

*   Purpose:  Read data for log or semi-log constraint between discharge
*             and water-surface elevation.

*   Arguments:
      INTEGER fUnit, LogCount, Code

*   Argument definitions:
*     fUnit - FORTRAN unit number for input.
*     LogCount - sequence number for current constraint.
*     Code - boundary condition code.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'logcnst.inc'

*   Local Variables:
      CHARACTER*3 TheEnd

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Nov   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      
      ReadLogStreamConstraint = .FALSE.

      ID( LogCount ) = Code
      READ(fUnit,*)  a( LogCount )
      READ(fUnit,*)  b( LogCount )
      READ(fUnit,*) Zo( LogCount )


 100  CONTINUE
      READ(fUnit,'(A3)',END=200) TheEnd
      
      IF( TheEnd .EQ. 'end' ) THEN
      ELSE
*       WRITE(*,*) ' ...(ReadLogStreamConstraint),',
*    #             'skipping for "end"...'
*       WRITE(*,*) ' ...read "',TheEnd,'"'
        GO TO 100
      END IF
      
      GO TO 300
      
 200  CONTINUE
        WRITE(*,*) ' ####Error(ReadLogStreamConstraint)'
        WRITE(*,*) ' Failed to successfully read log-constraint data...'
        WRITE(*,*) ' for code...',Code
        WRITE(*,*) ' ',a( LogCount )
        WRITE(*,*) ' ',b( LogCount )
        WRITE(*,'(A4)') TheEnd
        RETURN

 300  CONTINUE        

      ReadLogStreamConstraint = .TRUE.

      RETURN
      END



*== Private (GetLogStreamConstraintData) ===============================

      LOGICAL FUNCTION GetLogStreamConstraintData(
     #                                      Code, sa, sb, sZo
     #                                               )

      IMPLICIT NONE

*   Purpose:  Get data for log or semi-log constraint between discharge
*             and water-surface elevation.

*   Arguments:

*       (input)
      INTEGER Code
      
*       (output)
      REAL sa, sb, sZo

*   Argument definitions:
*     Code - current boundary-condition code.
*     sa   - coefficient, intercept.
*     sb   - coefficient, slope.
*     sZo  - water-surface elevation for zero flow.
*     

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'logcnst.inc'

*   Local Variables:
      INTEGER i, j

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Nov   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      GetLogStreamConstraintData = .FALSE.


*---- Determine which constraint.

      DO 100 j=1,2*MaxChannels
        IF( Code .EQ. ID(j) ) THEN
          i = j
          GO TO 102
        END IF
100   CONTINUE
      WRITE(*,*) ' ####Error(GetLogStreamConstraintData)'
      WRITE(*,*) ' Could not find boundary-condition code...',Code
      RETURN
102   CONTINUE

      sa  = a(i)
      sb  = b(i)
      sZo = Zo(i)

      GetLogStreamConstraintData = .TRUE.

      RETURN
      END



*== Private (ZLogStreamConstraint) ==================================

      LOGICAL FUNCTION ZLogStreamConstraint()

      IMPLICIT NONE

*   Purpose:  Compute solution coefficients constraining
*             water-surface elevation to the
*             simple relation:
*
*             Z = Zo + a * Q ** b,
*
*             where Q  = discharge,
*                   Z  = water-surface elevation, and
*                   Zo = reference datum.



*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'strmcnst.inc'

*   Local Variables:
      REAL a, b, Zo
      REAL Depth, Q
      REAL ExpectedH, Tolerance, Error
      PARAMETER ( Tolerance = 0.02 )

*   Routines by module:

***** Local:
      LOGICAL   GetLogStreamConstraintData
      EXTERNAL  GetLogStreamConstraintData

*   Intrinsics:
      REAL      ABS, SIGN
      INTRINSIC ABS, SIGN

*   Programmed by: Lew DeLong
*   Date:          Nov   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      ZLogStreamConstraint = .FALSE.

*---- Get required data.

*     Note: All arguments in this function except the first are
*           returned only.  Only the first argument must be set
*           prior to call.  The first argument was previously 
*           set by a call to SetStreamConstraintVariables.

      IF( GetLogStreamConstraintData(
     #                   ConditionCode, a, b, Zo
     #                                  )    ) THEN
      ELSE
        RETURN
      END IF

      Depth = WSElev(1) - Zo
      Q = ABS( Discharge(1) )
      

*---- Compute coefficients.

      ConstraintCoef(1) = - SIGN( (a * b * Q ** (b-1.0)), Discharge(1) )   
*     ConstraintCoef(1) = - a * b * Q ** (b-1.0)  
      ConstraintCoef(2) = 1.0
      ConstraintRightSide = a * Q ** b - Depth

*---- Check to see if current values do satisfy the prescribed relation.

      ExpectedH = ABS( a * Q ** b )
      Error = Depth - ExpectedH
      
      IF( ABS(Error) .LE. Tolerance ) THEN
      ELSE
      
        WRITE(*,*) ' '
        WRITE(*,*) ' ####Warning( ZLogStreamConstraint )'
        WRITE(*,*) ' Boundary-condition code...', ConditionCode
        WRITE(*,*) ' Expected depth, current depth...', ExpectedH, Depth
        WRITE(*,*) ' Depth error...', Error
      
      END IF

      

      ZLogStreamConstraint = .TRUE.

      RETURN
      END



*== Private (QLogStreamConstraint) ==================================

      LOGICAL FUNCTION QLogStreamConstraint()

      IMPLICIT NONE

*   Purpose:  Compute solution coefficients constraining
*             discharge to the
*             simple relation:
*
*             Q = a * ( Z - Zo ) ** b,
*
*             where Q  = discharge,
*                   Z  = water-surface elevation, and
*                   Zo = reference datum.



*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'strmcnst.inc'

*   Local Variables:
      REAL a, b, Zo
      REAL Depth, Q

*   Routines by module:

***** Local:
      LOGICAL   GetLogStreamConstraintData
      EXTERNAL  GetLogStreamConstraintData

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Nov   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      QLogStreamConstraint = .FALSE.

*---- Get required data.

*     Note: All arguments in this function except the first are
*           returned only.  Only the first argument must be set
*           prior to call.  The first argument was previously 
*           set by a call to SetStreamConstraintVariables.

      IF( GetLogStreamConstraintData(
     #                   ConditionCode, a, b, Zo
     #                                  )    ) THEN
      ELSE
        RETURN
      END IF

      Depth = WSElev(1) - Zo
      Q =  Discharge(1)
      
      
      IF( Depth .GT. 0.0 ) THEN
      
*------ there is flow.


*------ Compute coefficients.

        ConstraintCoef(1) = 1.0  
        ConstraintCoef(2) = - a * b * Depth ** ( b - 1.0 )
        ConstraintRightSide = a * Depth ** b - Q
        
      ELSE
      
*------ there is no flow.

        ConstraintCoef(1) = 1.0  
        ConstraintCoef(2) = 0.0
        ConstraintRightSide = 0.0 - Q
      
      END IF
      

      QLogStreamConstraint = .TRUE.

      RETURN
      END



*== Private (ZSemiLogStreamConstraint) ==================================

      LOGICAL FUNCTION ZSemiLogStreamConstraint()

      IMPLICIT NONE

*   Purpose:  Compute solution coefficients constraining
*             discharge and water-surface elevation to the
*             simple relation:
*
*             Z = Zo + a + b * LOG10 ( Q ),
*
*             where Q  = discharge,
*                   Z  = water-surface elevation, and
*                   Zo = reference datum.



*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'strmcnst.inc'

*   Local Variables:
      REAL a, b, Zo
      REAL Depth, Q
      REAL ExpectedH, Tolerance, Error
      PARAMETER ( Tolerance = 0.02 )

*   Routines by module:

***** Local:
      LOGICAL   GetLogStreamConstraintData
      EXTERNAL  GetLogStreamConstraintData

*   Intrinsics:
      REAL      LOG, LOG10, ABS, SIGN
      INTRINSIC LOG, LOG10, ABS, SIGN

*   Programmed by: Lew DeLong
*   Date:          Nov   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      ZSemiLogStreamConstraint = .FALSE.

*---- Get required data.

*     Note: All arguments in this function except the first are
*           returned only.  Only the first argument must be set
*           prior to call.  The first argument was previously 
*           set by a call to SetStreamConstraintVariables.

      IF( GetLogStreamConstraintData(
     #                   ConditionCode, a, b, Zo
     #                                  )    ) THEN
      ELSE
        RETURN
      END IF


*---- Check for zero discharge.

      Depth = WSElev(1) - Zo
      Q = ABS( Discharge(1) )
      
      IF( Q .GT. 1.0E-06 ) THEN
      ELSE
        
        WRITE(*,*) ' ####Error( ZSemiLogStreamConstraint )'
        WRITE(*,*) ' near zero discharge, undefined relation...'
        RETURN
        
      END IF

*---- Compute coefficients.

      ConstraintCoef(1) =  - SIGN( b, Discharge(1) )

      ConstraintCoef(2) = Q * LOG(10.0)      
      
      ConstraintRightSide = ConstraintCoef(2) * (
     #                              a + b * LOG10( Q ) - Depth
     #                                          )

*---- Check to see if current values satisfy the prescribed relation.

      ExpectedH = ABS( a + b * LOG10( Q ) - Depth )
      Error = ABS( ( Depth - ExpectedH )  )
      IF( Error .LE. Tolerance ) THEN
      ELSE
      
        WRITE(*,*) ' '
        WRITE(*,*) ' ####Warning( ZSemiLogStreamConstraint )'
        WRITE(*,*) ' Boundary-condition code...', ConditionCode
        WRITE(*,*) ' Expected depth, current depth...', ExpectedH, Depth
        WRITE(*,*) ' Error...', Error
      
      END IF
      

      ZSemiLogStreamConstraint = .TRUE.

      RETURN
      END



*==== EOF chcnstrt =====================================================
