*===== BOF chcxtbl.inc ================================================

*   Note: If used, this include must appear after "Network.inc".


      INTEGER     MaxTables, MaxLinesPerTable, MaxLines
      PARAMETER ( MaxTables = 200,
     #            MaxLinesPerTable = 21,
     #            MaxLines = MaxLinesPerTable*MaxTables )
      
      INTEGER     FirstTable(MaxChannels), LastTable(MaxChannels)
      INTEGER     Lines(MaxChannels), Offset(MaxTables)
      REAL        XDistance(MaxTables), Datum(MaxTables)
      REAL        Depth(MaxLines)
      REAL        Width(MaxLines), A(MaxLines), K(MaxLines)
      REAL        P(MaxLines), N(MaxLines)
      REAL        Bta(MaxLines), MA(MaxLines), MQ(MaxLines)
      LOGICAL     Print, WriteIntrp, HermiteBtm
      COMMON   /HYDPRP/ XDistance, Datum, Depth, Width, A, K, P, N,
     #                  Bta, MA,MQ, Print,
     #                  WriteIntrp, HermiteBtm
      COMMON   /HYDPT/ FirstTable, LastTable, Lines, Offset
C
      INTEGER N1, N2, NP(4), PreviousBranch, PreviousX
      INTEGER DegreesOfFreedom, PreviousH
      REAL    Xi, Eta, dH, Shape(4), dShapeDX(4)
      COMMON /CMNFCT/ Xi, Eta, dH, Shape, dShapeDX, N1, N2, NP,
     #                PreviousBranch, PreviousX, PreviousH,
     #                DegreesOfFreedom
      SAVE /HYDPRP/,/HYDPT/,/CMNFCT/


      CHARACTER*16 ID(MaxTables)
      COMMON /CXiD/ ID
      SAVE   /CXiD/


*
*     MaxTables        - maximum number of tables.
*     MaxLinesPerTable - maximum number of lines per table.
*     MaxLines         - maximum total number of lines in tables.
*
*     FirstTable(m) - table number of first table for a branch.
*     LastTable(m)  - table number of last table of a branch.
*     Lines(m) - number of lines per table for the curren branch.
*     m - branch number.
*
*     Offset(i) - pointer to first line of a table in properties arrays.
*     XDistance(i) - downstream-distance coordinate.
*     Datum(i) - elevation of bottom of the channel.
*     i - table number, same as user cross-section sequence number.
*
*     Depth(j) - depth of flow.
*     Width(j) - width of channel.
*     A(j)     - cross-sectional area.
*     K(j)     - sinuosity-weighted conveyance.
*     P(j)     - wetted perimeter.
*     N(j)     - one over effective Manning's "n".
*     Bta(j)   - momentum coefficient.
*     MA(j)    - area-weighted sinuosity.
*     MQ(j)    - discharge-weighted sinuosity.
*     j - line number.    
*
*     N1 - adjacent upstream cross section. 
*     N2 - adjacent downstream cross section.
*
*     NP(k)       - pointers to table positions in adjacent upstream and downstream
*                   property tables bracketing the estimation point.  Order is
*                   counter clockwise viewing the stream profile with upstream
*                   cross section on the left, downstream cross section on the
*                   right, and beginning with the lower of two bracketing points
*                   in the upstream cross section.
*     Shape(k)    - local bilinear shape function for the estimation point.
*     dShapeDX(k) - derivative of Shape with respect to the local depth coordinate.
*     k = degree of freedom.

*     DegreesOfFreedom - maximum number of degrees of freedom,
*                        four for bilinear interpolation.
*
*     Xi - stream-wise local coordinate of the estimation point.
*     Eta - depth-wise local coordinate of the estimation point.
*     dH - global depth corresponding to a local depth of 1 at the
*          estimation point.
*
*     PreviousBranch - branch of previous estimation point.
*     PreviousX      - int( X location ) at previous estimation point.
*     PreviousH      - Depth of flow at previous estimation point.

*     WriteIntrp - index, if .TRUE., write interpolated properties
*                  tables at computational locations.
*     HermiteBtm - index, if .TRUE., interpolate bottom elevation
*                  using hermites.

*===== BOF chcxtbl.inc ================================================
