*==== BOF master ==============================================================
*******************************************************************************
*
*     MASTER FILE MODULE: Initialize the master file and return user or default
*       file names and FORTRAN unit numbers from the master file.
*
C== Public (OpenMaster) =======================================================

      LOGICAL   FUNCTION OpenMaster ()

C   Purpose:  This routine confirms the existence of a master file.  If it
C               exists, then open it and parse the contents for matches to the
C               default list of files (instances of files).

      IMPLICIT NONE

C   Module data:

      INCLUDE       'master.inc'

C   Local variables:

      INTEGER       BufferLength
      PARAMETER     (BufferLength = 80)

      INTEGER       Hit, IOStatus
      LOGICAL       Done
      CHARACTER*80  MasterBuffer
      CHARACTER*12  TargetName, FileName, UnitNumber


C   Functions:

      INTEGER       SearchForFile
      EXTERNAL      SearchForFile

      INTEGER*4     StrToL
      EXTERNAL      StrToL

      LOGICAL       OpenOldText
      EXTERNAL      OpenOldText

      CHARACTER*12  ParseBuffer
      EXTERNAL      ParseBuffer

 1000 FORMAT(' Warning(MasterInit): ', 3A)
 1001 FORMAT(' Error(MasterInit): ', 3A)


C   Programmed by:  Dave Thompson
C   Date:           Tue  12-11-1990
C   Modified by:
C   Last modified:
*

*** TTD:  Make this routine skip any blank lines it finds.

C-- Implementation ------------------------------------------------------------

      IF (MasterInit) THEN

        OpenMaster = .TRUE.
        WRITE (*, 1000) 'Master file already initialized.'

      ELSE

        OpenMaster = .TRUE.

C------ Open the master file, if it exists.

        IF (.NOT. OpenOldText(MasterUnit, MasterFileName)) THEN

          WRITE (*, 1000) 'Master file does not exist, using defaults.'
          WRITE (*,*) ' '

        ELSE
        
          Done = .FALSE.

   10     CONTINUE

C---------- Read the master file, and associate file instances with user
C           specified names appearing in the master file.

            READ (MasterUnit, '(A80)', IOSTAT = IOStatus) MasterBuffer

            IF (IOStatus .NE. 0) THEN

              Done = .TRUE.

            ELSEIF (IOStatus .EQ. 0) THEN

              TargetName = ParseBuffer(MasterBuffer)
              FileName   = ParseBuffer(MasterBuffer)
              UnitNumber = ParseBuffer(MasterBuffer)

*             write (*,*) ' Debug information...'
*             write (*,*) ' TargetName: ', TargetName
*             write (*,*) ' FileName:   ', FileName
*             write (*,*) ' UnitNumber: ', UnitNumber
*             write (*,*) ' '

              Hit = SearchForFile(TargetName)

              IF (Hit .EQ. 0) THEN

                WRITE (*, 1001) 'Target file name, ', TargetName,
     *                          ', not found.'

              ELSE

                UserName(Hit) = FileName
                UserUnit(Hit) = StrToL(UnitNumber)
                FileUsed(Hit) = .TRUE.

              ENDIF

            ENDIF

          IF (.NOT. Done) GOTO 10

        ENDIF
      ENDIF

      RETURN
      END

C== Public (CloseMaster) ======================================================

      LOGICAL   FUNCTION CloseMaster ()

C   Purpose:  This routine closes the master file, and writes to it either
C             the default file names and unit numbers (if the user did not
C             specify any), or writes those file names and unit numbers the
C             user specified.

      IMPLICIT NONE

C   Module data:

      INCLUDE     'master.inc'

C   Local variables:

      INTEGER     I

C   Functions:

      LOGICAL     OpenNewText
      EXTERNAL    OpenNewText

 1000 FORMAT(A, 1X, A, 1X, I4)

C   Programmed by: Dave Thompson
C   Date:          Wed  12-05-1990
C   Modified by:
C   Last modified:
*

C-- Implementation ------------------------------------------------------------

      CLOSE (MasterUnit)

      IF (OpenNewText(MasterUnit, MasterFileName)) THEN

        DO 10 I = 1, MaxFiles
        
          IF (FileUsed(I)) THEN
            WRITE (MasterUnit, 1000) DefaultName(I), UserName(I),
     *                               UserUnit(I)
          ENDIF

   10   CONTINUE

        CloseMaster = .TRUE.

      ELSE

        WRITE (*,*) ' Error(CloseMaster): Master file not opened.'
        WRITE (*,*) ' Master file data lost.'

        CloseMaster = .FALSE.

      ENDIF

      RETURN
      END

C== Public (GetFileUnit) ======================================================

      INTEGER   FUNCTION GetFileUnit (
     I                                Class
     *                               )

C   Purpose:  This routine takes a target file name and searches the list of
C             available class names.  If the name is found, it returns the
C             unit number of the target and marks the file as used.  Otherwise,
C             it returns a 0 to indicate failure.

      IMPLICIT NONE

C   Arguments:

      CHARACTER*12   Class
      
C   Argument definitions:
C     Class - target file name.

C   Module data:

      INCLUDE         'master.inc'

C   Local variables:

      INTEGER         FileNumber

C   Functions:

      INTEGER         SearchForFile
      EXTERNAL        SearchForFile

C   Data Initializations:


C   Programmed by:  Dave Thompson
C   Date:           Wed  12-12-1990
C   Modified by:
C   Last modified:
*

C-- Implementation ------------------------------------------------------------

      FileNumber = SearchForFile(Class)

      IF (FileNumber .EQ. 0) THEN
        GetFileUnit = 0
      ELSE
        GetFileUnit          = UserUnit(FileNumber)
        FileUsed(FileNumber) = .TRUE.
      ENDIF

      RETURN
      END

C== Public (GetFileName) ======================================================

      CHARACTER*12 FUNCTION GetFileName (
     I                                   Class
     *                                  )

C   Purpose:  Search module data for the file Class and return its value
C             to the client prorgram.

      IMPLICIT NONE

C   Arguments:

      CHARACTER*12 Class

C   Argument definitions:
C     Class - target file name.

C   Module data:

      INCLUDE       'master.inc'

C   Local variables:

      INTEGER       FileNumber

C   Functions:

      INTEGER       SearchForFile
      EXTERNAL      SearchForFile

C   Data Initializations:


C   Programmed by:  Dave Thompson
C   Date:           Wed  12-12-1990
C   Modified by:
C   Last modified:
*

C-- Implementation ------------------------------------------------------------

      FileNumber = SearchForFile(Class)

      IF (FileNumber .EQ. 0) THEN
        GetFileName = ' '
      ELSE
        GetFileName          = UserName(FileNumber)
        FileUsed(FileNumber) = .TRUE.
      ENDIF

      RETURN
      END

C== Private (ParseBuffer) =====================================================

      CHARACTER*12 FUNCTION ParseBuffer (
     M                                   Buffer
     *                                  )

C   Purpose:  Parse the incoming buffer for discrete character strings which
C             don't exceed 12 characters in length.  Extract the substring
C             from the incoming string.

C   Algorithm:  Remove leading blanks from the incoming string.  Then, take
C               characters from the string until either a blank is encountered
C               or twelve characters have been taken from the string.
C               Characters taken from the left end of the buffer are actually
C               removed, shortening the string.

      IMPLICIT NONE

C   Arguments:

      CHARACTER*80   Buffer

C   Module data:


C   Local variables:

      INTEGER         I, BufferLength, FirstBlank, EndWord
      CHARACTER*12    TempBuffer

C   Functions:

      INTEGER         MaxNameLength
      PARAMETER      (MaxNameLength = 12)

      INTEGER         StrLen
      EXTERNAL        StrLen

      CHARACTER*255   LeftTrim
      EXTERNAL        LeftTrim

      INTEGER         INDEX, MIN0
      INTRINSIC       INDEX, MIN0

 1001 FORMAT(' Error(ParseBuffer): ', 3A)

C   Data Initializations:


C   Programmed by:  Dave Thompson
C   Date:           Wed  12-12-1990
C   Modified by:
C   Last modified:
*

C-- Implementation ------------------------------------------------------------

*     write (*,*) 'In ParseBuffer...'
*     write (*,*) 'Before LeftTrim, Buffer =', Buffer
      Buffer        = LeftTrim(Buffer)

      TempBuffer    = ' '
*     write (*,*) 'After LeftTrim, Buffer  =', Buffer

      BufferLength  = StrLen(Buffer)

*     write (*,*) 'Length of incoming buffer: ', BufferLength

      FirstBlank    = INDEX(Buffer, ' ')

      IF (FirstBlank .EQ. 0) THEN

C------ Use entire buffer.

        FirstBlank = BufferLength

      ENDIF

      IF (FirstBlank .GT. MaxNameLength+1) THEN

        WRITE (*, 1001) 'file name too long; using first 12 chars.'

      ENDIF

      EndWord    = MIN0 (FirstBlank-1, MaxNameLength)

C---- Grab the default filename (character*12)

      DO 10 I = 1, EndWord

        TempBuffer(I:I) = Buffer(I:I)

   10 CONTINUE

      IF (BufferLength .GT. EndWord) THEN

C------ Move the remaining characters to the front of the buffer.

        DO 20 I = 1, BufferLength - FirstBlank + 1

          Buffer(I:I) = Buffer(I+FirstBlank-1:I+FirstBlank-1)

   20   CONTINUE

C------ Clear remaining characters.

        DO 30 I = BufferLength - FirstBlank+2, BufferLength

          Buffer(I:I) = ' '

   30   CONTINUE

      ENDIF

*     write (*,*) 'TempBuffer: ', TempBuffer
*     write (*,*) 'Buffer:     ', Buffer

      ParseBuffer = TempBuffer

      RETURN
      END

C== Private (SearchForFile) ===================================================

      INTEGER   FUNCTION SearchForFile (
     I                                  TargetName
     *                                 )

C   Purpose:  Search the default (standard) file names (module data) for the
C             occurrence of TargetName.

      IMPLICIT NONE

C   Arguments:

      CHARACTER*12 TargetName


C   Module data:

      INCLUDE       'master.inc'


C   Local variables:

      INTEGER     FileNumber
      LOGICAL     Done

C   Programmed by: Dave Thompson
C   Date:          Tue  12-11-1990
C   Modified by:
C   Last modified:
*

C-- Implementation ------------------------------------------------------------

      FileNumber    = 0
      Done          = .FALSE.
      SearchForFile = 0

   10 FileNumber = FileNumber + 1
      
        IF (TargetName .EQ. DefaultName(FileNumber)) THEN
          SearchForFile = FileNumber
          Done = .TRUE.
        ENDIF

      IF (FileNumber .LT. MaxFiles .AND. .NOT. Done) GOTO 10
      
      RETURN
      END

C== Private (MasterModule) ====================================================

      BLOCK DATA MasterModule

C   Purpose:  Initialize module data.

      IMPLICIT NONE

C   Module data:

      INCLUDE       'master.inc'

C   Data initializations:

      DATA MasterFileName / 'master.fil  ' /
      DATA MasterInit  /.FALSE. /
      DATA MasterUnit  / 50 /
      DATA FileUsed    / MaxFiles * .FALSE. /

C-- Ruler -------------- 123456789012 -- 123456789012
      DATA DefaultName /'intpgeom.dat', 'netspc1.out ',
     1                  'notused2.dat', 'notused3.dat',
     2                  'nettran.in  ', 'nettran.out ',
     3                  'control.dat ', 'schemat.dat ',
     4                  'rawgeom.dat ', 'cxgeom.dat  ',
     5                  'netspcz.dat ', 'netbnd.dat  ',
     6                  'netts.dat   ', 'tables30.dat',
     7                  'initcond.dat', 'temp.txt    ',
     8                  'perturb.dat ', 'bndval.dat  ',
     9                  'neterror.dat', 'netspcq.dat ',
     *                  'netprint.dat', 'nettsq.dat  ',
     1                  'nettsz.dat  ', 'density.dat ',
     2                  'strmcnst.dat', 'netrstrt.dat'/
*
      DATA DefaultUnit / 51, 52, 53, 54, 55, 56, 57, 58, 59, 30,
     1                   31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
     2                   41, 42, 43, 44, 45, 46
     *                 /
C-- Ruler -------------- 123456789012 -- 123456789012
      DATA UserName    /'intpgeom.dat', 'netspc1.out ',
     1                  'notused2.dat', 'notused3.dat',
     2                  'nettran.in  ', 'nettran.out ',
     3                  'control.dat ', 'schemat.dat ',
     4                  'rawgeom.dat ', 'cxgeom.dat  ',
     5                  'netspcz.dat ', 'netbnd.dat  ',
     6                  'netts.dat   ', 'tables30.dat',
     7                  'initcond.dat', 'temp.txt    ',
     8                  'perturb.dat ', 'bndval.dat  ',
     9                  'neterror.dat', 'netspcq.dat ',
     *                  'netprint.dat', 'nettsq.dat  ',
     1                  'nettsz.dat  ', 'density.dat ', 
     2                  'strmcnst.dat', 'netrstrt.dat'/

*
      DATA UserUnit    / 51, 52, 53, 54, 55, 56, 57, 58, 59, 30,
     1                   31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
     2                   41, 42, 43, 44, 45, 46
     *                 /


C   Programmed by:  Dave Thompson
C   Date:           Wed  12-12-1990
C   Modified by:    Lew DeLong
C   Last modified:  Oct   1992
*

      END

*==== EOF master ==============================================================
