*== Public (InitializeNetworkBoundaryVal) ================================================

      LOGICAL FUNCTION InitializeNetworkBoundaryVal()

      IMPLICIT NONE

*   Purpose:  Initialize network boundary-value module , the purpose of
*             which is to get user-supplied time-dependent values of
*             variables at boundaries of network.  In this implementation,
*             values are linearly interpolated from values read from a file
*             supplied by user.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netbnd.inc'

*   Local Variables:
      INTEGER I
      CHARACTER*12 FileName, InternalFileName

*   Routines by module:

***** Network control:
      INTEGER  NetworkPrintLevel, NetworkSeconds
      EXTERNAL NetworkPrintLevel, NetworkSeconds

***** Master file:
        INTEGER GetFileUnit
        CHARACTER*12 GetFileName
        EXTERNAL GetFileName, GetFileUnit

***** File utilities:
        LOGICAL  OpenOldText
        EXTERNAL OpenOldText

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:   Lew DeLong
*   Last modified: Aug    1992
*

*-- Implementation -----------------------------------------------------

      InternalFileName = 'netbnd.dat  '
      
      IF( ReadBoundaryValues ) THEN

        NetBndValUnit = GetFileUnit(InternalFileName)
        FileName = GetFileName(InternalFileName)

        IF(OpenOldText(NetBndValUnit,FileName)) THEN
          READ(NetBndValUnit,*) StartSeconds
          READ(NetBndValUnit,*) dT
          READ(NetBndValUnit,*) BndValues
          IF( NetworkPrintLevel() .GT. 0 ) THEN
            WRITE(*,*) ' Boundary values read from...',FileName
            WRITE(*,*) ' Boundary-data starting elapse time...',
     #                   StartSeconds
            WRITE(*,*) ' Boundary-data time increment...', dT
            WRITE(*,*) ' Number of values each read...',BndValues
          END IF
          
*-------- Check for compatibility with Network starting time.
          
          IF( StartSeconds .GT. NetworkSeconds() ) THEN
            WRITE(*,*) ' ####error(InitializeNetworkBoundarVal)'
            WRITE(*,*) ' Network starting time preceeds that of',
     #                 ' boundary values...'
            WRITE(*,*) ' Abnormal program end.'
            STOP
          END IF
          
*-------- Set old and new boundary values.

          OldTime = StartSeconds
          DO 100 I=1,BndValues
            READ(NetBndValUnit,*,END = 200) Channel(I)
 100      CONTINUE

          DO 120 I=1,BndValues
            READ(NetBndValUnit,*,END = 200) Old(I)
 120      CONTINUE

          NewTime = OldTime + dT
          DO 150 I=1,BndValues
            READ(NetBndValUnit,*,END = 200) New(I)
 150      CONTINUE

          GO TO 202
 200      CONTINUE
          WRITE(*,*) ' ####error(InitializeNetworkBoundaryVal)'
          WRITE(*,*) ' End of file while reading boundary data...'
          WRITE(*,*) ' file...',FileName
          WRITE(*,*) ' Abnormal program end.'
          STOP
 202      CONTINUE
          
        ELSE
          IF( NetworkPrintLevel() .GT. 0 ) THEN
            WRITE(*,*) ' #####Warning...'
            WRITE(*,*) ' Boundary values will not be read from',
     #                 ' a time-series file...'
            WRITE(*,*) ' File...', FileName,' not opened...'
          END IF
          BndValues = 0
          ReadBoundaryValues = .FALSE.
        END IF
        
      END IF
      
      
      InitializeNetworkBoundaryVal = .TRUE.

      RETURN
      END



*== Private (ReadNetworkBoundaryValues) ==================================

      LOGICAL FUNCTION ReadNetworkBoundaryValues()

      IMPLICIT NONE

*   Purpose:  Read network boundary values to be used for the end
*             of the current time step.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netbnd.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      
        OldTime = NewTime
        NewTime = NewTime + dT

        DO 100 I=1,BndValues

          Old(I) = New(I)
          READ(NetBndValUnit,*,END=200) New(I)

          
100     CONTINUE
        
        GO TO 202
200     CONTINUE
        WRITE(*,*) ' ####error(ReadNetworkBoundaryValues)'
        WRITE(*,*) ' End of file while reading boundary data.'
        WRITE(*,*) ' Abnormal program end.'
        STOP
202     CONTINUE
        
      
      ReadNetworkBoundaryValues = .TRUE.

      RETURN
      END



*== Private (AssignNetworkBoundaryValues) ==================================

      LOGICAL FUNCTION AssignNetworkBoundaryValues()

      IMPLICIT NONE

*   Purpose:  Assign network boundary values to be used for the end
*             of the current time step.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netbnd.inc'

*   Local Variables:
      INTEGER I, J

*   Routines by module:

***** Local:

*   Intrinsics:
      INTEGER   IABS
      INTRINSIC IABS

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      
        DO 100 I=1,BndValues

          J = Channel(I)

          IF(  J .NE. 0 
     #           .AND.
     #         IABS(J) .LE. NumCh  ) THEN

*-------- Valid channel number.

            IF( J .GT. 0 ) THEN

*------------ Upstream end of channel.

              StreamBndValue( 2*J-1 ) = Current(I)
              
            ELSE
            
*------------ Downstream end of channel.

              StreamBndValue( -2*J ) = Current(I)
              
            END IF

          ELSE

*-------- Invalid channel number.

            WRITE(*,*) ' ######Error (AssignNetworkBoundaryValues)'
            WRITE(*,*) ' Channel ',IABS(J),' does not exist...'
          END IF
          
100     CONTINUE
        
      
      AssignNetworkBoundaryValues = .TRUE.

      RETURN
      END



*== Public (SetUserNetBndValues) ================================================

      LOGICAL FUNCTION SetUserNetBndValues()

      IMPLICIT NONE

*   Purpose:  Set current values of user-supplied boundary values.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netbnd.inc'

*   Local Variables:
      INTEGER I
      REAL Shape1, Shape2
      LOGICAL OK

*   Routines by module:

***** Local:
      LOGICAL  ReadNetworkBoundaryValues
      LOGICAL  AssignNetworkBoundaryValues
      EXTERNAL ReadNetworkBoundaryValues
      EXTERNAL AssignNetworkBoundaryValues
      
***** Network control:
      INTEGER  NetworkSeconds
      EXTERNAL NetworkSeconds

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Aug   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      CurrentTime = NetworkSeconds()

 100  CONTINUE

      IF( ReadBoundaryValues ) THEN
      IF( CurrentTime .GT. OldTime ) THEN
        IF( CurrentTime .LT. NewTime ) THEN

*-------- Interpolate current values.

          Shape1 = FLOAT(NewTime - CurrentTime)/
     #             FLOAT(NewTime - OldTime)
          Shape2 = 1.0 - Shape1
          
          DO 200 I=1,BndValues
            Current(I) = Shape1*Old(I) + Shape2*New(I)
 200      CONTINUE

        ELSE IF( CurrentTime .EQ. NewTime ) THEN

*-------- Assign current values.

          DO 300 I=1,BndValues
            Current(I) = New(I)
 300      CONTINUE

        ELSE
        
*-------- Read new values.
          
          OK = ReadNetworkBoundaryValues()

          GO TO 100

        END IF
      ELSE IF(CurrentTime .EQ. OldTime ) THEN
        
*------ Read new values.

        OK = ReadNetworkBoundaryValues()

        GO TO 100
        
      ELSE
      
*------ Shouldn't ever get here!

        WRITE(*,*) ' ####error(SetUserNetBndValues)'
        WRITE(*,*) ' Current time < available from boundary values.'
        WRITE(*,*) ' Abnormal program end.'
        STOP
      END IF
      END IF
      
      OK = AssignNetworkBoundaryValues()
      
     
      SetUserNetBndValues = .TRUE.

      RETURN
      END



*== Public (SetUserNetBndValueIndex) ================================================

      LOGICAL FUNCTION SetUserNetBndValueIndex(N)

      IMPLICIT NONE

*   Purpose:  Set UserBndValueIndex indicating if boundary values
*             are to be supplied by user.

*   Arguments:
      INTEGER N

*   Argument definitions:
*     N - index indicating values
*          [0] - will not be supplied by user, or
*          [1] - will be supplied by user.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netbnd.inc'

*   Local Variables:

*   Routines by module:

***** Local:

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          Aug   1992
*   Modified by:   
*   Last modified: 
*

*-- Implementation -----------------------------------------------------

      IF( N .EQ. 1 ) THEN
        ReadBoundaryValues = .TRUE.
      ELSE
        ReadBoundaryValues = .FALSE.
      END IF

      SetUserNetBndValueIndex = .TRUE.

      RETURN
      END




*== Public (UpstreamBoundaryValue) =====================================

      REAL FUNCTION UpstreamBoundaryValue()

      IMPLICIT NONE

*   Purpose:  Return a boundary value, computed from a simple boundary
*             equation, for the upstream end of the current channel.


*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netbnd.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      UpstreamBoundaryValue = StreamBndValue( 2*Branch - 1)

      RETURN
      END



*== Public (DownstreamBoundaryValue) ===================================

      REAL FUNCTION DownstreamBoundaryValue()

      IMPLICIT NONE

*   Purpose:  Return a boundary value, computed from a simple boundary
*             equation, for the downstream end of the current channel.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netbnd.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      DownstreamBoundaryValue = StreamBndValue( 2*Branch )

      RETURN
      END




*== Public (SetNetworkBoundaryValue) =====================================

      LOGICAL FUNCTION SetNetworkBoundaryValue(I,Value)

      IMPLICIT NONE

*   Purpose:  Set a boundary value for the upstream end
*             of current channel.


*   Arguments:
      INTEGER I
      REAL Value

*   Argument definitions:
*     I - channel number, + upstream, - downstream.
*     Value - boundary value to be set.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netbnd.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

            IF( I .GT. 0 ) THEN

*------------ upstream end of channel.
              StreamBndValue( 2 * I - 1 ) = Value
              

            ELSE IF( I .LT. 0 ) THEN

*------------ downstream end of channel.
              StreamBndValue( - 2 * I ) = Value

            ELSE
              WRITE(*,*) ' *** Error (SetNetworkBoundaryValue)'
              WRITE(*,*) ' Channel number = ',I
              WRITE(*,*) ' Abnormal program end.'
              STOP
            END IF
            
            SetNetworkBoundaryValue = .TRUE.

      RETURN
      END
