*==== BOF netcntr2 =====================================================


*   Module data:
*
*    'network.inc'
*     MaxChannels - maximum number of channels.
*     NumCh - current number of channels.
*     Branch - current selected or active channel.
*     MaxLocations - maximum number of locations (computational or user).
*
*
*
*    'netcntrl.inc'
*     MaxQuadPts - maximum number of quadrature points allowed
*                  for numerical spatial integration between
*                  adjacent computational cross sections.
*     MaxBndEq - maximum number of equation-type boundaries.
*     MaxTS - Maximum number of locations at which timeseries
*             output may be accumulated.

*     Terms - index indicating form of 1-D flow equations.
*              [1] full dynamic-wave equation.
*              [2] diffusion-wave equation.
*              [3] kinematic-wave equation.
*
*     VariableSinuosity - .TRUE. if allowed to vary, .FALSE. otherwise.
*     VariableDensity - .TRUE. if allowed to vary, .FALSE. otherwise.
*     Perturbation - .TRUE. if perturbation of input is requested,
*                    otherwise .FALSE. .
*
*     Gravity - acceleration due to gravity.
*
*     Seed - seed for random-number generator.
*
*     MaxTimeSteps - maximum number of time steps allowed.
*     TimeStep - current time step.
*     MaxIterations - Maximum number of iterations per time step.
*     Iteration - current iteration number.
*     TotalIterations - total iterations.
*     LuInc - interval of complete forward eliminations, in terms of
*             number of iterations during a time step.
*
*     PrintLevel - level of printing activity, 0 to 9, activity
*                   increasing with increasing number.
*     PrintCount - counter for printed output.
*     PrintInc - increment, in time steps, for printed output.
*     TimeSeriesCount - counter for time-series output.
*     TimeSeriesInc - increment, in time steps, for time-series output.
*     SpatialSeriesCount - counter for spatial-series output.
*     SpatialSeriesInc - increment for spatial-series output.
*
*     TimeSeriesLocation(i) - global computational location sequence number
*                              for the "i"th time series.
*     TSChannel(i) - channel number coreesponding to TimeSeriesLocation(i).
*     TSX(i) - downstream distance corresponding to TimeSeriesLocation(i).
*
*     TimeSeries - total number of time series requested.
*
*
*
*                 Equation-type boundary values ...
*
*       value = EqBase + for each component [
*
*         Amplitude * cos( TwoPI* (Hours+PhaseAngle) / Period )
*
*                                           ]
*
*
*
*             ( time unit for following constants is hours... )
*
*     BoundaryEquations - number of user-supplied boundary equations.
*     EqNumber(k+MaxChannels+1) - equation number for channel k
*      (upstream end of channel k>0, downstream end of channel k<0).
*     EqChannelNumber(j) - channel number, + upsteam, - downstream, equation j.
*     EqComponents(j) - number of harmonic components in equation j.
*     MaxEqComponents - maximum harmonic components per equation.
*     EqBase(j) - base value, equation j.
*     Amplitude1(i,j) - amplitude of component i, equation j.
*     Period(i,j) - period, in hours, of component i, equation j.
*     PhaseAngle(i,j) - phase angle, in hours, of component i, equation j.
*     EqStart(j) - elapse time, in hours, at which equation becomes effective.
*     EqEnd(j) - elapse time, in hours, time at which equation is no longer effective.
*     j - sequential storage index.
*     StreamBndValue(k) - current value computed from equation, possibly
*                         one upstream and one downstream value,
*                         sequentially, odd "k" are upstream,
*                         even are downstream.
*
*
*
*          Standard deviations of normal distributions of errors, used to
*          perturb boundary-equation values, follow. Perturbation will be active
*          when the file "perturb.dat" exists and may be opened and read
*          successfully.  Standard deviations are initialized to zero.
*          Non-zero values are read from file "perturb.dat" if it exists.
*
*
*          group 1: applied each time step and held constant over a time step.
*
*     EqBaseStdDev(j) - standard deviation of a random normal distribution
*                          of errors in base value for equation j.
*     AmplitudeStdDev(j,i) - standard deviation of a random normal distribution
*                          of errors in amplitude for component i, equation j.
*     PeriodStdDev(j,i) - standard deviation of a random normal distribution
*                          of errors in period for component i, equation j.
*     PhaseAngleStdDev(j,i) - standard deviation of a random normal distribution
*                          of errors in phase angle for component i, equation j.
*
*          group 2: applied only once, at the beginning of a model run,
*                   and held constant through out the entire run.
*
*     EqBaseBias(j,i) - standard deviation of a random normal distribution
*                          of errors in base value for equation j.
*     AmplitudeBias(j,i) - standard deviation of a random normal distribution
*                          of errors in amplitude for component i, equation j.
*     PeriodBias(j,i) - standard deviation of a random normal distribution
*                          of errors in period for component i, equation j.
*     PhaseAngleBias(j,i) - standard deviation of a random normal distribution
*                          of errors in phase angle for component i, equation j.
*
*
*
*     DT - time increment, in seconds.
*     Theta - time-weighting factor.
*     ToleranceQ - closure tolerance for discharge.
*     ToleranceZ - closure tolerance for water-surface elevation.
*     RatioQ - current maximum ratio of change in Q to prior Q (flow).
*     ChangeZ - current maximum change in Z (water-surface elevation).
*     ChannelRatioQ - channel in which RatioQ occurs.
*     NodeRatioQ - location within ChannelRatioQ that RatioQ occurs.
*     ChannelChangeZ - channel in which ChangeZ occurs.
*     NodeChangeZ - location within ChannelChangeZ that ChangeZ occurs.
*
*     CloseQ - .TRUE. if closure criteria for discharge has been
*                determined to currently be satisfied, .FALSE. otherwise.
*     CloseZ - .TRUE. if closure criteria for ws_elev has been determined
*                to currently be satisfied, .FALSE. otherwise.
*     EndIteration - .TRUE. if it has determined that iterations are to
*                      stop, .FALSE. otherwise.
*
*
*     MaxQuadPts - Maximum number of quadrature points.
*     QuadPts - current number of quadrature points.
*     QuadPt(i) - location of quadrature point "i",
*                  in local coordinate ( 0 to 1 ).
*     QuadWt(i) - weight, corresponding to quadrature point "i",
*                  for numerical integration.
*
*     Seconds - elapse time, in seconds




*== Public (SetNetworkBoundaryEqValues) ================================

      LOGICAL FUNCTION SetNetworkBoundaryEqValues()

      IMPLICIT NONE

*   Purpose:  Compute boundary equation values.

*
*                 Equation-type boundary values ...
*
*       value = EqBase + for each component [
*
*         Amplitude * cos( TwoPI* (Hours+PhaseAngle) / Period )
*
*                                           ]
*

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
      INTEGER I, ChannelNumber, K
      REAL    TwoPI, Amp, Per, Phs, Hours
      REAL    AmplitudeError, PeriodError, PhaseError
      REAL    Value
      PARAMETER ( TwoPI = 2.0*3.14159 )
      LOGICAL OK

*   Routines by module:

***** Local:
      REAL     GAUSSF
      EXTERNAL GAUSSF

***** Channel schematic:
      INTEGER  UpstreamPointer, DownstreamPointer
      EXTERNAL UpstreamPointer, DownstreamPointer

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel
      
***** Network boundary values:
      LOGICAL  SetNetworkBoundaryValue
      EXTERNAL SetNetworkBoundaryValue

*   Intrinsics:
      INTEGER   IABS
      REAL      COS, FLOAT
      INTRINSIC IABS, COS, FLOAT

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: October  1991
*


*-- Implementation -----------------------------------------------------

      SetNetworkBoundaryEqValues = .FALSE.

      Hours = FLOAT(Seconds) / 3600.0
      
      IF( BoundaryEquations .GT. 0 ) THEN
        DO 100 I=1,BoundaryEquations

          ChannelNumber = IABS( EqChannelNumber(I) )

          IF( OpenChannel( ChannelNumber ) ) THEN



            IF( EqEnd(I) .GE. Hours 
     #                   .AND.
     #        EqStart(I) .LE. Hours ) THEN

*------------ Get random numbers from normal distribution.

              IF( Perturbation ) THEN
                AmplitudeError    = GAUSSF(Seed)
                PeriodError       = GAUSSF(Seed)
                PhaseError        = GAUSSF(Seed)
                Value             = EqBase(I)
     #                            + EqBaseStdDev(I) * GAUSSF(Seed)
              ELSE
                Value = EqBase(I)
              END IF

*------------ finally, compute boundary value.
              
              DO 50 K=1,EqComponents(I)


                IF( Period(K,I) .GT. 1.0E-06 ) THEN

                  Amp   = Amplitude(K,I)
                  Per   = Period(K,I)
                  Phs   = PhaseAngle(K,I)

*---------------- Perturb sinusoidal component.

                  IF( Perturbation ) THEN
                    Amp = Amp+AmplitudeStdDev(K,I) * AmplitudeError
                    Per = Per+PeriodStdDev(K,I)    * PeriodError
                    Phs = Phs+PhaseAngleStdDev(K,I) * PhaseError
                  END IF

*---------------- The equation...

                  Value = Value +
     #                           Amp * COS(
     #                  TwoPI*( Hours + Phs ) / Per
     #                                    )
                END IF

 50           CONTINUE

*------------ Set computed value.

              OK = SetNetworkBoundaryValue( 
     #                        EqChannelNumber(I), Value
     #                                    )


            END IF

          ELSE
            WRITE(*,*) ' *** Error (SetNetworkBoundaryEqValues)'
            WRITE(*,*) ' Channel open prior to calling this routine.'
            WRITE(*,*) ' Abnormal program end.'
            STOP
          END IF

          OK = CloseChannel()

 100    CONTINUE
      END IF

      SetNetworkBoundaryEqValues = .TRUE.

      RETURN
      END






*== Public (VariableStreamDensity) ===================================================

      LOGICAL FUNCTION VariableStreamDensity()

      IMPLICIT NONE

*   Purpose: Return .TRUE. if density is allowed to vary with
*             time and distance, .FALSE. otherwise.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      VariableStreamDensity = VariableDensity

      RETURN
      END



*== Public (VariableStreamSinuosity) ===================================================

      LOGICAL FUNCTION VariableStreamSinuosity()

      IMPLICIT NONE

*   Purpose: Return .TRUE. if sinuosity is allowed to vary with
*             depth of flow and distance, .FALSE. otherwise.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:

*   Routines by module:

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      VariableStreamSinuosity = VariableSinuosity

      RETURN
      END



*== Public (PerturbBoundaryEq) =========================================

      LOGICAL FUNCTION PerturbBoundaryEq()

      IMPLICIT NONE

*   Purpose:  Perturb parameters of boundary equations.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'netcntrl.inc'

*   Local Variables:
        INTEGER fUnit, TempUnit, Iterations, I, J
        INTEGER Eq, ChannelNumber, Components
        REAL    StdDev1, StdDev2
        REAL    AmplitudeError, PhaseAngleError, PeriodError
        CHARACTER*12 FileName, TempFile, InternalFileName1
        CHARACTER*12 InternalFileName2

*   Routines by module:

***** Local:
      REAL     GAUSSF
      EXTERNAL GAUSSF

***** File utilities:
        LOGICAL  OpenOldText, OpenNewText
        EXTERNAL OpenOldText, OpenNewText
        
***** Master File:
        INTEGER GetFileUnit
        CHARACTER*12 GetFileName
        EXTERNAL GetFileName, GetFileUnit

*   Intrinsics:

*   Programmed by: Lew DeLong
*   Date:          October 1991
*   Modified by:   Barry Wicktom (use of master file names added)
*   Last modified: 1/8/92
*

*-- Implementation -----------------------------------------------------
     
      InternalFileName1 = 'temp.txt'
      InternalFileName2 = 'perturb.dat '

      fUnit = GetFileUnit(InternalFileName2)
      FileName = GetFileName(InternalFileName2)

      TempUnit = GetFileUnit(InternalFileName1)
      TempFile = GetFileName(InternalFileName1)

        IF(OpenOldText(fUnit,FileName)) THEN

*-------- Perturbation is active.

*-------- Initialize standard deviations.

          DO 40 I=1,2*MaxChannels
            EqBaseStdDev(I) = 0.0
            EqBaseBias(I) = 0.0
            DO 30 J=1,MaxComponents
              AmplitudeStdDev(J,I)  = 0.0
              AmplitudeBias(J,I)    = 0.0
              PeriodStdDev(J,I)     = 0.0
              PeriodBias(J,I)       = 0.0
              PhaseAngleStdDev(J,I) = 0.0
              PhaseAngleBias(J,I)   = 0.0
 30         CONTINUE
 40       CONTINUE

          WRITE(*,*) ' '
          WRITE(*,*) ' Perturbation of boundary equations is active...'
          WRITE(*,*) ' Perturbation control read from file...', FileName
          
          Perturbation = .TRUE.
          
*-------- Get seed for random number generator,
*         stop if zero.

          READ( fUnit,*,END=200 ) Seed
          READ( fUnit,*,END=200 ) Iterations
          IF( Seed .EQ. 0 ) THEN
            WRITE(*,*) ' Execution stopped, Seed = 0...'
            STOP
          END IF
          
          Iterations = Iterations + 1
          
          WRITE(*,*) ' Perturbation iteration...',Iterations
          
*-------- Open temporary file.

          IF(OpenNewText(TempUnit,TempFile)) THEN
          ELSE
             WRITE(*,*) ' Temporary deviation file not opened...'
             WRITE(*,*) ' File name : ', TempFile
             STOP
          END IF


*-------- Perturb equation parameters.

          DO 100 I=1,2*MaxChannels
          
            READ(fUnit,*,END=200) ChannelNumber
            WRITE(TempUnit,*) ChannelNumber
            Eq = EqNumber( ChannelNumber + MaxChannels + 1 )
            WRITE(*,*) ' Channel & equation numbers...',ChannelNumber,Eq

            READ(fUnit,*,END=200) Components
            IF( Components .NE. EqComponents(Eq) ) THEN
              WRITE(*,*) ' ####error(PerturbBoundarEq)'
              WRITE(*,*) ' Channel...',ChannelNumber
              WRITE(*,*) ' Mismatch number of components.'
              STOP
            END IF
            WRITE(TempUnit,*) Components

            READ(fUnit,*,END=200) EqBaseBias(Eq), EqBaseStdDev(Eq)
            WRITE(TempUnit,*)     EqBaseBias(Eq), EqBaseStdDev(Eq)
            EqBase(Eq) = EqBase(Eq) + EqBaseBias(Eq) * GAUSSF(Seed)
            WRITE(*,*) ' EqBase.......',EqBase(Eq)
            
            AmplitudeError  = GAUSSF(Seed)
            PeriodError     = GAUSSF(Seed)
            PhaseAngleError = GAUSSF(Seed)

            DO 50 J=1,Components

              READ(fUnit,*,END=200) 
     #             AmplitudeBias(J,Eq), AmplitudeStdDev(J,Eq)
              WRITE(TempUnit,*)
     #             AmplitudeBias(J,Eq), AmplitudeStdDev(J,Eq)
              Amplitude(J,Eq) = Amplitude(J,Eq) 
     #                        + AmplitudeError * AmplitudeBias(J,Eq)

              READ(fUnit,*,END=200)
     #             PeriodBias(J,Eq), PeriodStdDev(J,Eq)
              WRITE(TempUnit,*)
     #             PeriodBias(J,Eq), PeriodStdDev(J,Eq)
              Period(J,Eq) = Period(J,Eq)
     #                     + PeriodBias(J,Eq) * PeriodError

              READ(fUnit,*,END=200)
     #              PhaseAngleBias(J,Eq), PhaseAngleStdDev(J,Eq)
              WRITE(TempUnit,*)
     #              PhaseAngleBias(J,Eq), PhaseAngleStdDev(J,Eq)
              PhaseAngle(J,Eq) = PhaseAngle(J,Eq)
     #                         + PhaseAngleBias(J,Eq) * PhaseAngleError

              WRITE(*,*) ' Period.......',Period(J,Eq)
              WRITE(*,*) ' PhaseAngle...',PhaseAngle(J,Eq)

 50         CONTINUE

 100      CONTINUE

 200      CONTINUE

*-------- Write current seed and deviations to TempFile.

          REWIND(fUnit)
          REWIND(TempUnit)
           
          WRITE(fUnit,*) Seed
          WRITE(fUnit,*) Iterations

          DO 400 I=1,BoundaryEquations
          
            READ(TempUnit,*,END=500) ChannelNumber
            WRITE(fUnit,*) ChannelNumber
            
            READ(TempUnit,*,END=500) Components
            WRITE(fUnit,*) Components

            READ(TempUnit,*) StdDev1, StdDev2
            WRITE(fUnit,*)   StdDev1, StdDev2

            DO 350 J=1,Components

              READ(TempUnit,*)  StdDev1, StdDev2
              WRITE(fUnit,*)    StdDev1, StdDev2

              READ(TempUnit,*)  StdDev1, StdDev2
              WRITE(fUnit,*)    StdDev1, StdDev2

              READ(TempUnit,*)  StdDev1, StdDev2
              WRITE(fUnit,*)    StdDev1, StdDev2

 350        CONTINUE

 400      CONTINUE

 500      CONTINUE
 
          CLOSE(fUnit)
          CLOSE(TempUnit)
          WRITE(*,*) ' '

        ELSE
        
*-------- Equation parameters will not be perturbed.
          WRITE(*,*) ' Note: perturbation file not opened.'
          WRITE(*,*) ' File name : ', FileName
          WRITE(*,*) ' Perturbation not used...'
          
          Perturbation = .FALSE.
   
        END IF

      
      PerturbBoundaryEq = .TRUE.

      RETURN
      END

C==================================================
C==================================================  GAUSSF       ======
      REAL FUNCTION GAUSSF(IRAN)
      
      IMPLICIT NONE
C
C  GAUSSIAN RANDOM NUMBER FCN -  RANDUB/BOX-MULLER
C  REVISION AND EXTENSION OF JIM SLACK'S GAUSSV.    WK 8/75. 4/78. 6/83.
C  USES SINGLE PRECISION AND CALLS ONLY RANDUV, NOT RANDUB.
C     ..................................................................
C
*   ARGUMENTS:
       INTEGER IRAN
       
*   ARGUMENT DEFINITIONN:

*   LOCAL VARIABLES:
        REAL DIST, ANGL, FNEG2, TWOPI, W(2)

*   Programmed by: W. H. Kirby

      SAVE
C
      LOGICAL NONE

      REAL      SIN, COS, ALOG, SQRT
      INTRINSIC SIN, COS, ALOG, SQRT

      EXTERNAL RANDUV

      DATA TWOPI/6.2831853/, FNEG2/-2./, ANGL/1E38/, NONE/.TRUE./
C
      IF(NONE) GO TO 10
      GAUSSF=DIST*SIN(ANGL)
      NONE=.TRUE.
      RETURN
   10 CALL RANDUV(IRAN,W,2)
      DIST=SQRT(FNEG2*ALOG(W(2)))
      ANGL=TWOPI*W(1)
      GAUSSF=DIST*COS(ANGL)
      NONE=.FALSE.
      RETURN
      END

C==================================================
C==================================================  RANDUV       ====
      SUBROUTINE RANDUV(IIRAN,X, N)
      
      IMPLICIT NONE
C
C  *** IBM ***   *** IBM ***   *** IBM ***
C    *** PRIME ***     *** PRIME ***     *** PRIME ***    (ALSO)
C
C  NOTE - AN ASSEMBLY-LANGUAGE VERSION FOR IBM/360-370 IS AVAILABLE

*   ARGUMENTS:
       INTEGER IIRAN, I, N 
       REAL X(2)
       
*   ARGUMENT DEFINITIONS:

*   LOCAL VARIABLES:
      REAL SCALE
      DOUBLE PRECISION IRAN, MODU, MULT

      INTEGER   INT
      REAL      SNGL
      DOUBLE PRECISION DMOD, DBLE
      INTRINSIC  SNGL, DMOD, DBLE, INT
    
*   Programmed by: W. H. Kirby

C
      DATA MODU, MULT / 2147483647D0, 16807D0/
C
      SCALE=1./SNGL(MODU)
      IRAN = DBLE(IIRAN)
      DO 10 I = 1, N
      IRAN = IRAN*MULT
      IRAN = DMOD(IRAN, MODU)
      X(I) = SCALE*SNGL(IRAN)
   10 CONTINUE
      IIRAN = INT(IRAN)
      RETURN
      END


*==== EOF NETCNTRL =====================================================
