************************************************************************
*
*    See solveutl.f for explanation.
*
************************************************************************



*== Private (ExtentMatrix) =============================================

      LOGICAL FUNCTION ExtentMatrix()

      IMPLICIT NONE

*   Purpose:  Determine extent of coefficient matrix and initial order
*             of equations.  This routine is specific to the four-point-
*             implicit finite-difference solution as used in the flow
*             model FourPt.
*
*        Note:  At this point, IDIAG(I) represents the column number of
*               the left extent of the coefficient matrix on the Ith row.
*               LE(I) represents the row number of the upper extent of
*               column I. These definitions are changed in subroutine ORDER.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline2.inc'

*   Local Variables:
      INTEGER I, J, M, N
      INTEGER ChannelNumber
      LOGICAL OK

*   Routines by module:

***** Channel schematic:
      INTEGER  NumberOfStreamLocations
      EXTERNAL NumberOfStreamLocations

      INTEGER  NumberOfChannels, UpstreamPointer, DownstreamPointer
      EXTERNAL NumberOfChannels, UpstreamPointer, DownstreamPointer

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      ExtentMatrix = .FALSE.

*---- I = current row number.
      I = 0

*---- J = current equation number.
      J = 0

      DO 200 M=1,NumberOfChannels()
        ChannelNumber = M

        IF( OpenChannel( ChannelNumber ) ) THEN

*-------- flow constraint.
          I = I + 1
          IDIAG(I) = I
          LE(I) = I
          UpConstraintEq(ChannelNumber) = I

*-------- first subdomain of channel.
          J = J + 1

*-------- mass equation.
          I = I + 1
          IDIAG(I) = I - 1
          LE(I) = I
          MASSEQ(J) = I
          MASSOF(J) = -1

*-------- dynamic equation.
          I = I + 1
          IDIAG(I) = I - 2
          LE(I) = I - 1
          DYNMEQ(J) = I
          DYNMOF(J) = -2

*-------- set pointer to first equation of channel.
          EqPointer( ChannelNumber ) = J


          IF( NumberOfStreamLocations() .GT. 2 ) THEN

            DO 100 N=UpstreamPointer()+2,DownstreamPointer()

*------------ general subdomain.
              J = J + 1

*------------ mass equation.
              I = I + 1
              IDIAG(I) = I - 1
              LE(I) = I - 2
              MASSEQ(J) = I
              MASSOF(J) = -1

*------------ dynamic equation.
              I = I + 1
              IDIAG(I) = I - 2
              LE(I) = I - 1
              DYNMEQ(J) = I
              DYNMOF(J) = -2

 100        CONTINUE

          END IF

*-------- Downstream water-surface-elevation constraint.
          I = I + 1
          IDIAG(I) = I
          LE(I) = I - 2
          DownConstraintEq(ChannelNumber) = I

        ELSE
          WRITE(*,*) ' *** Error (ExtentFPT)'
          WRITE(*,*) ' Could not open channel...',ChannelNumber
          STOP
        END IF

        OK = CloseChannel()

 200  CONTINUE


      IF( I .EQ. Equations ) THEN

        ExtentMatrix = .TRUE.

      ELSE

        WRITE(*,*) ' *** Error (ExtentMatrix)'
        WRITE(*,*) I,' rows, ',Equations,' equations...'
        WRITE(*,*) ' Abnormal program end.'
        STOP

      END IF

      RETURN
      END



*== Private (MaxBandMatrix) ============================================

      LOGICAL FUNCTION MaxBandMatrix()

      IMPLICIT NONE

*   Purpose:  Compute maximum bandwidth and extent of coefficient matrix,
*             and modify matrix shape for internal and external boundary
*             conditions.  This routine is specific to the four-point-
*             implicit finite-difference solution as used in the flow
*             model FOURPT.

*        Note:  At this point, IDIAG(I) represents the column number of
*               the left extent of the coefficient matrix on the Ith row.
*               LE(I) represents the row number of the upper extent of
*               column I. These definitions are changed in subrouine ORDER.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline2.inc'

*   Local Variables:
      INTEGER I, J, M
      INTEGER Code, ChannelNumber, Bandwidth
      LOGICAL OK

*   Routines by module:

***** Network control:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkprintLevel

***** Channel schematic:
      INTEGER  UpstreamCode, DownstreamCode, NumberOfStreamLocations
      EXTERNAL UpstreamCode, DownstreamCode, NumberOfStreamLocations

      INTEGER  NumberOfChannels
      EXTERNAL NumberOfChannels

      INTEGER  UpstreamConnections, DownstreamConnections
      EXTERNAL UpstreamConnections, DownstreamConnections

      LOGICAL  OpenChannel, CloseChannel
      EXTERNAL OpenChannel, CloseChannel

***** Local:
      LOGICAL  ModForUpstreamConnections, ModForDownstreamConnections
      EXTERNAL ModForUpstreamConnections, ModForDownstreamConnections

*   Intrinsics:
      INTEGER    MOD
      INTRINSIC  MOD



*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:   Lew DeLong
*   Last modified: December 1991
*


*-- Implementation -----------------------------------------------------

      MaxBandMatrix = .FALSE.

*---- Boundary-condition code          Use...
*
*       water surface
*          1                        explicitly known
*          4                        self-setting (downstream only)
*         11                        equal to another water surface
* ........31                        3-parameter relation
* ........41                        (reserved for four-parameter rating)
* ........51                        user-defined
*
*       flow
*          2                        explicitly known
*         12                        sum of flows equal zero
* ........32                        3-parameter relation
* ........42                        (reserved for four-parameter rating)
* ........52                        user-defined




*---- default maximum bandwidth.
      Bandwidth = 5

*---- I = current row number.
      I = 0

*---- J = current equation number.


*---- Modify matrix shape for type of constraint.

      IF( NetworkPrintLevel() .GT. 0 ) THEN
        WRITE(*,*) ' '
        WRITE(*,*) ' Modifying matrix shape for constraint equations...'
      END IF

      DO 100 M=1,NumberOfChannels()

        ChannelNumber = M

        IF( OpenChannel( ChannelNumber ) ) THEN

*-------- Upstream end of channel.
          Code = MOD( UpstreamCode(), 100 ) 
          I = I + 1
          J = EqPointer(Branch)

          IF( Code .EQ. 1
     #             .OR.
     #        Code .EQ. 11
     #             .OR.
     #        Code .EQ. 31 
     #             .OR.
     #        Code .EQ. 51  ) THEN

*---------- Upstream stage-type boundary.

*---------- substitute first mass equation for flow constraint.
            MASSEQ(J) = MASSEQ(J) - 1
            MASSOF(J) = 0

*---------- make room for it.
            LE(I) = I
            LE(I+1) = I
            LE(I+2) = I
            LE(I+3) = I

*---------- reduce space previously reserved for first mass equation.
            IF( Code .NE. 31
     #               .AND.
     #          Code .NE. 51 ) THEN
              IDIAG(I+1) = I + 1
            END IF

*---------- move constraint to water-surface-elevation row.
            UpConstraintEq(ChannelNumber) = I + 1

*---------- adjust maximum bandwidth.
            IF( Bandwidth .LT. 7 ) THEN
              Bandwidth = 7
            END IF
            
          ELSE IF( Code .EQ. 32
     #                  .OR.
     #             Code .EQ. 52 ) THEN
          
*---------- Upstream 3-parameter rating or user-supplied
*            constraint applied as flow constraint.

*---------- make room for stage.
            LE(I+1) = I
          
          END IF

*-------- Downstream end of channel.
          Code = MOD( DownstreamCode(), 100 )
          I = I + 2 * (NumberOfStreamLocations() - 1 ) + 1
          J = J + NumberOfStreamLocations() - 2

          IF( Code .EQ. 2
     #             .OR.
     #        Code .EQ. 12
     #             .OR.
     #        Code .EQ. 32
     #             .OR.
     #        Code .EQ. 52  ) THEN

*-------- Flow constraint applied at downstream end of channel.

*---------- substitute last dynamic equation for watersurface constraint.
            DYNMEQ(J) = DYNMEQ(J) + 1
            DYNMOF(J) = - 3

*---------- make room for it.
            IDIAG(I) = I - 3

*---------- reduce space previously reserved for last dynamic equation.
            IDIAG(I-1) = I - 1

*---------- move constraint to flow row.
            DownConstraintEq(ChannelNumber) = I - 1

*---------- adjust maximum bandwidth.
            IF( Bandwidth .LT. 7 ) THEN
              Bandwidth = 7
            END IF

          ELSE IF( Code .EQ. 4 ) THEN

*-------- Self-setting condition applied at downstream end of channel.

*---------- make room for it.
            IDIAG(I) = I - 2

          ELSE IF( Code .EQ. 31
     #                  .OR.
     #             Code .EQ. 51 ) THEN
          
*---------- Downstream 3-parameter rating or user-supplied
*            constraint applied as stage constraint.

*---------- move left extent to include flow.
            IDIAG(I) = I - 1
          
          END IF

        ELSE
          WRITE(*,*) ' ***Error (MaxBandMatrix)'
          WRITE(*,*) ' Could not open channel...',ChannelNumber
          STOP
        END IF

        OK = CloseChannel()

 100  CONTINUE

*---- Modify matrix shape and compute maximum bandwith,
*      based on connections.

      IF( NetworkPrintLevel() .GT. 0 ) THEN
        WRITE(*,*) ' Modifying matrix shape for connections...'
        WRITE(*,*) ' '
      END IF



      IB = Bandwidth

      DO 200 M=1,NumberOfChannels()

        ChannelNumber = M
        IF( OpenChannel( ChannelNumber ) ) THEN


*-------- Upstream end of channel.

          IF( UpstreamConnections() .GT. 0 ) THEN
            OK = ModForUpstreamConnections()
          ELSE
          END IF

*-------- Downstream end of channel.

          IF( DownstreamConnections() .GT. 0 ) THEN
            OK = ModForDownstreamConnections()
          ELSE
          END IF

          OK = CloseChannel()

        END IF

 200  CONTINUE

      MaxBandMatrix = .TRUE.

      RETURN
      END



*== Private (ModForUpstreamConnections) ================================

      LOGICAL FUNCTION ModForUpstreamConnections()

      IMPLICIT NONE

*   Purpose:  Modify matrix shape and maximum bandwidth
*             for upstream connections to other channels.

*        Note:  At this point, IDIAG(I) represents the column number of
*               the left extent of the coefficient matrix on the Ith row.
*               LE(I) represents the row number of the upper extent of
*               column I. These definitions are changed in subrouine ORDER.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline2.inc'

*   Local Variables:
      INTEGER I, Code, Node, ConstraintNode, Row, Col, Span
      INTEGER MaxSpan

*   Routines by module:

***** Channel schematic:
      INTEGER  UpstreamConnect
      EXTERNAL UpstreamConnect

      INTEGER  UpstreamConnections, UpstreamCode
      EXTERNAL UpstreamConnections, UpstreamCode

      INTEGER  UpstreamPointer
      EXTERNAL UpstreamPointer

      INTEGER  StreamEndNode
      EXTERNAL StreamEndNode

***** Local:
      INTEGER  UpstreamConstraintRow
      EXTERNAL UpstreamConstraintRow

*   Intrinsics:
      INTEGER   IABS, MOD
      INTRINSIC IABS, MOD

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: October   1992
*


*-- Implementation -----------------------------------------------------

      MaxSpan = 0

*---- Boundary-condition code          Use...
*
*       water surface
*          1                        explicitly known
*          4                        self-setting (downstream only)
*         11                        equal to another water surface
* ........31                        3-parameter relation
* ........41                        (reserved for four-parameter rating)
* ........51                        user-defined
*
*       flow
*          2                        explicitly known
*         12                        sum of flows equal zero
* ........32                        3-parameter relation
* ........42                        (reserved for four-parameter rating)
* ........52                        user-defined

      DO 100 I=1,UpstreamConnections()

        Node = StreamEndNode( UpstreamConnect(I) )

        IF(Node .NE. 0 ) THEN

*-------- Node is connected externally.

          Code = MOD( UpstreamCode(), 100 )
          
          ConstraintNode = UpstreamPointer()
          Row = UpstreamConstraintRow()

          IF( Code .EQ. 11
     #             .OR.
     #        Code .EQ. 12 
     #             .OR.
     #        Code .EQ. 31 
     #             .OR.
     #        Code .EQ. 32 
     #             .OR.
     #        Code .EQ. 51
     #             .OR.
     #        Code .EQ. 52  ) THEN


              Span = 2 * ( IABS(Node) - ConstraintNode )

              Col = Row + Span

              IF( Span .LT. 0 ) THEN

*-------------- Connecting Node is left of diagonal,
*                 make room for connecting flow or stage.

                IF( Code .NE. 31
     #                   .AND.
     #              Code .NE. 51 ) THEN
                  IF( IDIAG(Row) .GT. Col ) IDIAG(Row) = Col
                ELSE
                  IF( IDIAG(Row) .GT. Col - 1 ) IDIAG(Row) = Col - 1
                  Span = Span - 1
                END IF

              ELSE IF( Span .GT. 0 ) THEN

*-------------- Connecting Node is right of diagonal,
*                 make room for connecting flow or stage.

                IF( Code .EQ. 11
     #                   .OR.
     #              Code .EQ. 12 ) THEN
                  IF( LE(Col) .GT. Row ) LE(Col) = Row

                ELSE IF( Code .EQ. 31
     #                        .OR.
     #                   Code .EQ. 51 ) THEN
                  IF( LE(Col) .GT. Row ) LE(Col) = Row
                  IF( LE(Col-1) .GT. Row ) LE(Col-1) = Row

                ELSE IF( Code .EQ. 32
     #                        .OR.
     #                   Code .EQ. 52 ) THEN
                  IF( LE(Col) .GT. Row ) LE(Col) = Row
                  IF( LE(Col+1) .GT. Row ) LE(Col+1) = Row
                  Span = Span + 1

                END IF

              END IF

          END IF

        END IF

        IF( IABS(Span) .GT. MaxSpan ) MaxSpan = IABS(Span)

 100  CONTINUE

*---- Adjust maximum bandwidth.

      MaxSpan = 2 * MaxSpan + 1

      IF( MaxSpan .GT. IB ) IB = MaxSpan

      ModForUpstreamConnections = .TRUE.

      RETURN
      END



*== Private (ModForDownstreamConnections) ==============================

      LOGICAL FUNCTION ModForDownstreamConnections()

      IMPLICIT NONE

*   Purpose:  Modify matrix shape and maximum bandwidth
*             for Downstream connections to other channels.

*        Note:  At this point, IDIAG(I) represents the column number of
*               the left extent of the coefficient matrix on the Ith row.
*               LE(I) represents the row number of the upper extent of
*               column I. These definitions are changed in subrouine ORDER.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline2.inc'

*   Local Variables:
      INTEGER I, Code, Node, ConstraintNode, Row, Col, Span
      INTEGER MaxSpan

*   Routines by module:

***** Channel schematic:
      INTEGER  DownstreamConnect
      EXTERNAL DownstreamConnect

      INTEGER  DownstreamConnections, DownstreamCode
      EXTERNAL DownstreamConnections, DownstreamCode

      INTEGER  DownstreamPointer
      EXTERNAL DownstreamPointer

      INTEGER  StreamEndNode
      EXTERNAL StreamEndNode

***** Local:
      INTEGER  DownstreamConstraintRow
      EXTERNAL DownstreamConstraintRow

*   Intrinsics:
      INTEGER   IABS, MOD
      INTRINSIC IABS, MOD

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: October   1992
*


*-- Implementation -----------------------------------------------------

      MaxSpan = 0

*---- Boundary-condition code          Use...
*
*       water surface
*          1                        explicitly known
*          4                        self-setting (downstream only)
*         11                        equal to another water surface
* ........31                        3-parameter relation
* ........41                        (reserved for four-parameter rating)
* ........51                        user-defined
*
*       flow
*          2                        explicitly known
*         12                        sum of flows equal zero
* ........32                        3-parameter relation
* ........42                        (reserved for four-parameter rating)
* ........52                        user-defined

      DO 100 I=1,DownstreamConnections()

        Node = StreamEndNode( DownstreamConnect(I) )

        IF(Node .NE. 0 ) THEN

*-------- Node is connected externally.

          Code = MOD( DownstreamCode(), 100 )

          ConstraintNode = DownstreamPointer()
          Row = DownstreamConstraintRow()

          IF( Code .EQ. 11
     #             .OR.
     #        Code .EQ. 12 
     #             .OR.
     #        Code .EQ. 31 
     #             .OR.
     #        Code .EQ. 32 
     #             .OR.
     #        Code .EQ. 51
     #             .OR.
     #        Code .EQ. 52  ) THEN

              Span = 2 * ( IABS(Node) - ConstraintNode )

              Col = Row + Span

              IF( Span .LT. 0 ) THEN

*-------------- Connecting node is left of diagonal,
*                 make room for flow or stage.

                IF( Code .NE. 31
     #                   .AND.
     #              Code .NE. 51 ) THEN
                  IF( IDIAG(Row) .GT. Col ) IDIAG(Row) = Col
                
                ELSE
                  IF( IDIAG(Row) .GT. Col - 1 ) IDIAG(Row) = Col - 1
                  Span = Span - 1
                END IF

              ELSE IF( Span .GT. 0 ) THEN

*-------------- Connecting Node is right of diagonal,
*                 make room for connecting flow or stage.

                IF( Code .EQ. 11
     #                   .OR.
     #              Code .EQ. 12 ) THEN
                  IF( LE(Col) .GT. Row ) LE(Col) = Row

                ELSE IF( Code .EQ. 31
     #                        .OR.
     #                   Code .EQ. 51 ) THEN
                  IF( LE(Col) .GT. Row ) LE(Col) = Row
                  IF( LE(Col-1) .GT. Row ) LE(Col-1) = Row

                ELSE IF( Code .EQ. 32
     #                        .OR.
     #                   Code .EQ. 52 ) THEN
                  IF( LE(Col) .GT. Row ) LE(Col) = Row
                  IF( LE(Col+1) .GT. Row ) LE(Col+1) = Row
                  Span = Span + 1

                END IF


              END IF

          END IF

        END IF

        IF( IABS(Span) .GT. MaxSpan ) MaxSpan = IABS(Span)

 100  CONTINUE

*---- Adjust maximum bandwidth.

      MaxSpan = 2 * MaxSpan + 1

      IF( MaxSpan .GT. IB ) IB = MaxSpan

      ModForDownstreamConnections = .TRUE.

      RETURN
      END


*==== EOF solvefpt =====================================================
