************************************************************************
*
*    See solveutl.f for explanation.
*
************************************************************************


*== Public (InitializeSolver) ==========================================

      LOGICAL FUNCTION InitializeSolver()

      IMPLICIT NONE

*   Purpose: Initialize the matrix-solution module skymod.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline6.inc'

*   Local Variables:
      INTEGER I
      CHARACTER*12 InternalFileName, FileName

*   Routines by module:

***** Channel schematic:
      INTEGER  TotalStreamLocations
      EXTERNAL TotalStreamLocations

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileUnit, GetFileName
      
***** File utilities:
      LOGICAL  OpenNewText
      EXTERNAL OpenNewText

***** Network control:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel

***** Local:
      LOGICAL  ExtentMatrix, MaxBandMatrix, Reorder
      EXTERNAL ExtentMatrix, MaxBandMatrix, Reorder

*   Programmed by:   Lew DeLong
*   Date:            July 1990
*   Modified by:     Lew DeLong
*   Last modified:   April 1992
*
*  


*-- Implementation -----------------------------------------------------

      InitializeSolver = .FALSE.

      InternalFileName = 'neterror.dat'
      
*---- FORTRAN units.

      ErrUnit = GetFileUnit(InternalFileName)
      FileName = GetFileName(InternalFileName)

      Equations = 2*TotalStreamLocations()

      IndexedNetworkSolver = .TRUE.

*---- Set up matrix-shape indicies.
        IF( ExtentMatrix() ) THEN
          IF( MaxBandMatrix() ) THEN
            IF( Reorder() ) THEN
            ELSE
              WRITE(*,*) ' Reorder failed...'
              RETURN
            END IF
          ELSE
            WRITE(*,*) ' MaxBandMatrix failed ...'
            RETURN
          END IF
        ELSE
          WRITE(*,*) ' ExtentMatrix failed ...'
          RETURN
        END IF

*---- Write Skyline-storage structure to file.
      IF(NetworkPrintLevel().GT.8) THEN
        IF (OpenNewText(ErrUnit,InternalFileName)) THEN
          WRITE(ErrUnit,'(3I10)') IB,Equations,NUMFLT,
     #    (I,IDIAG(I),LE(I),I=1,Equations)
        ELSE
          WRITE(*,*) ' Could not open ', FileName
        END IF
      END IF
      
      
*---- Initialize 'first iteration' index.

      FirstTime = .TRUE.

      InitializeSolver = .TRUE.

      RETURN
      END



*=======================================================================
*   Public: SetupSolver
*=======================================================================

      LOGICAL FUNCTION SetupSolver()

      IMPLICIT NONE

*   Purpose:  Record indices for Gaussian elimination,
*             using skyline storage.

*                      Programmed by:   Lew DeLong
*                      Date programmed: March, 1992
*                      Modified by:     Lew DeLong
*                      Last modified:   April 1992
*

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline3.inc'
      INCLUDE 'skyline5.inc'
      INCLUDE 'skyline6.inc'

*   Local Variables:
      INTEGER N2, N3, N4
      INTEGER NEM1,IP1,J1,K, KK, K1,K2,IROW
      INTEGER IBD2,JEND
      INTEGER I,J, JJ, II, M2, M3, M4
      INTEGER IDIAGI,IDIAGJ
      INTEGER Rows, Columns
      INTEGER I3, I4, I5, I7, I6
      INTEGER Temp1(HBW), Temp2(HBW), Temp3(HBW)
      LOGICAL Flag, DimensionsOK
C     LOGICAL OK

*   Functions:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel
C     LOGICAL  SayTime
C     EXTERNAL SayTime


*   Intrinsics:
      REAL      ABS
      INTRINSIC ABS

*-- Implementation -----------------------------------------------------

      SetupSolver = .FALSE.
      
      DimensionsOK = .TRUE.
      
C     WRITE(*,*) ' Begin matrix-solver setup...'
      
C     OK = SayTime()

*---- Check for "0" diagonal.
        Flag = .TRUE.
        DO 100 I=1,Equations
          II=IDIAG(I)
          IF(ABS(A(II)).LT.1.0E-30) THEN
            Flag = .FALSE.
            WRITE(*,*) ' '
            WRITE(*,*) ' ****ZERO DIAGONAL, ROW',I,'****'
            WRITE(*,*) ' ****LE(ROW)=',LE(I),'****'
            WRITE(*,*) ' ****IDIAG(ROW)=',II,'****'
            WRITE(*,*) ' '
          END IF
 100    CONTINUE
        
        IF( Flag ) THEN
        ELSE
          WRITE(*,*) ' Abnormal program end...'
          STOP
        END IF

      N2   = 0
      N3   = 0
      N4   = 0

      M2   = 0
      M3   = 0
      M4   = 0
      NEM1 = Equations-1
      IBD2 = (IB-1)/2
	  
      IF( HBW .LT. IBD2 ) THEN
        WRITE(*,*) ' ####Error(SetupSolver)'
        WRITE(*,*) ' Half maximum bandwidth exceeds HBW...'
        WRITE(*,*) ' Increase HBW in "skyline6.inc" to...',IBD2
        WRITE(*,*) ' Abnormal program end...'
        STOP
      END IF
      

*---- Major loop, forward elimination,
*      do from 1 to Equations - 1.

      I3 = 0
      I4 = 0
      I5 = 0
      DO 450 I=1,NEM1
       IDIAGI = IDIAG(I)
       
       I3 = I3 + 1
       IF( I3 .LE. MaxS3 ) THEN
         S3( I3 ) = IDIAGI
       ELSE
         DimensionsOK = .FALSE.
       END IF
       
       IP1    = I + 1
       JEND   = I + IBD2
       IF (Equations.LT.JEND) JEND = Equations


*------ Determine occupied columns to the right of diagonal,
*        in current row.

        Columns = 1
        DO 300 K=IP1,JEND
          K1 = LE(K)-K+I
          IF (K1.GT.IDIAG(K-1)) THEN

*---------- Position K1 is potentially occupied in row I.

            IF( Occupied(K1) .EQ. '*' ) THEN

*------------ Position K1 is occupied in row I.
               Columns = Columns + 1

              Temp1( Columns ) = K
              Temp2( Columns ) = K1

            END IF
          END IF

 300    CONTINUE

*----- Save column indices for later use.

       S2(I) = Columns
       
       IF( (I3+Columns) .LE. MaxS3 ) THEN
       
         DO 344 II=2,Columns

           I3 = I3 + 1
           S3( I3 )  = Temp2(II)

344      CONTINUE
       
       ELSE
         I3 = I3 + Columns
         DimensionsOK = .FALSE.
       END IF


*----- Eliminate diagonal coefficient, row I, from
*       all following equations,
*       do from I+1 to last equation.


       Rows    = 0
       DO 440 J=IP1,JEND
        IDIAGJ = IDIAG(J)
        J1 = IDIAGJ - J + I

        IF ( J1 .GE. LE(J) ) THEN

*------- Subject coefficient is potentially present in row J.

         IF( Occupied(J1) .EQ. '*' ) THEN
		
*--------- Subject coefficient is actually present in row J.

           Rows = Rows + 1
           Temp3( Rows ) = J

*--------- Eliminate it.

*          A(J1) = A(J1) / A(IDIAGI)
           N4 = N4 + 1

           I4 = I4 + 1
           IF( I4 .LE. MaxS4 ) THEN
             S4( I4 ) = J1
           ELSE
             DimensionsOK = .FALSE.
           END IF

*--------- Follow through for balance of equation,
*           do from I + 1 to end of equation.

           DO 400 KK=1,Columns-1
             K = Temp1(KK+1)
           
                IF(K.GT.J) THEN
                  K2 = LE(K)-K+J
                ELSE
                  K2 = IDIAGJ-J+K
                END IF
             
*            A(K2) = A(K2) - A(J1) * A (Temp2(KK+1))
             Occupied(K2) = '*'
             I4 = I4 + 1
             IF( I4 .LE. MaxS4 ) THEN
               S4( I4 ) = K2
             ELSE
               DimensionsOK = .FALSE.
             END IF
             N3 = N3 + 1
             N2 = N2 + 1

400       CONTINUE

*         X(J)=X(J)-X(I)*A(J1)
          M3 = M3 + 1
          M2 = M2 + 1
          N2 = N2 + 1
          N3 = N3 + 1
          
         END IF
        END IF
440    CONTINUE

       S1(I) = Rows
       
       IF( (I5 + Rows) .LE. MaxS5 ) THEN
       
          DO 445 J=1,Rows
            I5 = I5 + 1
            S5(I5) = Temp3(J)
445       CONTINUE

       ELSE
         I5 = I5 + Rows
         DimensionsOK = .FALSE.
       END IF


450   CONTINUE

      IF( NetworkPrintLevel() .GT. 1 ) THEN
        WRITE(*,*) ' Floating-point operations:'
        WRITE(*,*) '     forward elimination...'
        WRITE(*,*) '       Divisions.........', N4
        WRITE(*,*) '       Multiplications...', N3
        WRITE(*,*) '       Subtractions......', N2
      END IF

*---- Back substitutuion.

      I7 = 0
      I6 = 0

      IROW=Equations
      IDIAGI=IDIAG(Equations)
*     X(Equations)=X(Equations)/A(IDIAGI)
      M4 = M4 + 1
      N4 = N4 + 1

      DO 470 I=1,NEM1
       IROW=IROW-1
       IDIAGI=IDIAG(IROW)
       IP1=IROW+1
       JEND=IP1+IBD2-1
       IF (JEND.GT.Equations) JEND=Equations

       Columns = 0
       DO 460 J=IP1,JEND
        J1=LE(J)-J+IROW
        IF (J1.GT.IDIAG(J-1)) THEN
          IF( Occupied(J1) .EQ. '*' ) THEN
*           X(IROW)=X(IROW)-A(J1)*X(J)
            N3 = N3 + 1
            M3 = M3 + 1
            N2 = N2 + 1
            M2 = M2 + 1
            Columns = Columns + 1
            Temp1(Columns) = J1
            Temp2(Columns) = J
          END IF
        END IF
460    CONTINUE

*      X(IROW)=X(IROW)/A(IDIAGI)
       N4 = N4 + 1
       M4 = M4 + 1
       
       IF( (I7+Columns+1) .LE. MaxS7 ) THEN
         DO 465 JJ=1,Columns
           I7 = I7 + 1
           S7(I7) = Temp1(JJ)
           S8(I7) = Temp2(JJ)
465      CONTINUE
         I7 = I7 + 1
         S7(I7) = IDIAGI
         S8(I7) = IROW
         I6 = I6 + 1
         S6(I6) = Columns
       ELSE
         I7 = I7 + Columns + 1
         DimensionsOK = .FALSE.
       END IF

470   CONTINUE


      SetupSolver = .TRUE.
      
      IF( NetworkPrintLevel() .GT. 1 ) THEN
        WRITE(*,*) '     total with full elimination...'
        WRITE(*,*) '       Divisions.........', N4
        WRITE(*,*) '       Multiplications...', N3
        WRITE(*,*) '       Subtractions......', N2
        WRITE(*,*) '     total with constant coefficients matrix...'
        WRITE(*,*) '       Divisions.........', M4
        WRITE(*,*) '       Multiplications...', M3
        WRITE(*,*) '       Subtractions......', M2
      END IF
        
      IF( NetworkPrintLevel() .GT. 1
     #                        .OR.
     #               .NOT. DimensionsOK          ) THEN

        WRITE(*,*) ' '
        WRITE(*,*) ' '
        WRITE(*,*) '      -- Dimensions --'
        WRITE(*,*) ' include file "skyline6.inc"'
        WRITE(*,*) ' '
        WRITE(*,*) ' Array... Required... Available...'
        WRITE(*,*) ' '
        WRITE(*,*) ' S3()...  ', I3, MaxS3
        WRITE(*,*) ' S4()...  ', I4, MaxS4
        WRITE(*,*) ' S5()...  ', I5, MaxS5
        WRITE(*,*) ' S7()...  ', I7, MaxS7
        WRITE(*,*) ' '
       
         WRITE(*,*) ' Maximum half band width ...',IBD2
         WRITE(*,*) '               available ...',HBW
         WRITE(*,*) ' '
       END IF
      
      IF( DimensionsOK ) THEN
      
C       WRITE(*,*) ' End matrix-solver setup...'
        
C       OK = SayTime()
      
      ELSE
      
        WRITE(*,*) ' ####Error(SetupSolver)'
        WRITE(*,*) ' Dimension(s) exceeded...'
        WRITE(*,*) ' Abnormal program end.'
        STOP
      
      END IF
      
      
      RETURN
      END



*== Public (InitializeMatrix) ==========================================

      LOGICAL FUNCTION InitializeMatrix()

      IMPLICIT NONE

*   Purpose:  Initialize skyline coefficient matrix to zero, If
*             a forward elimination will be performed during the
*             current iteration.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline3.inc'
      INCLUDE 'skyline5.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  ForwardElim
      EXTERNAL ForwardElim

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:   Lew DeLong
*   Last modified: April 1992
*


*-- Implementation -----------------------------------------------------

      InitializeMatrix = .FALSE.

      IF( ForwardElim() ) THEN
        DO 100 I=1,NUMFLT
          A(I) = 0.0
          Occupied(I) = ' '
 100    CONTINUE
      END IF

      InitializeMatrix = .TRUE.

      RETURN
      END



*=======================================================================
*   Public: SolveMatrix
*=======================================================================

      LOGICAL FUNCTION SolveMatrix()

      IMPLICIT NONE

*   Purpose:  Solve matrix using skyline storage.

*                      Programmed by:   Lew DeLong
*                      Date programmed: ALongLongTimeAgo
*                      Modified by:     Lew DeLong
*                      Last modified:   July, 1992
*

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline3.inc'
      INCLUDE 'skyline6.inc'

*   Local Variables:
      INTEGER I, J, JJ
      INTEGER J1,KK
      INTEGER JEnd, KEnd
      INTEGER I1, I2, I3, I4, I5, I6, I7
      INTEGER Diagonal, Row, DiagonalSub
C     LOGICAL OK

*   Functions:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel
C     LOGICAL  SayTime
C     EXTERNAL SayTime


*   Intrinsics:
      REAL      ABS
      INTRINSIC ABS

*-- Implementation -----------------------------------------------------

      SolveMatrix = .FALSE.
      
*---- Write skyline matrix to file.
      IF(NetworkPrintLevel() .GT. 8) THEN
        WRITE(ErrUnit,'(I10,E16.8)') (I,X(I),I=1,Equations),
     #                            (I,A(I),I=1,NUMFLT)
      END IF

      
C     WRITE(*,*) ' Begin full Gauss elimination...'
      
C     OK = SayTime()
      
      I1 = 0
      I2 = 0
      I3 = 0
      I4 = 0
      I5 = 0
      DO 450 I=1,Equations-1
       I1 = I1 + 1
       JEnd   = S1(I1)

       DiagonalSub = I3 + 1
       Diagonal = S3(DiagonalSub)

       I2 = I2 + 1
       KEnd = S2(I2)
       I3 = I3 + KEnd

       DO 440 JJ=1,JEnd
       
          I4 = I4 + 1
          J1 = S4(I4)
          
          A(S4(I4)) = A(S4(I4)) / A(Diagonal)

          DO 400 KK=1,KEnd-1
             I4 = I4 + 1
             A(S4(I4)) = A(S4(I4))
     #                 - A(J1) * A(S3(DiagonalSub+KK))
400       CONTINUE

        I5 = I5 + 1
        X(S5(I5)) = X(S5(I5)) - X(I) * A(J1)
             
440    CONTINUE

450   CONTINUE

*     WRITE(*,*) ' Begin back substitution...'

      X(Equations) = X(Equations) / A( IDIAG(Equations) )

      I6 = 0
      I7 = 0
      Row = Equations
      DO 470 I=1,Equations-1
        Row = Row - 1
        I6 = I6 + 1
        DO 460 J=1,S6(I6)
          I7 = I7 + 1
          X(Row) = X(Row) - A(S7(I7)) * X(S8(I7))
 460    CONTINUE

        I7 = I7 + 1
        X(Row) = X(Row) / A(S7(I7))

 470  CONTINUE


C     WRITE(*,*) ' End full Gauss elimination...'
      
C     OK = SayTime()

      SolveMatrix = .TRUE.
            
      RETURN
      END



*== Private (BackSubs) ================================================

      LOGICAL FUNCTION BackSubs()

      IMPLICIT NONE

*   Purpose:  Perform back substitution part of Gaussian elimination.

*                      Programmed by:   Lew DeLong
*                      Date programmed: ALongLongTimeAgo
*                      Modified by:     Lew DeLong
*                      Last modified:   March, 1992
*

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline3.inc'
      INCLUDE 'skyline6.inc'

*   Local Variables:
      INTEGER I, J, JJ
      INTEGER JEnd, KEnd
      INTEGER I1, I2, I3, I4, I5, I6, I7
      INTEGER Row
C     LOGICAL OK

*   Functions:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel
C     LOGICAL  SayTime
C     EXTERNAL SayTime


*   Intrinsics:
      REAL      ABS
      INTRINSIC ABS

*-- Implementation -----------------------------------------------------

      BackSubs = .FALSE.
      
*---- Write skyline matrix to file.
      IF(NetworkPrintLevel() .GT. 8) THEN
        WRITE(ErrUnit,'(I10,E16.8)') (I,X(I),I=1,Equations),
     #                            (I,A(I),I=1,NUMFLT)
      END IF

      
C     WRITE(*,*) ' Begin solution with constant coefficients...'
      
C     OK = SayTime()
      
      I1 = 0
      I2 = 0
      I3 = 0
      I4 = 0
      I5 = 0
      DO 450 I=1,Equations-1
       I1 = I1 + 1
       JEnd   = S1(I1)

       I2 = I2 + 1
       KEnd = S2(I2)
       I3 = I3 + KEnd

       DO 440 JJ=1,JEnd
       
         I5 = I5 + 1
         X(S5(I5)) = X(S5(I5)) - X(I) * A(S4(I4+1))
         I4 = I4 + KEnd          
             
440    CONTINUE

450   CONTINUE

      X(Equations) = X(Equations) / A( IDIAG(Equations) )

      I6 = 0
      I7 = 0
      Row = Equations
      DO 470 I=1,Equations-1
        Row = Row - 1
        I6 = I6 + 1
        DO 460 J=1,S6(I6)
          I7 = I7 + 1
          X(Row) = X(Row) - A(S7(I7)) * X(S8(I7))
 460    CONTINUE

        I7 = I7 + 1
        X(Row) = X(Row) / A(S7(I7))

 470  CONTINUE


C     WRITE(*,*) ' End solution with constant coefficients...'
      
C     OK = SayTime()

      BackSubs = .TRUE.
            
      RETURN
      END



*== Public (SolveFourPt) ===============================================

      LOGICAL FUNCTION SolveFourPt()

      IMPLICIT NONE

*   Purpose:  Solve the coefficient matrix assembled by FourPt.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'
      INCLUDE 'skyline6.inc'

*   Local Variables:
      INTEGER I
      LOGICAL OK

*   Routines by module:

***** Channel schematic:
      INTEGER  TotalStreamLocations
      EXTERNAL TotalStreamLocations

***** Local:
      LOGICAL  SolveMatrix, BackSubs
      EXTERNAL SolveMatrix, BackSubs
      LOGICAL  ForwardElim
      EXTERNAL ForwardElim
      LOGICAL  SetupSolver
      EXTERNAL SetupSolver

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      SolveFourPt = .FALSE.
      
      IF( FirstTime ) THEN
        FirstTime = .FALSE.
      
*------ Setup solution indices.

        IF( SetupSolver() ) THEN
        ELSE
          WRITE(*,*) ' SetupSolver failed...'
          WRITE(*,*) ' Abnormal program end...'
          STOP
        END IF
      END IF
      

      DO 100 I=1,2*TotalStreamLocations()
        X(I) = XOld(I) + XAdj(I)
 100  CONTINUE

      IF( ForwardElim() ) THEN
        OK = SolveMatrix()
      ELSE
        OK = BackSubs()
      END IF

      SolveFourPt = .TRUE.

      RETURN
      END



*== Public (StoreMassEq) ===============================================

      LOGICAL FUNCTION StoreMassEq(EqNum,Num,Coef)

      IMPLICIT NONE

*   Purpose:  Store a vector of solution-matrix coefficients for the
*             one-dimensional conservation-of-mass equation.

*   Arguments:
      INTEGER EqNum,Num
      REAL    Coef(Num)

*   Argument definitions:
*     EqNum - mass-equation sequence number, beginning in the network
*             adjacent to the lowest cross-section number and
*             progressing in the direction of increasing cross-section
*             numbers.
*              This number is local to the current channel and the sequence 
*              does not include boundary equations.
*     Num - number of coefficients in vector.
*     Coef - coefficient vector.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'
      INCLUDE 'skyline3.inc'
      INCLUDE 'skyline5.inc'

*   Local Variables:
      INTEGER I, J, K, N

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      K = EqPointer(Branch) + EqNum - 1
      J = MASSOF(K) - 1

      DO 100 I=1,Num
        J = J+1

*------ Get element number of floating-point array.
        N = SkyScript(MASSEQ(K), J)

*------ Store coefficient in global coefficient matrix.
        A(N) = A(N)+Coef(I)
        
*------ Store index for occupied location.
        Occupied(N) = '*'

 100  CONTINUE

      StoreMassEq = .TRUE.

      RETURN
      END



*== Public (StoreDynamicEq) ============================================

      LOGICAL FUNCTION StoreDynamicEq(EqNum, Num, Coef)

      IMPLICIT NONE

*   Purpose:  Store a vector of solution-matrix coefficients for the
*             one-dimensional dynamic equation.

*   Arguments:
      INTEGER EqNum,Num
      REAL    Coef(Num)

*   Argument definitions:
*     EqNum  - dynamic-equation sequence number, beginning in the network adjacent
*              to the lowest cross-section number and progressing in the
*              direction of increasing cross-section numbers.
*     Num    - number of coefficients in vector.
*     Coef   - coefficient vector.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'
      INCLUDE 'skyline3.inc'
      INCLUDE 'skyline5.inc'

*   Local Variables:
      INTEGER I, J, K, N

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      K = EqPointer(Branch) + EqNum - 1
      J = DYNMOF(K) - 1

      DO 100 I=1,Num
        J = J+1

*------ Get element number of floating-point array.
        N = SkyScript(DYNMEQ(K), J)

*------ Store coefficient in global coefficient matrix.
        A(N) = A(N)+Coef(I)
        
*------ Store index for occupied location.
        Occupied(N) = '*'

 100  CONTINUE

      StoreDynamicEq = .TRUE.

      RETURN
      END



*== Public (StoreAtRowOffset) ==========================================

      LOGICAL FUNCTION StoreAtRowOffset(Row, Offset, Value)

      IMPLICIT NONE

*   Purpose:  Store a value in the coeficient matrix at a location
*             corresponding to Row and Offset from the diagonal in
*             the virtual matrix.

*             Values are stored only if complete forward elimination
*             is to be performed on the coefficient matrix, indicated
*             by ForwardElim() = .TRUE..

*   Arguments:
      INTEGER Row, Offset
      REAL    Value

*   Argument definitions:
*     Row    - row number in the virtual matrix.
*     Offset - offset from the diagonal in the virtual matrix,
*              positive if column number is greater than that
*              of the diagonal.
*     Value  - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'
      INCLUDE 'skyline5.inc'

*   Local Variables:
      INTEGER N

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript
      LOGICAL  ForwardElim
      EXTERNAL ForwardElim

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: April 1992
*


*-- Implementation -----------------------------------------------------

      IF( ForwardElim() ) THEN
        N = SkyScript(Row,Offset)
        A(N) = Value
        Occupied(N) = '*'
      END IF

      StoreAtRowOffset = .TRUE.

      RETURN
      END



*== Public (AddAtRowOffset) ============================================

      LOGICAL FUNCTION AddAtRowOffset(Row, Offset, Value)

      IMPLICIT NONE

*   Purpose:  Add a value to the coeficient matrix at a location
*             corresponding to Row and Offset from the diagonal in
*             the virtual matrix.

*   Arguments:
      INTEGER Row, Offset
      REAL    Value

*   Argument definitions:
*     Row    - row number in the virtual matrix.
*     Offset - offset from the diagonal in the virtual matrix,
*              positive if column number is greater than that
*              of the diagonal.
*     Value  - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'
      INCLUDE 'skyline5.inc'

*   Local Variables:
      INTEGER N

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      N = SkyScript(Row,Offset)
      A(N) = A(N) + Value
      Occupied(N) = '*'

      AddAtRowOffset = .TRUE.

      RETURN
      END
