************************************************************************
*
*      See solveutl.f for explanation.
*
************************************************************************


*== Public (InitializeSolver) ==========================================

      LOGICAL FUNCTION InitializeSolver()

      IMPLICIT NONE

*   Purpose: Initialize the matrix-solution module skymod.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'

*   Local Variables:
      INTEGER I
      CHARACTER*12 InternalFileName, FileName

*   Routines by module:

***** Channel schematic:
      INTEGER  TotalStreamLocations
      EXTERNAL TotalStreamLocations

***** Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileUnit, GetFileName
      
***** File utilities:
      LOGICAL  OpenNewText
      EXTERNAL OpenNewText

***** Network control:
      INTEGER  NetworkPrintLevel
      EXTERNAL NetworkPrintLevel

***** Local:
      LOGICAL  ExtentMatrix, MaxBandMatrix, Reorder
      EXTERNAL ExtentMatrix, MaxBandMatrix, Reorder

*   Programmed by:   Lew DeLong
*   Date:            July 1990
*   Modified by:     Lew DeLong
*   Last modified:   January, 1991
*


*-- Implementation -----------------------------------------------------

      InitializeSolver = .FALSE.

      InternalFileName = 'neterror.dat'
      
*---- FORTRAN units.

      ErrUnit = GetFileUnit(InternalFileName)
      FileName = GetFileName(InternalFileName)

      Equations = 2*TotalStreamLocations()

      IndexedNetworkSolver = .FALSE. 

*---- Set up matrix-shape indicies.
        IF( ExtentMatrix() ) THEN
          IF( MaxBandMatrix() ) THEN
            IF( Reorder() ) THEN
            ELSE
              WRITE(*,*) ' Reorder failed...'
              RETURN
            END IF
          ELSE
            WRITE(*,*) ' MaxBandMatrix failed ...'
            RETURN
          END IF
        ELSE
          WRITE(*,*) ' ExtentMatrix failed ...'
          RETURN
        END IF

*---- Write Skyline-storage structure to file.
      IF(NetworkPrintLevel().GT.8) THEN
        IF (OpenNewText(ErrUnit,InternalFileName)) THEN
          WRITE(ErrUnit,'(3I10)') IB,Equations,NUMFLT,
     #    (I,IDIAG(I),LE(I),I=1,Equations)
        ELSE
          WRITE(*,*) ' Could not open ', FileName
        END IF
      END IF

      InitializeSolver = .TRUE.

      RETURN
      END



*== Public (InitializeMatrix) ==========================================

      LOGICAL FUNCTION InitializeMatrix()

      IMPLICIT NONE

*   Purpose:  Initialize skyline coefficient matrix to zero, If
*             a forward elimination will be performed during the
*             current iteration.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Local:
      LOGICAL  ForwardElim
      EXTERNAL ForwardElim

*   Programmed by: Lew DeLong
*   Date:          January 1991
*   Modified by:   Lew DeLong
*   Last modified: April 1992
*


*-- Implementation -----------------------------------------------------

      InitializeMatrix = .FALSE.

      IF( ForwardElim() ) THEN
        DO 100 I=1,NUMFLT
          A(I) = 0.0
 100    CONTINUE
      END IF

      InitializeMatrix = .TRUE.

      RETURN
      END



*== Public (SolveMatrix) ===============================================

      LOGICAL FUNCTION SolveMatrix()

      IMPLICIT NONE

*   Purpose:  Solve asymmetric, skyline-stored matrix by Gaussian elimination.
*             (Previously named SKYV.)

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline1.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER   NVEC,LU
      PARAMETER (NVEC = 20)
      INTEGER   NEM1,IP1,J1,K,K1,K2,IROW
      INTEGER   IBD2,JEND
      INTEGER   I,J,II
      INTEGER   IDIAGI,IDIAGJ
      INTEGER   NL,LOCI(NVEC),KK
      INTEGER   TimeStep, Iteration
      REAL      FAC
      REAL      VI(NVEC)
      LOGICAL   Flag

*   Routines by module:

***** Local:
      LOGICAL  ForwardElim
      EXTERNAL ForwardElim
      
***** Network control:
      INTEGER  NetworkTimeStep, NetworkPrintLevel, NetworkIteration
      EXTERNAL NetworkTimeStep, NetworkPrintLevel, NetworkIteration

*   Intrinsics:
      REAL      ABS
      INTRINSIC ABS

*   Programmed by:   Lew DeLong
*   Date programmed: Dark ages
*   Modified by:     Lew DeLong
*        modified:   July, 1990
*   Modified by:     Lew DeLong
*   Last modified:   April 1992
*


*-- Implementation -----------------------------------------------------

      SolveMatrix = .FALSE.

      IF( ForwardElim() ) THEN
        LU = 1
      ELSE
        LU = 0
      END IF

*     LU     - index for Gaussian elimination,
*              [0] only backward substitution will be performed.
*              [1] forward elimination & backward substitution
*                  will be performed.

*---- Write skyline matrix to file.
      IF(NetworkPrintLevel() .GT. 8) THEN
        WRITE(ErrUnit,'(I10,E16.8)') (I,X(I),I=1,Equations),
     #                            (I,A(I),I=1,NUMFLT)
      END IF

*---- Check for "0" diagonal first call only.
      TimeStep = NetworkTimeStep()
      IF(TimeStep .GT. 1) THEN

      ELSE

        Iteration = NetworkIteration()
        Flag = .TRUE.
        DO 100 I=1,Equations
          II=IDIAG(I)
          IF(ABS(A(II)).LT.1.0E-30) THEN
            Flag = .FALSE.
            WRITE(*,*) ' '
            WRITE(*,*) ' Time step',TimeStep,' , iteration',Iteration
            WRITE(*,*) ' ****ZERO DIAGONAL, ROW',I,'****'
            WRITE(*,*) ' ****LE(ROW)=',LE(I),'****'
            WRITE(*,*) ' ****IDIAG(ROW)=',II,'****'
            WRITE(*,*) ' '
          END IF
 100    CONTINUE

        IF( Flag ) THEN
        ELSE
          RETURN
        END IF
      END IF

      NEM1 = Equations-1
      IBD2 = (IB-1)/2

*---- Forward elimination.
      DO 450 I=1,NEM1
        IDIAGI = IDIAG(I)
        IP1    = I+1
        JEND   = I+IBD2
        IF (Equations.LT.JEND) JEND=Equations

        NL = 0
        DO 300 K=IP1,JEND
          K1 = LE(K)-K+I
          IF (K1.GT.IDIAG(K-1)) THEN
            NL = NL+1

*---------- Check for range error.
            IF(NL.LT.NVEC) THEN
            ELSE
              WRITE(*,*) ' Range error ... (SKYV)'
              WRITE(*,*) '  IB,NVEC,NL ...',IB,NVEC,NL
            END IF
            LOCI(NL) = K
            VI(NL)   = A(K1)
          END IF
 300    CONTINUE

        DO 440 J=IP1,JEND
          IDIAGJ = IDIAG(J)
          J1     = IDIAGJ-J+I
          IF (J1.GE.LE(J)) THEN
            IF (LU.EQ.1)     THEN
              FAC   = A(J1)/A(IDIAGI)
              A(J1) = FAC

              DO 400 KK=1,NL
                K = LOCI(KK)
                IF(K.GT.J) THEN
                  K2 = LE(K)-K+J
                ELSE
                  K2 = IDIAGJ-J+K
                END IF

                A(K2) = A(K2)-FAC*VI(KK)

 400          CONTINUE

            END IF
            X(J) = X(J)-X(I)*A(J1)

          END IF
 440    CONTINUE
 450  CONTINUE


*---- Back substitution.

      IROW      = Equations
      IDIAGI    = IDIAG(Equations)
      X(Equations) = X(Equations)/A(IDIAGI)


      DO 470 I=1,NEM1
        IROW   = IROW-1
        IDIAGI = IDIAG(IROW)
        IP1    = IROW+1
        JEND   = IP1+IBD2-1
        IF (JEND.GT.Equations) JEND=Equations

        DO 460 J=IP1,JEND
          J1 = LE(J)-J+IROW
          IF (J1.GT.IDIAG(J-1)) THEN
            X(IROW) = X(IROW)-A(J1)*X(J)
          END IF
 460    CONTINUE

        X(IROW) = X(IROW)/A(IDIAGI)

 470  CONTINUE

      SolveMatrix = .TRUE.

      RETURN
      END



*== Public (SolveFourPt) ===============================================

      LOGICAL FUNCTION SolveFourPt()

      IMPLICIT NONE

*   Purpose:  Solve the coefficient matrix assembled by FourPt.

*   Arguments:

*   Argument definitions:

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER I

*   Routines by module:

***** Channel schematic:
      INTEGER  TotalStreamLocations
      EXTERNAL TotalStreamLocations

***** Local:
      LOGICAL  SolveMatrix
      EXTERNAL SolveMatrix

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      SolveFourPt = .FALSE.

      DO 100 I=1,2*TotalStreamLocations()
        X(I) = XOld(I) + XAdj(I)
 100  CONTINUE

      IF( SolveMatrix() ) THEN
      ELSE
        RETURN
      END IF

      SolveFourPt = .TRUE.

      RETURN
      END



*== Public (StoreMassEq) ===============================================

      LOGICAL FUNCTION StoreMassEq(EqNum,Num,Coef)

      IMPLICIT NONE

*   Purpose:  Store a vector of solution-matrix coefficients for the
*             one-dimensional conservation-of-mass equation.

*   Arguments:
      INTEGER EqNum,Num
      REAL    Coef(Num)

*   Argument definitions:
*     EqNum - mass-equation sequence number, beginning in the network
*             adjacent to the lowest cross-section number and
*             progressing in the direction of increasing cross-section
*             numbers.
*              This number is local to the current channel and the sequence 
*              does not include boundary equations.
*     Num - number of coefficients in vector.
*     Coef - coefficient vector.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER I, J, K, N

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      K = EqPointer(Branch) + EqNum - 1
      J = MASSOF(K) - 1

      DO 100 I=1,Num
        J = J+1

*------ Get element number of floating-point array.
        N = SkyScript(MASSEQ(K), J)

*------ Store coefficient in global coefficient matrix.
        A(N) = A(N)+Coef(I)

 100  CONTINUE

      StoreMassEq = .TRUE.

      RETURN
      END



*== Public (StoreDynamicEq) ============================================

      LOGICAL FUNCTION StoreDynamicEq(EqNum, Num, Coef)

      IMPLICIT NONE

*   Purpose:  Store a vector of solution-matrix coefficients for the
*             one-dimensional dynamic equation.

*   Arguments:
      INTEGER EqNum,Num
      REAL    Coef(Num)

*   Argument definitions:
*     EqNum  - dynamic-equation sequence number, beginning in the network adjacent
*              to the lowest cross-section number and progressing in the
*              direction of increasing cross-section numbers.
*     Num    - number of coefficients in vector.
*     Coef   - coefficient vector.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline2.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER I, J, K, N

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      K = EqPointer(Branch) + EqNum - 1
      J = DYNMOF(K) - 1

      DO 100 I=1,Num
        J = J+1

*------ Get element number of floating-point array.
        N = SkyScript(DYNMEQ(K), J)

*------ Store coefficient in global coefficient matrix.
        A(N) = A(N)+Coef(I)

 100  CONTINUE

      StoreDynamicEq = .TRUE.

      RETURN
      END



*== Public (StoreAtRowOffset) ==========================================

      LOGICAL FUNCTION StoreAtRowOffset(Row, Offset, Value)

      IMPLICIT NONE

*   Purpose:  Store a value in the coeficient matrix at a location
*             corresponding to Row and Offset from the diagonal in
*             the virtual matrix.

*             Values are stored only if complete forward elimination
*             is to be performed on the coefficient matrix, indicated
*             by ForwardElim() = .TRUE..

*   Arguments:
      INTEGER Row, Offset
      REAL    Value

*   Argument definitions:
*     Row    - row number in the virtual matrix.
*     Offset - offset from the diagonal in the virtual matrix,
*              positive if column number is greater than that
*              of the diagonal.
*     Value  - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER N

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript
      LOGICAL  ForwardElim
      EXTERNAL ForwardElim

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:   Lew DeLong
*   Last modified: April 1992
*


*-- Implementation -----------------------------------------------------

      IF( ForwardElim() ) THEN
        N = SkyScript(Row,Offset)
        A(N) = Value
      END IF

      StoreAtRowOffset = .TRUE.

      RETURN
      END



*== Public (AddAtRowOffset) ============================================

      LOGICAL FUNCTION AddAtRowOffset(Row, Offset, Value)

      IMPLICIT NONE

*   Purpose:  Add a value to the coeficient matrix at a location
*             corresponding to Row and Offset from the diagonal in
*             the virtual matrix.

*   Arguments:
      INTEGER Row, Offset
      REAL    Value

*   Argument definitions:
*     Row    - row number in the virtual matrix.
*     Offset - offset from the diagonal in the virtual matrix,
*              positive if column number is greater than that
*              of the diagonal.
*     Value  - value to be stored.

*   Module data:
      INCLUDE 'network.inc'
      INCLUDE 'skyline3.inc'

*   Local Variables:
      INTEGER N

*   Routines by module:

***** Local:
      INTEGER  SkyScript
      EXTERNAL SkyScript

*   Programmed by: Lew DeLong
*   Date:          February 1991
*   Modified by:
*   Last modified:
*


*-- Implementation -----------------------------------------------------

      N = SkyScript(Row,Offset)
      A(N) = A(N) + Value

      AddAtRowOffset = .TRUE.

      RETURN
      END
