C=====INITABLE3 bof======================================================
C
C-----Purpose:
C       Initialize the 3 parameter tables by openning the file unit
C     Programmed by: JM Fulford
C     Date: 12-91
C     Modified by:   Lew DeLong (use of master file names added)
C     Last modified: May   1992
*
C
      LOGICAL FUNCTION INITABLE3()
C
      IMPLICIT NONE
C
C-----Module data:
      INCLUDE 'table3.inc'
      SAVE / TABLE3 /
C
C-----Local Variables:
      CHARACTER*12  TBNAME,TYPE
      CHARACTER*12  InternalFileName

C-----Routines by module:

C---- Master file:
      INTEGER GetFileUnit
      CHARACTER*12 GetFileName
      EXTERNAL GetFileUnit, GetFileName

C
C   Intrinsics:
C
      INITABLE3 = .TRUE.

C---- Get file name and unit from master.fil.
      InternalFileName = 'tables30.dat'
      TBNAME           = GetFileName( InternalFileName )
      TB3UNIT          = GetFileUnit( InternalFileName )

      OPEN (TB3UNIT,FILE=TBNAME,STATUS='OLD',ERR=999)
      READ(TB3UNIT,'(A12)') TYPE
      IF(TYPE.NE.TBNAME) GO TO 999
      GO TO 1000
 999  CONTINUE
      WRITE(*,*) ' Could not open 3-parameter-table file...',TBNAME
      INITABLE3 = .FALSE.
1000  RETURN
C
      END
C
C=====INITABLE3 eof========================================================
C
C=====GETAB3 bof===========================================================C
C
C-----Purpose:
C       point to table in the tables30.dat file that matches tableid and
C       read that table contents into common block TABLE3
C     Programmed by: JM Fulford
C     Date: 12-91
C     Modified by:
C     Last modified:
*
C
      LOGICAL FUNCTION GETAB3(TAB3ID)
C
      IMPLICIT NONE
C
C-----Arguments:
      INTEGER TAB3ID
C
C-----Argument definitions:
C     TAB3ID - integer identifier for table to get
C
C-----Module data:
      INCLUDE 'table3.inc'
      SAVE / TABLE3 /
C
C-----Local Variables:
      INTEGER I,IDENT
      REAL NWSU
      CHARACTER*3 LABEL
C
C-----Local definitions
C     TBUNIT - unit that table file is openned on
C     LABEL  - label that identifies beginning of table
C     IDENT  - integer identifier for table
C
      GETAB3 = .FALSE.
 10   CONTINUE
        READ(TB3UNIT,'(A3)',END=999) LABEL
        IF(LABEL.NE.'TAB') GO TO 10
        READ(TB3UNIT,'(I10,1X,I2,1X,I2)') IDENT,NWS,NQ
        IF(IDENT.NE.TAB3ID) GO TO 10
C
        READ(TB3UNIT,'(F9.3)')BASEL
        READ(TB3UNIT,*,ERR=999)(WSDSTB(I),I=1,NWS)
        READ(TB3UNIT,*,ERR=999)(QTB(I),I=1,NQ)
        NWSU=NWS*NQ
        READ(TB3UNIT,*,ERR=999)(WSUSTB(I),I=1,NWSU)
*       WRITE(*,*)IDENT,NWS,NQ
*       WRITE(*,*)'BASEL=',BASEL
*       WRITE(*,*) (WSDSTB(I),I=1,NWS)
*       WRITE(*,*) (QTB(I),I=1,NQ)
C
        GETAB3 = .TRUE.
C
 999  CONTINUE
      REWIND (TB3UNIT)
      READ(TB3UNIT,'(A3)') LABEL
C
      RETURN
      END
C
C=====GETAB3 eof===========================================================
C
C=====BRACKET eof==========================================================
C
C-----Purpose:
C       brackets the desired value in the table
C     Programmed by: JM Fulford
C     Date: 12-91
C     Modified by:
C     Last modified:
*
C
      INTEGER FUNCTION BRACKET (VALUE,N,ARRAY)
C
      IMPLICIT NONE
C
C-----Arguments:
      INTEGER N
      REAL VALUE,ARRAY(N)
C
C-----Argument definitions:
C     N      - size of array
C     VALUE  - value to bracket
C     ARRAY  - array to bracket value in
C
C-----Module data:  none
C
C-----Local Variables:
      INTEGER IFLG,I
C
C-----Routines by module:
C
C-----Local:
C     IFLG   - flags bracket location
C
C-----Intrinsics:
C
      IFLG=0
      I=0
      BRACKET = 0
 10   CONTINUE
        I=I+1
        IF(VALUE.LE.ARRAY(I)) IFLG=1
      IF(I.LT.N.AND.IFLG.EQ.0) GO TO 10
      IF(I.LE.1.AND.VALUE.LT.ARRAY(I)) THEN
        BRACKET =0
      ELSE IF(IFLG.EQ.0) THEN
        BRACKET = -1
      ELSE
        BRACKET = I
      ENDIF
C
      RETURN
      END
C
C=====BRACKET eof =========================================================
C
C=====TB3WSUS public =======================================================
C
C-----Purpose:
C       interpolates water surface elevation from tabulated
C       values in a 3 parameter table.  Interpolations for tables
C       not located in tables30.dat or outside of tabulated values
C       return -100,000. for tbswsus.
C     Programmed by: JM Fulford
C     Date: 12-91
C     Modified by:
C     Last modified:
*
C
      REAL FUNCTION TB3WSUS (STRUCTID,WSDS,Q)
C
      IMPLICIT NONE
C
C-----Arguments:
      INTEGER STRUCTID
      REAL WSDS,Q
C
C-----Argument definitions:
C    STRUCTID - 3 parameter table identifier
C    WSDS   - water surface downstream
C    Q      - discharge
C
C-----Module data:
      INCLUDE 'table3.inc'
C
C-----Local Variables:
C
C-----Routines by module:
      LOGICAL GETAB3
      INTEGER BRACKET
      REAL INTERP3
      EXTERNAL BRACKET, INTERP3, GETAB3
C
C-----Local:
      INTEGER IQ,IWS,ERR
      REAL WSDS0
C
C-----Intrinsics:
C
      ERR=0
      TB3WSUS=-100000.
      IF(GETAB3(STRUCTID)) THEN
        IQ=BRACKET(Q,NQ,QTB)
        WSDS0=WSDS-BASEL
        IWS=BRACKET(WSDS0,NWS,WSDSTB)
        IF(IQ.LE.0)THEN
          WRITE(*,*)'DISCHARGE OUT OF COMPUTED RANGE'
          ERR=1
        ENDIF
        IF(IWS.LE.0) THEN
          WRITE(*,*) 'DOWNSTREAM WATER SURFACE OUT OF COMPUTED RANGE'
          ERR=1
        ENDIF
        IF(ERR.NE.1) THEN
          TB3WSUS=INTERP3(WSDS0,IWS,Q,IQ)
          TB3WSUS=TB3WSUS+BASEL
        ENDIF
      ELSE
        WRITE(*,*)STRUCTID,' STRUCTURE NOT FOUND IN FILE'
      ENDIF
C
      RETURN
      END
C
C=====TB3WSUS eof =========================================================
C
C=====INTERP3 bof =========================================================
C
C-----Purpose:
C       Linearly interpolation of a point on a plane.  Uses four known
C       or tabulated points that describe a rectangle that bound the point.
C     Programmed by: JM Fulford
C     Date: 12-91
C     Modified by:
C     Last modified:
*
C
      REAL FUNCTION INTERP3(WSDS,IWS,Q,IQ)
C
      INTEGER IWS,IQ
      REAL WSDS,Q
C
      INCLUDE 'table3.inc'
C
      INTEGER I(4),IWEIGH
      REAL DELQ,DELWSDS,A,N(4),B
*------------------------------^
C
      INTERP3=0.0
      DELQ=QTB(IQ)-QTB(IQ-1)
      DELWSDS=WSDSTB(IWS)-WSDSTB(IWS-1)
      A=(WSDS-WSDSTB(IWS-1))/DELWSDS
      B=(Q-QTB(IQ-1))/DELQ
      N(1)=(1.0-A)*(1.0-B)
      N(2)=A*(1.0-B)
      N(3)=A*B
      N(4)=(1.0-A)*B
      I(4)=(IQ-1)*NWS+IWS-1
      I(3)=(IQ-1)*NWS+IWS
      I(2)=(IQ-2)*NWS+IWS
      I(1)=(IQ-2)*NWS+IWS-1
      DO 10 IWEIGH=1,4
        INTERP3=INTERP3+N(IWEIGH)*WSUSTB(I(IWEIGH))
  10  CONTINUE
C
      RETURN
      END
C
C=====INTERP3 eof ==========================================================
C
C=====TB3DWSUS bof =========================================================                                                                     
C
C-----Purpose:
C       interpolates upstream water surface elevation and derivatives
C       from tabulated values in a 3 parameter table.  Interpolations
C       for tables not found in tables30.dat or for points outside of
C       the tabulated values return -100000 for wsus, and 100000 for
C       dq and dwsds.  ERR returned zero is no error, 1 error.
C     Programmed by: JM Fulford
C     Date: 12-91
C     Modified by:
C     Last modified:
*
C
      SUBROUTINE TB3DWSUS
     I                   (STRUCTID,WSDS,Q,
     O                    WSUS,DWSDS,DQ,ERR)
C
      IMPLICIT NONE
C
C-----Arguments:
      INTEGER STRUCTID,ERR
      REAL WSDS,Q,WSUS,DWSDS,DQ
C
C-----Argument definitions:
C     STRUCTID - 3 parameter table identifier
C     WSDS   - water surface downstream
C     Q      - discharge
C     WSUS   - water surface upstream
C     DWSDS  - derivative of WSUS with respect to WSDS
C     DQ     - derivative of WSUS with respect to Q
C
C-----Module data:
      INCLUDE 'table3.inc'
C
C-----Local Variables:
C
C-----Routines by module:
      LOGICAL GETAB3
      INTEGER BRACKET
      REAL INTERP3
      EXTERNAL BRACKET, INTERP3, GETAB3, INTERPD3
C
C-----Local:
      INTEGER IQ,IWS
      REAL WSDS0
C
C-----Intrinsics:
C
      ERR=0
      WSUS=-100000.
      DWSDS=100000.
      DQ=100000.
      IF(GETAB3(STRUCTID)) THEN
        IQ=BRACKET(Q,NQ,QTB)
        WSDS0=WSDS-BASEL
        IWS=BRACKET(WSDS0,NWS,WSDSTB)
        IF(IQ.LE.0)THEN
          WRITE(*,*)'DISCHARGE OUT OF COMPUTED RANGE'
          ERR=1
        ENDIF
        IF(IWS.LE.0) THEN
          WRITE(*,*) 'DOWNSTREAM WATER SURFACE OUT OF COMPUTED RANGE'
          ERR=1
        ENDIF
        IF(ERR.NE.1) THEN
          WSUS=INTERP3(WSDS0,IWS,Q,IQ)
          CALL INTERPD3(WSDS0,IWS,Q,IQ,DWSDS,DQ)
          WSUS=WSUS+BASEL
        ENDIF
      ELSE
        WRITE(*,*)STRUCTID,' STRUCTURE NOT FOUND IN FILE'
      ENDIF
C
      RETURN
      END
C
C=====TB3DWSUS eof ========================================================
C
C=====INTERPD3 bof ========================================================
C
C-----Purpose:
C     Programmed by: JM Fulford
C     Date: 12-91
C     Modified by:
C     Last modified:
*
C
      SUBROUTINE INTERPD3
     I                   (WSDS,IWS,Q,IQ,
     O                    DWSDS,DQ)
C
      IMPLICIT NONE
C
C-----Arguments:
      INTEGER IWS,IQ
      REAL WSDS,Q,DWSDS,DQ
C
C-----Argument definitions:
C
C-----Module data:
      INCLUDE 'table3.inc'
C-----Local Variables:
C
C-----Routines by module:
C
C-----Local:
      INTEGER I(4)
      REAL DELQ,DELWSDS,A,B
C
C-----Intrinsics:
      DELQ=QTB(IQ)-QTB(IQ-1)
      DELWSDS=WSDSTB(IWS)-WSDSTB(IWS-1)
      A=(WSDS-WSDSTB(IWS-1))/DELWSDS
      B=(Q-QTB(IQ-1))/DELQ
      I(4)=(IQ-1)*NWS+IWS-1
      I(3)=(IQ-1)*NWS+IWS
      I(2)=(IQ-2)*NWS+IWS
      I(1)=(IQ-2)*NWS+IWS-1
      DWSDS=(1.0-B)*(WSUSTB(I(2))-WSUSTB(I(1)))
     #      +B*(WSUSTB(I(3))-WSUSTB(I(4)))
      DWSDS=DWSDS/DELWSDS
      DQ=(1.0-A)*(WSUSTB(I(4))-WSUSTB(I(1)))
     #      +A*(WSUSTB(I(3))-WSUSTB(I(2)))
      DQ=DQ/DELQ
C
      RETURN
      END
C
C=====INTERPD3 eof ========================================================
C
C=====TB3FLOW bof =========================================================
C
C-----Purpose:
C       interpolates discharge from a 3 parameter table. Interpolations
C       for tables not located in tables30.dat and outside of range of
C       tabulated values return tb3flow = -1.0
C     Programmed by: JM Fulford
C     Date: 12-91
C     Modified by:
C     Last modified:
*
C
      REAL FUNCTION TB3FLOW (STRUCTID,WSUS,WSDS)
C
      IMPLICIT NONE
C
C-----Arguments:
      INTEGER STRUCTID
      REAL WSDS,WSUS
C
C-----Argument definitions:
C    STRUCTID - 3 parameter table identifier
C    WSUS   - water surface upstream of structure
C    WSDS   - water surface downstream  structure
C
C-----Argument definitions:
C
C-----Module data:
      INCLUDE 'table3.inc'
C
C-----Local Variables:
      INTEGER KFLG,IQ,IWS,I1,I2,I3,I4
      REAL A,DELWSDS,DELQ,WSUS0,WSDS0,NTW,USWS
C
C-----Routines by module:
      LOGICAL GETAB3
      INTEGER BRACKET
      EXTERNAL GETAB3, BRACKET
C
C-----Local:
C
C-----Intrinsics:
      TB3FLOW=-1.0
      IF(GETAB3(STRUCTID)) THEN
        WSDS0=WSDS-BASEL
        IWS=BRACKET(WSDS0,NWS,WSDSTB)
        IF(IWS.LE.0)THEN
          WRITE(*,*)'DOWNSTREAM WATER SURFACE OUT OF TABLE RANGE'
        ELSE
          KFLG=0
          IQ=0
          WSUS0=WSUS-BASEL
          DELWSDS=WSDSTB(IWS)-WSDSTB(IWS-1)
          NTW=(WSDS0-WSDSTB(IWS-1))/DELWSDS
 10       CONTINUE
            IQ=IQ+1
            I3=IQ*NWS+IWS
            I4=I3-1
            USWS=WSUSTB(I4)+NTW*(WSUSTB(I3)-WSUSTB(I4))
            IF(USWS.GE.WSUS0) KFLG=1
          IF(IQ.LT.NQ.AND.KFLG.EQ.0) GO TO 10
          IF(IQ.GT.NQ.OR.KFLG.EQ.0.) THEN
            WRITE(*,*)'DISCHARGE OUTSIDE OF TABLE RANGE'
          ELSE
            DELQ=QTB(IQ)-QTB(IQ-1)
            A=(WSDS0-WSDSTB(IWS-1))/DELWSDS
            I2=(IQ-1)*NWS+IWS
            I1=(IQ-1)*NWS+IWS-1
            TB3FLOW=(1.0-A)*(WSUSTB(I4)-WSUSTB(I1)) +
     #              A*(WSUSTB(I3)-WSUSTB(I2))
            TB3FLOW=(WSUS0-(1.0-A)*WSUSTB(I1)-A*WSUSTB(I2))/TB3FLOW
            TB3FLOW=QTB(IQ)+DELQ*TB3FLOW
          ENDIF
        ENDIF
      ELSE
        WRITE(*,*) STRUCTID,' STRUCTURE NOT FOUND IN FILE'
      ENDIF
C
      RETURN
      END
C
C=====TB3FLOW eof==========================================================
