#!/bin/sh
# usr/opt/wrdapp/FourPt95.01/test/test.sh -- run FourPt test data sets
#
# Usage: test.sh [start [stop]]
#        test.sh [start [stop]] | tee test.out
#
#        where: start = starting test number
#                stop = ending test number (may be same as start)
#
# To use this script for another program in a different directory, globally
# change the program name (be sure to change both upper and lower case
# occurrences) and set the default value for the Stop variable as appropriate.
#
# History: 12/24/91, mblalock, initial coding
#          04/27/92, mygoze, restructuring
#          09/06/95, rsregan, modified for use with FourPt
#
# Variable definitions
# --------------------
#
  TOPDIR=..
  BIN=$TOPDIR/bin
  DATA=$TOPDIR/data
  CHECK=$TOPDIR/test/check.sh
  CLEAN=$TOPDIR/test/clean.sh
  PROG1=$BIN/fourpt.rec
  PROGNM=FourPt
  END=9
  DIVD=========================================
#
#
  exec 2>&1                                # stderr shows up in .out file

  Start=${1:-1}                            # by default, start at 1
  Stop=${2:-$END}                          # by default, stop at 4
  if [ $Start -lt 1 ] ; then Start=1 ; fi
  if [ $Stop -lt 1 ] ; then Stop=$END ; fi
  if [ $Start -gt $END -o $Stop -gt $END ] ; then
    echo
    echo "Warning, invalid arguments--test range is 1 - $END"
    echo "input arguments were $Start - $Stop"
    echo
    if [ $Stop -gt $END ] ; then Stop=$END ; fi
    if [ $Start -gt $END ] ; then
      echo "no tests will be performed"
    else
      echo "Tests $Start - $Stop will be performed"
    fi
    echo
  fi
#
# begin test runs
#
  $CLEAN
  Test=$Start

  if [ $Test -ge $Start -a $Test -le $Stop ] ; then
    echo
    echo $DIVD$DIVD
    echo "Begin processing $PROGNM test runs $Start to $Stop"
    echo
    date
  fi

  while [ $Test -ge $Start -a $Test -le $Stop ]
  do
    echo
    echo
    echo $DIVD$DIVD
    echo "Test run number $Test"
    Name=fourpt$Test
    rm -f master.fil
    cp $DATA/$Name.ctr .
    cp $DATA/$Name.fil master.fil
    cp $DATA/$Name.geo .
    cp $DATA/$Name.sch .
    if [ $Test -eq 3 -o $Test -eq 9 ] ; then cp $DATA/$Name.cns . ; fi
    $PROG1 > $Name.log
    Test=`expr $Test + 1`
  done
#
  if [ $Test -ge $Start -a $Test -le $Stop ] ; then
    echo
    echo
    echo $DIVD$DIVD
    echo "Completed $PROGNM test runs $Start to $Stop"
    echo
  fi
# check output against original output in $DATA directory
  $CHECK fourpt
